@@ -1,4 +1,4 @@
-# Copyright (C) 2015-2018 Regents of the University of California
+# Copyright (C) 2015-2021 Regents of the University of California
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -11,24 +11,21 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-from __future__ import absolute_import
-from future import standard_library
-standard_library.install_aliases()
-from future.utils import with_metaclass
-from builtins import object
 import enum
+import logging
 import os
 import shutil
-import logging
-from abc import ABCMeta, abstractmethod
+from abc import ABC, abstractmethod
 from collections import namedtuple
 from contextlib import contextmanager
+from typing import Any, Optional, Tuple
 
-from toil.lib.objects import abstractclassmethod
-from toil.batchSystems import registry
+from toil.batchSystems.registry import (BATCH_SYSTEM_FACTORY_REGISTRY,
+                                        DEFAULT_BATCH_SYSTEM)
 from toil.common import Toil, cacheDirName
-from toil.fileStores.abstractFileStore import AbstractFileStore
 from toil.deferred import DeferredFunctionManager
+from toil.fileStores.abstractFileStore import AbstractFileStore
+from toil.lib.threading import LastProcessStandingArena
 
 try:
     from toil.cwl.cwltoil import CWL_INTERNAL_JOBS
@@ -53,7 +50,7 @@ class BatchJobExitReason(enum.Enum):
     LOST = 3  # Preemptable failure (job's executing host went away).
     KILLED = 4  # Job killed before finishing.
     ERROR = 5  # Internal error.
-
+    MEMLIMIT = 6 # Job hit batch system imposed memory limit
 
 # Value to use as exitStatus in UpdatedBatchJobInfo.exitStatus when status is not available.
 EXIT_STATUS_UNAVAILABLE_VALUE = 255
@@ -68,14 +65,13 @@ WorkerCleanupInfo = namedtuple('WorkerCleanupInfo', (
     'cleanWorkDir'))
 
 
-class AbstractBatchSystem(with_metaclass(ABCMeta, object)):
+class AbstractBatchSystem(ABC):
     """
     An abstract (as far as Python currently allows) base class to represent the interface the batch
     system must provide to Toil.
     """
-
-    # noinspection PyMethodParameters
-    @abstractclassmethod
+    @classmethod
+    @abstractmethod
     def supportsAutoDeployment(cls):
         """
         Whether this batch system supports auto-deployment of the user script itself. If it does,
@@ -88,16 +84,18 @@ class AbstractBatchSystem(with_metaclass(ABCMeta, object)):
         """
         raise NotImplementedError()
 
-    # noinspection PyMethodParameters
-    @abstractclassmethod
+    @classmethod
+    @abstractmethod
     def supportsWorkerCleanup(cls):
         """
-        Indicates whether this batch system invokes :meth:`workerCleanup` after the last job for
-        a particular workflow invocation finishes. Note that the term *worker* refers to an
-        entire node, not just a worker process. A worker process may run more than one job
-        sequentially, and more than one concurrent worker process may exist on a worker node,
-        for the same workflow. The batch system is said to *shut down* after the last worker
-        process terminates.
+        Indicates whether this batch system invokes
+        :meth:`BatchSystemSupport.workerCleanup` after the last job for a
+        particular workflow invocation finishes. Note that the term *worker*
+        refers to an entire node, not just a worker process. A worker process
+        may run more than one job sequentially, and more than one concurrent
+        worker process may exist on a worker node, for the same workflow. The
+        batch system is said to *shut down* after the last worker process
+        terminates.
 
         :rtype: bool
         """
@@ -115,11 +113,11 @@ class AbstractBatchSystem(with_metaclass(ABCMeta, object)):
         raise NotImplementedError()
 
     @abstractmethod
-    def issueBatchJob(self, jobNode):
+    def issueBatchJob(self, jobDesc):
         """
         Issues a job with the specified command to the batch system and returns a unique jobID.
 
-        :param jobNode a toil.job.JobNode
+        :param jobDesc a toil.job.JobDescription
 
         :return: a unique jobID that can be used to reference the newly issued job
         :rtype: int
@@ -235,7 +233,20 @@ class AbstractBatchSystem(with_metaclass(ABCMeta, object)):
         :param setOption: A function with signature setOption(varName, parsingFn=None, checkFn=None, default=None)
            used to update run configuration
         """
-        pass
+        
+    def getWorkerContexts(self):
+        """
+        Get a list of picklable context manager objects to wrap worker work in,
+        in order.
+        
+        Can be used to ask the Toil worker to do things in-process (such as
+        configuring environment variables, hot-deploying user scripts, or
+        cleaning up a node) that would otherwise require a wrapping "executor"
+        process.
+        
+        :rtype: list
+        """
+        return []
 
 
 class BatchSystemSupport(AbstractBatchSystem):
@@ -273,7 +284,7 @@ class BatchSystemSupport(AbstractBatchSystem):
                                                    workflowID=self.config.workflowID,
                                                    cleanWorkDir=self.config.cleanWorkDir)
 
-    def checkResourceRequest(self, memory, cores, disk, name=None, detail=None):
+    def checkResourceRequest(self, memory: int, cores: float, disk: int, job_name: str = '', detail: str = ''):
         """
         Check resource request is not greater than that available or allowed.
 
@@ -283,25 +294,35 @@ class BatchSystemSupport(AbstractBatchSystem):
 
         :param int disk: amount of disk space being requested, in bytes
         
-        :param str name: Name of the job being checked, for generating a useful error report.
+        :param str job_name: Name of the job being checked, for generating a useful error report.
         
         :param str detail: Batch-system-specific message to include in the error.
 
         :raise InsufficientSystemResources: raised when a resource is requested in an amount
                greater than allowed
         """
-        assert memory is not None
-        assert disk is not None
-        assert cores is not None
-        if cores > self.maxCores:
-            raise InsufficientSystemResources('cores', cores, self.maxCores,
-                                              batchSystem=self.__class__.__name__, name=name, detail=detail)
-        if memory > self.maxMemory:
-            raise InsufficientSystemResources('memory', memory, self.maxMemory,
-                                              batchSystem=self.__class__.__name__, name=name, detail=detail)
-        if disk > self.maxDisk:
-            raise InsufficientSystemResources('disk', disk, self.maxDisk,
-                                              batchSystem=self.__class__.__name__, name=name, detail=detail)
+        batch_system = self.__class__.__name__ or 'this batch system'
+        for resource, requested, available in [('cores', cores, self.maxCores),
+                                               ('memory', memory, self.maxMemory),
+                                               ('disk', disk, self.maxDisk)]:
+            assert requested is not None
+            if requested > available:
+                unit = 'bytes of ' if resource in ('disk', 'memory') else ''
+                R = f'The job {job_name} is r' if job_name else 'R'
+                if resource == 'disk':
+                    msg = (f'{R}equesting {requested} {unit}{resource} for temporary space, '
+                           f'more than the maximum of {available} {unit}{resource} of free space on '
+                           f'{self.config.workDir} that {batch_system} was configured with, or enforced '
+                           f'by --max{resource.capitalize()}.  Try setting/changing the toil option '
+                           f'"--workDir" or changing the base temporary directory by setting TMPDIR.')
+                else:
+                    msg = (f'{R}equesting {requested} {unit}{resource}, more than the maximum of '
+                           f'{available} {unit}{resource} that {batch_system} was configured with, '
+                           f'or enforced by --max{resource.capitalize()}.')
+                if detail:
+                    msg += detail
+
+                raise InsufficientSystemResources(msg)
 
     def setEnv(self, name, value=None):
         """
@@ -384,20 +405,19 @@ class BatchSystemLocalSupport(BatchSystemSupport):
 
     def __init__(self, config, maxCores, maxMemory, maxDisk):
         super(BatchSystemLocalSupport, self).__init__(config, maxCores, maxMemory, maxDisk)
-        self.localBatch = registry.batchSystemFactoryFor(
-            registry.defaultBatchSystem())()(
+        self.localBatch = BATCH_SYSTEM_FACTORY_REGISTRY[DEFAULT_BATCH_SYSTEM]()(
                 config, config.maxLocalJobs, maxMemory, maxDisk)
 
-    def handleLocalJob(self, jobNode):  # type: (JobNode) -> Optional[int]
+    def handleLocalJob(self, jobDesc):  # type: (Any) -> Optional[int]
         """
         To be called by issueBatchJobs.
 
-        Returns the jobID if the jobNode has been submitted to the local queue,
+        Returns the jobID if the jobDesc has been submitted to the local queue,
         otherwise returns None
         """
         if (not self.config.runCwlInternalJobsOnWorkers
-                and jobNode.jobName.startswith(CWL_INTERNAL_JOBS)):
-            return self.localBatch.issueBatchJob(jobNode)
+                and jobDesc.jobName.startswith(CWL_INTERNAL_JOBS)):
+            return self.localBatch.issueBatchJob(jobDesc)
         else:
             return None
 
@@ -435,6 +455,64 @@ class BatchSystemLocalSupport(BatchSystemSupport):
         """To be called from shutdown()"""
         self.localBatch.shutdown()
 
+class BatchSystemCleanupSupport(BatchSystemLocalSupport):
+    """
+    Adds cleanup support when the last running job leaves a node, for batch
+    systems that can't provide it using the backing scheduler.
+    """
+    
+    @classmethod
+    def supportsWorkerCleanup(cls):
+        return True
+        
+    def getWorkerContexts(self):
+        # Tell worker to register for and invoke cleanup
+        
+        # Create a context manager that has a copy of our cleanup info
+        context = WorkerCleanupContext(self.workerCleanupInfo)
+       
+        # Send it along so the worker works inside of it
+        contexts = super(BatchSystemCleanupSupport, self).getWorkerContexts()
+        contexts.append(context)
+        return contexts
+        
+    def __init__(self, config, maxCores, maxMemory, maxDisk):
+        super(BatchSystemCleanupSupport, self).__init__(config, maxCores, maxMemory, maxDisk)
+        
+class WorkerCleanupContext:
+    """
+    Context manager used by :class:`BatchSystemCleanupSupport` to implement
+    cleanup on a node after the last worker is done working.
+    
+    Gets wrapped around the worker's work.
+    """
+    
+    def __init__(self, workerCleanupInfo):
+        """
+        Wrap the given workerCleanupInfo in a context manager.
+        
+        :param WorkerCleanupInfo workerCleanupInfo: Info to use to clean up the worker if we are
+                                                    the last to exit the context manager.
+        """
+        
+        
+        self.workerCleanupInfo = workerCleanupInfo
+        self.arena = None
+        
+    def __enter__(self):
+        # Set up an arena so we know who is the last worker to leave
+        self.arena = LastProcessStandingArena(Toil.getToilWorkDir(self.workerCleanupInfo.workDir), 
+                                              self.workerCleanupInfo.workflowID + '-cleanup')
+        self.arena.enter()
+        
+    def __exit__(self, type, value, traceback):
+        for _ in self.arena.leave():
+            # We are the last concurrent worker to finish.
+            # Do batch system cleanup.
+            logger.debug('Cleaning up worker')
+            BatchSystemSupport.workerCleanup(self.workerCleanupInfo)
+        # We have nothing to say about exceptions
+        return False
 
 class NodeInfo(object):
     """
@@ -542,52 +620,4 @@ class AbstractScalableBatchSystem(AbstractBatchSystem):
 
 
 class InsufficientSystemResources(Exception):
-    """
-    To be raised when a job requests more of a particular resource than is either currently allowed
-    or avaliable
-    """
-    def __init__(self, resource, requested, available, batchSystem=None, name=None, detail=None):
-        """
-        Creates an instance of this exception that indicates which resource is insufficient for current
-        demands, as well as the amount requested and amount actually available.
-
-        :param str resource: string representing the resource type
-
-        :param int|float requested: the amount of the particular resource requested that resulted
-               in this exception
-
-        :param int|float available: amount of the particular resource actually available
-        
-        :param str batchSystem: Name of the batch system class complaining, for
-                   generating a useful error report. If you are using a single machine
-                   batch system for local jobs in another batch system, it is important to
-                   know which one has run out of resources.
-        
-        :param str name: Name of the job being checked, for generating a useful error report.
-        
-        :param str detail: Batch-system-specific message to include in the error.
-        """
-        self.requested = requested
-        self.available = available
-        self.resource = resource
-        self.batchSystem = batchSystem if batchSystem is not None else 'this batch system'
-        self.unit = 'bytes of ' if resource == 'disk' or resource == 'memory' else ''
-        self.name = name
-        self.detail = detail
-
-    def __str__(self):
-        if self.name is not None:
-            phrases = [('The job {} is requesting {} {}{}, more than '
-                        'the maximum of {} {}{} that {} was configured '
-                        'with.'.format(self.name, self.requested, self.unit, self.resource,
-                                       self.available, self.unit, self.resource, self.batchSystem))]
-        else:
-            phrases = [('Requesting more {} than either physically available to {}, or enforced by --max{}. '
-                        'Requested: {}, Available: {}'.format(self.resource, self.batchSystem,
-                                                              self.resource.capitalize(),
-                                                              self.requested, self.available))]
-        
-        if self.detail is not None:
-            phrases.append(self.detail)
-            
-        return ' '.join(phrases)
+    pass
