@@ -1,4 +1,4 @@
-# Copyright (C) 2019 Regents of the University of California
+# Copyright (C) 2015-2021 Regents of the University of California
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -11,7 +11,6 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-
 """
 Batch system for running Toil workflows on Kubernetes.
 
@@ -21,77 +20,59 @@ Within non-priveleged Kubernetes containers, additional Docker containers
 cannot yet be launched. That functionality will need to wait for user-mode
 Docker
 """
-
-from __future__ import absolute_import
-from future import standard_library
-standard_library.install_aliases()
-from builtins import str
-
 import base64
 import datetime
 import getpass
-import kubernetes
 import logging
 import os
 import pickle
-import pytz
 import string
 import subprocess
 import sys
 import tempfile
 import time
 import uuid
+
+import kubernetes
+import pytz
 import urllib3
-
 from kubernetes.client.rest import ApiException
-from six.moves.queue import Empty, Queue
 
-from toil import applianceSelf, customDockerInitCmd
-from toil.batchSystems.abstractBatchSystem import (AbstractBatchSystem,
-                                                   BatchSystemSupport,
-                                                   BatchSystemLocalSupport,
-                                                   EXIT_STATUS_UNAVAILABLE_VALUE,
+from toil import applianceSelf
+from toil.batchSystems.abstractBatchSystem import (EXIT_STATUS_UNAVAILABLE_VALUE,
+                                                   BatchJobExitReason,
+                                                   BatchSystemCleanupSupport,
                                                    UpdatedBatchJobInfo)
 from toil.common import Toil
-from toil.lib.bioio import configureRootLogger
-from toil.lib.bioio import setLogLevel
 from toil.lib.humanize import human2bytes
-from toil.lib.threading import LastProcessStandingArena
+from toil.lib.retry import ErrorCondition, retry
 from toil.resource import Resource
-
-from toil.lib.retry import retry
+from toil.statsAndLogging import configure_root_logger, set_log_level
 
 logger = logging.getLogger(__name__)
-     
-def retryable_kubernetes_errors(e):
+retryable_kubernetes_errors = [urllib3.exceptions.MaxRetryError,
+                               urllib3.exceptions.ProtocolError,
+                               ApiException]
+
+
+def is_retryable_kubernetes_error(e):
     """
-    A function that determins whether or not Toil should retry or stop given 
-    exceptions thrown by Kubernetes. 
+    A function that determines whether or not Toil should retry or stop given
+    exceptions thrown by Kubernetes.
     """
-    if isinstance(e, urllib3.exceptions.MaxRetryError) or \
-       isinstance(e, urllib3.exceptions.ProtocolError) or \
-        isinstance(e, ApiException):
-        return True
+    for error in retryable_kubernetes_errors:
+        if isinstance(e, error):
+            return True
     return False
-    
-def retryable_kubernetes_errors_expecting_gone(e):
-    """
-    A function that determins whether or not Toil should retry or stop given 
-    exceptions thrown by Kubernetes, when Toil is expecting a 404 indicating
-    that a resource is gone, as it should be.
-    
-    Retries on errors other than 404.
-    """
-    
-    return retryable_kubernetes_errors(e) and not (isinstance(e, ApiException) and e.status == 404)
-    
+
+
 def slow_down(seconds):
     """
     Toil jobs that have completed are not allowed to have taken 0 seconds, but
-    Kubernetes timestamps things to the second. It is possible in Kubernetes for
+    Kubernetes timestamps round things to the nearest second. It is possible in Kubernetes for
     a pod to have identical start and end timestamps.
 
-    This function takes a possibly 0 job length in seconds an enforces a minimum length to satisfy Toil.
+    This function takes a possibly 0 job length in seconds and enforces a minimum length to satisfy Toil.
 
     :param float seconds: Kubernetes timestamp difference
 
@@ -101,24 +82,17 @@ def slow_down(seconds):
 
     return max(seconds, sys.float_info.epsilon)
 
-def utc_now():
-    """
-    Return a datetime in the UTC timezone corresponding to right now.
-    """
 
+def utc_now():
+    """Return a datetime in the UTC timezone corresponding to right now."""
     return datetime.datetime.utcnow().replace(tzinfo=pytz.UTC)
 
 
-class KubernetesBatchSystem(BatchSystemLocalSupport):
-
+class KubernetesBatchSystem(BatchSystemCleanupSupport):
     @classmethod
     def supportsAutoDeployment(cls):
         return True
 
-    @classmethod
-    def supportsWorkerCleanup(cls):
-        return True
-   
     def __init__(self, config, maxCores, maxMemory, maxDisk):
         super(KubernetesBatchSystem, self).__init__(config, maxCores, maxMemory, maxDisk)
 
@@ -154,8 +128,10 @@ class KubernetesBatchSystem(BatchSystemLocalSupport):
         else:    
             username = ''.join([c for c in getpass.getuser().lower() if c in acceptableChars])[:100]
         
+        self.uniqueID = uuid.uuid4()
+
         # Create a prefix for jobs, starting with our username
-        self.jobPrefix = '{}-toil-{}-'.format(username, uuid.uuid4())
+        self.jobPrefix = '{}-toil-{}-'.format(username, self.uniqueID)
         
         # Instead of letting Kubernetes assign unique job names, we assign our
         # own based on a numerical job ID. This functionality is managed by the
@@ -184,7 +160,10 @@ class KubernetesBatchSystem(BatchSystemLocalSupport):
         self.awsSecretName = os.environ.get("TOIL_AWS_SECRET_NAME", None)
 
         # Set this to True to enable the experimental wait-for-job-update code
-        self.enableWatching = True
+        # TODO: Make this an environment variable?
+        self.enableWatching = os.environ.get("KUBE_WATCH_ENABLED", False)
+
+        self.runID = 'toil-{}'.format(self.uniqueID)
 
         self.jobIds = set()
     
@@ -261,7 +240,8 @@ class KubernetesBatchSystem(BatchSystemLocalSupport):
                 return self._apis[kind]
             except KeyError: 
                 raise RuntimeError("Unknown Kubernetes API type: {}".format(kind))
-    
+
+    @retry(errors=retryable_kubernetes_errors)
     def _try_kubernetes(self, method, *args, **kwargs):
         """
         Kubernetes API can end abruptly and fail when it could dynamically backoff and retry.
@@ -272,21 +252,21 @@ class KubernetesBatchSystem(BatchSystemLocalSupport):
         
         This function gives Kubernetes more time to try an executable api.  
         """
+        return method(*args, **kwargs)
 
-        for attempt in retry(predicate=retryable_kubernetes_errors):
-            with attempt:
-                return method(*args, **kwargs)
-                
+    @retry(errors=retryable_kubernetes_errors + [
+               ErrorCondition(
+                   error=ApiException,
+                   error_codes=[404],
+                   retry_on_this_condition=False
+               )])
     def _try_kubernetes_expecting_gone(self, method, *args, **kwargs):
         """
         Same as _try_kubernetes, but raises 404 errors as soon as they are
         encountered (because we are waiting for them) instead of retrying on
         them.
         """
-        
-        for attempt in retry(predicate=retryable_kubernetes_errors_expecting_gone):
-            with attempt:
-                return method(*args, **kwargs)
+        return method(*args, **kwargs)
                 
     def _try_kubernetes_stream(self, method, *args, **kwargs):
         """
@@ -324,7 +304,7 @@ class KubernetesBatchSystem(BatchSystemLocalSupport):
                 raise
             else:
                 # It was from the Kubernetes watch generator we manage.
-                if retryable_kubernetes_errors(e):
+                if is_retryable_kubernetes_error(e):
                     # This is just cloud weather.
                     # TODO: We will also get an APIError if we just can't code good against Kubernetes. So make sure to warn.
                     logger.warning("Received error from Kubernetes watch stream: %s", e)
@@ -341,12 +321,12 @@ class KubernetesBatchSystem(BatchSystemLocalSupport):
         
     # setEnv is provided by BatchSystemSupport, updates self.environment
     
-    def issueBatchJob(self, jobNode):
+    def issueBatchJob(self, jobDesc):
         # TODO: get a sensible self.maxCores, etc. so we can checkResourceRequest.
         # How do we know if the cluster will autoscale?
         
         # Try the job as local
-        localID = self.handleLocalJob(jobNode)
+        localID = self.handleLocalJob(jobDesc)
         if localID:
             # It is a local job
             return localID
@@ -354,7 +334,7 @@ class KubernetesBatchSystem(BatchSystemLocalSupport):
             # We actually want to send to the cluster
             
             # Check resource requirements (managed by BatchSystemSupport)
-            self.checkResourceRequest(jobNode.memory, jobNode.cores, jobNode.disk)
+            self.checkResourceRequest(jobDesc.memory, jobDesc.cores, jobDesc.disk)
             
             # Make a batch system scope job ID
             jobID = self.getNextJobID()
@@ -363,14 +343,10 @@ class KubernetesBatchSystem(BatchSystemLocalSupport):
 
             # Make a job dict to send to the executor.
             # First just wrap the command and the environment to run it in
-            job = {'command': jobNode.command,
+            job = {'command': jobDesc.command,
                    'environment': self.environment.copy()}
             # TODO: query customDockerInitCmd to respect TOIL_CUSTOM_DOCKER_INIT_COMMAND
             
-            # Send the worker cleanup info so that the last worker on a
-            # host to shut down can clean up if warranted.
-            job['workerCleanupInfo'] = self.workerCleanupInfo
-
             if self.userScript is not None:
                 # If there's a user script resource be sure to send it along
                 job['userScript'] = self.userScript
@@ -394,10 +370,10 @@ class KubernetesBatchSystem(BatchSystemLocalSupport):
             # Kubernetes needs some lower limit of memory to run the pod at all without
             # OOMing. We also want to provision some extra space so that when
             # we test _isPodStuckOOM we never get True unless the job has
-            # exceeded jobNode.memory.
-            requirements_dict = {'cpu': jobNode.cores,
-                                 'memory': jobNode.memory + 1024 * 1024 * 512,
-                                 'ephemeral-storage': jobNode.disk + 1024 * 1024 * 512}
+            # exceeded jobDesc.memory.
+            requirements_dict = {'cpu': jobDesc.cores,
+                                 'memory': jobDesc.memory + 1024 * 1024 * 512,
+                                 'ephemeral-storage': jobDesc.disk + 1024 * 1024 * 512}
             # Use the requirements as the limits, for predictable behavior, and because
             # the UCSC Kubernetes admins want it that way.
             limits_dict = requirements_dict
@@ -449,13 +425,16 @@ class KubernetesBatchSystem(BatchSystemLocalSupport):
             pod_spec = kubernetes.client.V1PodSpec(containers=[container],
                                                    volumes=volumes,
                                                    restart_policy="Never")
+            # Make metadata to label the job/pod with info.
+            metadata = kubernetes.client.V1ObjectMeta(name=jobName,
+                                                    labels={"toil_run": self.runID})
+            
             # Wrap the spec in a template
-            template = kubernetes.client.V1PodTemplateSpec(spec=pod_spec)
+            template = kubernetes.client.V1PodTemplateSpec(spec=pod_spec, metadata=metadata)
+            
             # Make another spec for the job, asking to run the template with no backoff
             job_spec = kubernetes.client.V1JobSpec(template=template, backoff_limit=0)
-            # Make metadata to tag the job with info.
-            # We use generate_name to ensure a unique name
-            metadata = kubernetes.client.V1ObjectMeta(name=jobName)
+            
             # And make the actual job
             job = kubernetes.client.V1Job(spec=job_spec,
                                           metadata=metadata,
@@ -468,25 +447,10 @@ class KubernetesBatchSystem(BatchSystemLocalSupport):
             logger.debug('Launched job: %s', jobName)
             
             return jobID
-            
-            
-    def _isJobOurs(self, jobObject):
-        """
-        Determine if a Kubernetes job belongs to us.
-        
-        :param kubernetes.client.V1Job jobObject: a Kubernetes job being considered.
-
-        :return: True if the job is our responsibility, and false otherwise.
-        :rtype: bool
-        """
-        
-        return jobObject.metadata.name.startswith(self.jobPrefix)
-        
-        
     
-    def _ourJobObjects(self, onlySucceeded=False, limit=None):
+    def _ourJobObject(self, onlySucceeded=False):
         """
-        Yield all Kubernetes V1Job objects that we are responsible for that the
+        Yield Kubernetes V1Job objects that we are responsible for that the
         cluster knows about.
 
         Doesn't support a free-form selector, because there's only about 3
@@ -505,47 +469,65 @@ class KubernetesBatchSystem(BatchSystemLocalSupport):
         # https://github.com/kubernetes-client/python/issues/953. For now we
         # will just throw an error if we don't get to the end of the list in
         # time.
+
         token = None
-        
-        # Do our own limiting since we need to apply a filter that the server
-        # can't.
-        seen = 0
-        
-        # TODO: We ought to label our jobs by owning Toil workflow so we can
-        # look them up instead of filtering down later.
-        
+
         while True:
             # We can't just pass e.g. a None continue token when there isn't
             # one, because the Kubernetes module reads its kwargs dict and
             # cares about presence/absence. So we build a dict to send.
             kwargs = {}
-            if onlySucceeded:
-                # Check only successful jobs.
-                # Note that for selectors it is "successful" while for the
-                # actual object field it is "succeeded".
-                kwargs['field_selector'] = 'status.successful==1'
+            
             if token is not None:
                 kwargs['_continue'] = token
-            
-            results = self._try_kubernetes(self._api('batch').list_namespaced_job, self.namespace, **kwargs)
-            
+
+            if onlySucceeded: 
+                results =  self._try_kubernetes(self._api('batch').list_namespaced_job, self.namespace, 
+                                                label_selector="toil_run={}".format(self.runID), field_selector="status.successful==1", **kwargs)
+            else:
+                results = self._try_kubernetes(self._api('batch').list_namespaced_job, self.namespace, 
+                                                label_selector="toil_run={}".format(self.runID), **kwargs)
             for job in results.items:
-                if self._isJobOurs(job):
-                    # This job belongs to us
-                    yield job
-                    
-                    # Don't go over the limit
-                    seen += 1
-                    if limit is not None and seen >= limit:
-                        return
-                    
+                # This job belongs to us
+                yield job
+
             # Remember the continuation token, if any
             token = getattr(results.metadata, 'continue', None)
 
             if token is None:
                 # There isn't one. We got everything.
                 break
-                
+       
+    
+    def _ourPodObject(self):
+        """
+        Yield Kubernetes V1Pod objects that we are responsible for that the
+        cluster knows about.                                        
+        """
+
+        token = None
+
+        while True:
+            # We can't just pass e.g. a None continue token when there isn't
+            # one, because the Kubernetes module reads its kwargs dict and
+            # cares about presence/absence. So we build a dict to send.
+            kwargs = {}
+            
+            if token is not None:
+                kwargs['_continue'] = token
+
+            results = self._try_kubernetes(self._api('core').list_namespaced_pod, self.namespace, label_selector="toil_run={}".format(self.runID), **kwargs)
+            
+            for pod in results.items:
+                yield pod
+            # Remember the continuation token, if any
+            token = getattr(results.metadata, 'continue', None)
+
+            if token is None:
+                # There isn't one. We got everything.
+                break
+
+
     def _getPodForJob(self, jobObject):
         """
         Get the pod that belongs to the given job, or None if the job's pod is
@@ -693,44 +675,57 @@ class KubernetesBatchSystem(BatchSystemLocalSupport):
             return result
 
         # Otherwise we need to maybe wait.
-
         if self.enableWatching:
-            # Try watching for something to happen and use that.
+            for event in self._try_kubernetes_stream(self._api('batch').list_namespaced_job, self.namespace, 
+                                                        label_selector="toil_run={}".format(self.runID),
+                                                        timeout_seconds=maxWait):
+                # Grab the metadata data, ID, the list of conditions of the current job, and the total pods
+                jobObject = event['object']
+                jobID = int(jobObject.metadata.name[len(self.jobPrefix):])
+                jobObjectListConditions =jobObject.status.conditions
+                totalPods = jobObject.status.active + jobObject.status.finished + jobObject.status.failed
+                # Exit Reason defaults to 'Successfully Finsihed` unless said otherwise 
+                exitReason = BatchJobExitReason.FINISHED
+                exitCode = 0
 
-            if self.enableWatching:
-                for j in self._ourJobObjects():
-                    for event in self._try_kubernetes_stream(self._api('core').list_namespaced_pod, self.namespace, timeout_seconds=maxWait):
-                        # For each event from the stream until it times out or just disconnects
-                        pod = event['object']
-                        if pod.metadata.name.startswith(self.jobPrefix):
-                            if pod.status.phase == 'Failed' or pod.status.phase == 'Succeeded':
-                                containerStatuses =  pod.status.container_statuses
-                                logger.debug("FINISHED")
-                                if containerStatuses is None or len(containerStatuses) == 0: 
-                                    logger.debug("No job container statuses for job %s" % (pod.metadata.owner_references[0].name))
-                                    return UpdatedBatchJobInfo(jobID=int(pod.metadata.owner_references[0].name[len(self.jobPrefix):]), exitStatus=EXIT_STATUS_UNAVAILABLE_VALUE, wallTime=0, exitReason=None)
+                # Check if there are any active pods
+                if jobObject.status.acitve > 0:
+                    logger.info("%s has %d pods running" % jobObject.metadata.name, jobObject.status.active)
+                    continue
+                elif jobObject.status.failed > 0 or jobObject.status.finished > 0:
+                    # No more active pods in the current job ; must be finished
+                    logger.info("%s RESULTS -> Succeeded: %d Failed:%d Active:%d" % jobObject.metadata.name, 
+                                                                jobObject.status.succeeded, jobObject.status.failed, jobObject.status.active)
+                    # Get termination information of job
+                    termination = jobObjectListConditions[0]
+                    # Log out succeess/failure given a reason
+                    logger.info("%s REASON: %s", termination.type, termination.reason)
 
-                                # Get termination onformation from the pod
-                                termination = pod.status.container_statuses[0].state.terminated
-                                logger.info("REASON: %s Exit Code: %s", termination.reason, termination.exit_code)
-                                
-                                if termination.exit_code != 0:
-                                    # The pod failed. Dump information about it.
-                                    logger.debug('Failed pod information: %s', str(pod))
-                                    logger.warning('Log from failed pod: %s', self._getLogForPod(pod))
-                                jobID = int(pod.metadata.owner_references[0].name[len(self.jobPrefix):])
-                                terminated = pod.status.container_statuses[0].state.terminated
-                                runtime = slow_down((terminated.finished_at - terminated.started_at).total_seconds())
-                                result = UpdatedBatchJobInfo(jobID=jobID, exitStatus=terminated.exit_code, wallTime=runtime, exitReason=None)
-                                self._try_kubernetes(self._api('batch').delete_namespaced_job, 
-                                            pod.metadata.owner_references[0].name,
+                    # Log out reason of failure and pod exit code 
+                    if jobObject.status.failed > 0:
+                        exitReason = BatchJobExitReason.FAILED
+                        pod = self._getPodForJob(jobObject)
+                        logger.debug("Failed job %s", str(jobObject))
+                        logger.warning("Failed Job Message: %s", termination.message)
+                        exitCode = pod.status.container_statuses[0].state.terminated.exit_code
+                    
+                    runtime = slow_down((termination.completion_time - termination.start_time).total_seconds())
+                    result = UpdatedBatchJobInfo(jobID=jobID, exitStatus=exitCode, wallTime=runtime, exitReason=exitReason)
+
+                    if (exitReason == BatchJobExitReason.FAILED) or (jobObject.status.finished == totalPods):
+                        # Cleanup if job is all finished or there was a pod that failed
+                        self._try_kubernetes(self._api('batch').delete_namespaced_job, 
+                                            jobObject.metadata.name,
                                             self.namespace,
                                             propagation_policy='Foreground')
-
-                                self._waitForJobDeath(pod.metadata.owner_references[0].name)
-                                return result
-                            else:
-                                continue
+                        self._waitForJobDeath(jobObject.metadata.name)
+                        return result
+                    continue
+                else:
+                    # Job is not running/updating ; no active, successful, or failed pods yet
+                    logger.debug("Job %s -> %s" % (jobObject.metadata.name, jobObjectListConditions[0].reason))
+                    # Pod could be pending; don't say it's lost.
+                    continue
         else:
             # Try polling instead
             while result is None and (datetime.datetime.now() - entry).total_seconds() < maxWait:
@@ -771,13 +766,14 @@ class KubernetesBatchSystem(BatchSystemLocalSupport):
         jobObject = None
         # Put 'done', 'failed', or 'stuck' here
         chosenFor = ''
-        for j in self._ourJobObjects(onlySucceeded=True, limit=1):
+
+        for j in self._ourJobObject(onlySucceeded=True):
             # Look for succeeded jobs because that's the only filter Kubernetes has
             jobObject = j
             chosenFor = 'done'
 
         if jobObject is None:
-            for j in self._ourJobObjects():
+            for j in self._ourJobObject():
                 # If there aren't any succeeded jobs, scan all jobs
                 # See how many times each failed
                 failCount = getattr(j.status, 'failed', 0)
@@ -792,7 +788,7 @@ class KubernetesBatchSystem(BatchSystemLocalSupport):
 
         if jobObject is None:
             # If no jobs are failed, look for jobs with pods that are stuck for various reasons.
-            for j in self._ourJobObjects():
+            for j in self._ourJobObject():
                 pod = self._getPodForJob(j)
 
                 if pod is None:
@@ -976,31 +972,40 @@ class KubernetesBatchSystem(BatchSystemLocalSupport):
         
         # Shutdown local processes first
         self.shutdownLocal()
+       
+    
+        # Kill all of our jobs and clean up pods that are associated with those jobs
+        try:
+            self._try_kubernetes_expecting_gone(self._api('batch').delete_collection_namespaced_job, 
+                                                            self.namespace, 
+                                                            label_selector="toil_run={}".format(self.runID))
+            logger.debug('Killed jobs with delete_collection_namespaced_job; cleaned up')
+        except ApiException as e:
+            if e.status != 404:
+                # Anything other than a 404 is weird here.
+                logger.error("Exception when calling BatchV1Api->delete_collection_namespaced_job: %s" % e)
         
-        # Clears jobs belonging to this run
-        for job in self._ourJobObjects():
-            jobName = job.metadata.name
+            # aggregate all pods and check if any pod has failed to cleanup or is orphaned. 
+            ourPods = self._ourPodObject()
 
-            try:
-                # Look at the pods and log why they failed, if they failed, for debugging.
-                pod = self._getPodForJob(job)
-                if pod.status.phase == 'Failed':
-                    logger.debug('Failed pod encountered at shutdown: %s', str(pod))
-            except:
-                # Don't get mad if that doesn't work.
-                pass
-
-            # Kill jobs whether they succeeded or failed
-            try:
-                # Delete with background poilicy so we can quickly issue lots of commands
-                response = self._try_kubernetes_expecting_gone(self._api('batch').delete_namespaced_job, jobName, 
-                                                               self.namespace, 
-                                                               propagation_policy='Background')
-                logger.debug('Killed job for shutdown: %s', jobName)
-            except ApiException as e:
-                if e.status != 404:
-                    # Anything other than a 404 is weird here.
-                    logger.error("Exception when calling BatchV1Api->delte_namespaced_job: %s" % e)
+            for pod in ourPods:
+                try:
+                    if pod.status.phase == 'Failed':
+                            logger.debug('Failed pod encountered at shutdown: %s', str(pod))
+                    if pod.status.phase == 'Orphaned':
+                            logger.debug('Orphaned pod encountered at shutdown: %s', str(pod))
+                except:
+                    # Don't get mad if that doesn't work.
+                    pass
+                try:
+                    logger.debug('Cleaning up pod at shutdown: %s', str(pod))
+                    respone = self._try_kubernetes_expecting_gone(self._api('core').delete_namespaced_pod,  pod.metadata.name,
+                                        self.namespace, 
+                                        propagation_policy='Background')
+                except ApiException as e:
+                    if e.status != 404:
+                        # Anything other than a 404 is weird here.
+                        logger.error("Exception when calling CoreV1Api->delete_namespaced_pod: %s" % e)
 
 
     def _getIssuedNonLocalBatchJobIDs(self):
@@ -1008,7 +1013,7 @@ class KubernetesBatchSystem(BatchSystemLocalSupport):
         Get the issued batch job IDs that are not for local jobs.
         """
         jobIDs = []
-        got_list = self._ourJobObjects()
+        got_list = self._ourJobObject()
         for job in got_list:
             # Get the ID for each job
             jobIDs.append(self._getIDForOurJob(job))
@@ -1021,7 +1026,7 @@ class KubernetesBatchSystem(BatchSystemLocalSupport):
     def getRunningBatchJobIDs(self):
         # We need a dict from jobID (integer) to seconds it has been running
         secondsPerJob = dict()
-        for job in self._ourJobObjects():
+        for job in self._ourJobObject():
             # Grab the pod for each job
             pod = self._getPodForJob(job)
 
@@ -1088,10 +1093,10 @@ def executor():
 
     """
 
-    configureRootLogger()
-    setLogLevel("DEBUG")
+    configure_root_logger()
+    set_log_level("DEBUG")
     logger.debug("Starting executor")
-    
+
     # If we don't manage to run the child, what should our exit code be?
     exit_code = EXIT_STATUS_UNAVAILABLE_VALUE
 
@@ -1125,41 +1130,19 @@ def executor():
         if 'userScript' in job:
             job['userScript'].register()
             
-        # We need to tell other workers in this workflow not to do cleanup now that
-        # we are here, or else wait for them to finish. So get the cleanup info
-        # that knows where the work dir is.
-        cleanupInfo = job['workerCleanupInfo']
-        
-        # Join a Last Process Standing arena, so we know which process should be
-        # responsible for cleanup.
-        # We need to use the real workDir, not just the override from cleanupInfo.
-        # This needs to happen after the environment is applied.
-        arena = LastProcessStandingArena(Toil.getToilWorkDir(cleanupInfo.workDir), 
-            cleanupInfo.workflowID + '-kube-executor')
-        arena.enter()
-        try:
-            
-            # Start the child process
-            logger.debug("Invoking command: '%s'", job['command'])
-            child = subprocess.Popen(job['command'],
-                                     preexec_fn=lambda: os.setpgrp(),
-                                     shell=True)
+        # Start the child process
+        logger.debug("Invoking command: '%s'", job['command'])
+        child = subprocess.Popen(job['command'],
+                                 preexec_fn=lambda: os.setpgrp(),
+                                 shell=True)
 
-            # Reproduce child's exit code
-            exit_code = child.wait()
-            
-        finally:
-            for _ in arena.leave():
-                # We are the last concurrent executor to finish.
-                # Do batch system cleanup.
-                logger.debug('Cleaning up worker')
-                BatchSystemSupport.workerCleanup(cleanupInfo)
+        # Reproduce child's exit code
+        exit_code = child.wait()
+        
     finally:
         logger.debug('Cleaning up resources')
         # TODO: Change resource system to use a shared resource directory for everyone.
-        # Then move this into the last-process-standing cleanup
+        # Then move this into worker cleanup somehow
         Resource.cleanSystem()
         logger.debug('Shutting down')
         sys.exit(exit_code)
-
-
