@@ -1,4 +1,4 @@
-# Copyright (C) 2015-2016 Regents of the University of California
+# Copyright (C) 2015-2021 Regents of the University of California
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -11,19 +11,14 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-from __future__ import absolute_import
 
-from future import standard_library
-standard_library.install_aliases()
-from builtins import object
-from queue import Queue
-from collections import namedtuple
 from bisect import bisect
+from collections import namedtuple
+from queue import Queue
 from threading import Lock
 
 from toil.provisioners.abstractProvisioner import Shape
 
-
 TaskData = namedtuple('TaskData', (
     # Time when the task was started
     'startTime',
