@@ -1,4 +1,4 @@
-# Copyright (C) 2015-2016 Regents of the University of California
+# Copyright (C) 2015-2021 Regents of the University of California
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -11,25 +11,24 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-
-from past.utils import old_div
-from contextlib import contextmanager
 import logging
-import os
-import time
 import math
+import os
 import subprocess
 import sys
+import time
 import traceback
-from threading import Thread, Event
-from threading import Lock, Condition
-from six.moves.queue import Empty, Queue
+from contextlib import contextmanager
+from queue import Empty, Queue
+from threading import Condition, Event, Lock, Thread
 
 import toil
-from toil.batchSystems.abstractBatchSystem import BatchSystemSupport, EXIT_STATUS_UNAVAILABLE_VALUE, UpdatedBatchJobInfo
-from toil.lib.threading import cpu_count
 from toil import worker as toil_worker
+from toil.batchSystems.abstractBatchSystem import (EXIT_STATUS_UNAVAILABLE_VALUE,
+                                                   BatchSystemSupport,
+                                                   UpdatedBatchJobInfo)
 from toil.common import Toil
+from toil.lib.threading import cpu_count
 
 log = logging.getLogger(__name__)
 
@@ -40,7 +39,7 @@ class SingleMachineBatchSystem(BatchSystemSupport):
     give it as they come in, but in parallel.
 
     Uses a single "daddy" thread to manage a fleet of child processes.
-    
+
     Communication with the daddy thread happens via two queues: one queue of
     jobs waiting to be run (the input queue), and one queue of jobs that are
     finished/stopped and need to be returned by getUpdatedBatchJob (the output
@@ -71,7 +70,7 @@ class SingleMachineBatchSystem(BatchSystemSupport):
     physicalMemory = toil.physicalMemory()
 
     def __init__(self, config, maxCores, maxMemory, maxDisk):
-        
+        self.config = config
         # Limit to the smaller of the user-imposed limit and what we actually
         # have on this machine for each resource. 
         #
@@ -87,7 +86,9 @@ class SingleMachineBatchSystem(BatchSystemSupport):
                 # We have an actually specified limit and not the default
                 log.warning('Not enough memory! User limited to %i bytes but we only have %i bytes.', maxMemory, self.physicalMemory)
             maxMemory = self.physicalMemory
-        self.physicalDisk = toil.physicalDisk(config)
+
+        workdir = Toil.getLocalWorkflowDir(config.workflowID, config.workDir)  # config.workDir may be None; this sets a real directory
+        self.physicalDisk = toil.physicalDisk(workdir)
         if maxDisk > self.physicalDisk:
             if maxDisk != sys.maxsize:
                 # We have an actually specified limit and not the default
@@ -118,7 +119,7 @@ class SingleMachineBatchSystem(BatchSystemSupport):
         # A dictionary mapping IDs of submitted jobs to the command line
         self.jobs = {}
         """
-        :type: dict[str,toil.job.JobNode]
+        :type: dict[str,toil.job.JobDescription]
         """
 
         # A queue of jobs waiting to be executed. Consumed by the daddy thread.
@@ -148,7 +149,7 @@ class SingleMachineBatchSystem(BatchSystemSupport):
         """
 
         # A pool representing available CPU in units of minCores
-        self.coreFractions = ResourcePool(int(old_div(self.maxCores, self.minCores)), 'cores')
+        self.coreFractions = ResourcePool(int(self.maxCores / self.minCores), 'cores')
         # A pool representing available memory in bytes
         self.memory = ResourcePool(self.maxMemory, 'memory')
         # A pool representing the available space in bytes
@@ -201,7 +202,7 @@ class SingleMachineBatchSystem(BatchSystemSupport):
                         args = self.inputQueue.get_nowait()
                         jobCommand, jobID, jobCores, jobMemory, jobDisk, environment = args
 
-                        coreFractions = int(old_div(jobCores, self.minCores))
+                        coreFractions = int(jobCores / self.minCores)
                         
                         # Try to start the child
                         result = self._startChild(jobCommand, jobID,
@@ -483,36 +484,34 @@ class SingleMachineBatchSystem(BatchSystemSupport):
         log.debug('Child %d for job %s succeeded', pid, jobID)
 
         
-    def issueBatchJob(self, jobNode):
+    def issueBatchJob(self, jobDesc):
         """Adds the command and resources to a queue to be run."""
 
         self._checkOnDaddy()
 
         # Round cores to minCores and apply scale.
         # Make sure to give minCores even if asked for 0 cores, or negative or something. 
-        cores = max(math.ceil(jobNode.cores * self.scale / self.minCores) * self.minCores, self.minCores)
+        cores = max(math.ceil(jobDesc.cores * self.scale / self.minCores) * self.minCores, self.minCores)
         
         # Don't do our own assertions about job size vs. our configured size.
         # The abstract batch system can handle it.
-        self.checkResourceRequest(jobNode.memory, cores, jobNode.disk, name=jobNode.jobName,
-            detail='Scale is set to {}.'.format(self.scale))
-        
-        self.checkResourceRequest(jobNode.memory, cores, jobNode.disk)
-        log.debug("Issuing the command: %s with memory: %i, cores: %i, disk: %i" % (
-            jobNode.command, jobNode.memory, cores, jobNode.disk))
+        self.checkResourceRequest(jobDesc.memory, cores, jobDesc.disk, job_name=jobDesc.jobName,
+                                  detail=f'Scale is set to {self.scale}.')
+        log.debug(f"Issuing the command: {jobDesc.command} with "
+                  f"memory: {jobDesc.memory}, cores: {cores}, disk: {jobDesc.disk}")
         with self.jobIndexLock:
             jobID = self.jobIndex
             self.jobIndex += 1
-        self.jobs[jobID] = jobNode.command
+        self.jobs[jobID] = jobDesc.command
         
         if self.debugWorker:
             # Run immediately, blocking for return.
             # Ignore resource requirements; we run one job at a time
-            self._runDebugJob(jobNode.command, jobID, self.environment.copy())
+            self._runDebugJob(jobDesc.command, jobID, self.environment.copy())
         else:
             # Queue the job for later
-            self.inputQueue.put((jobNode.command, jobID, cores, jobNode.memory,
-                                jobNode.disk, self.environment.copy()))
+            self.inputQueue.put((jobDesc.command, jobID, cores, jobDesc.memory,
+                                jobDesc.disk, self.environment.copy()))
             
 
         return jobID
@@ -527,7 +526,7 @@ class SingleMachineBatchSystem(BatchSystemSupport):
             if jobID in self.runningJobs:
                 info = self.runningJobs[jobID]
                 info.killIntended = True
-                if info.popen != None:
+                if info.popen is not None:
                     log.debug('Send kill to PID %s', info.popen.pid)
                     info.popen.kill()
                     log.debug('Sent kill to PID %s', info.popen.pid)
@@ -583,7 +582,7 @@ class SingleMachineBatchSystem(BatchSystemSupport):
         setOption("scale", default=1)
 
 
-class Info(object):
+class Info:
     """
     Record for a running job.
 
@@ -599,7 +598,7 @@ class Info(object):
         self.killIntended = killIntended
 
 
-class ResourcePool(object):
+class ResourcePool:
     """
     Represents an integral amount of a resource (such as memory bytes).
 
