@@ -11,24 +11,14 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-
-from __future__ import absolute_import
-from __future__ import division
-from builtins import str
-from past.utils import old_div
 import logging
+import math
 import os
 from pipes import quote
-import time
-import math
-from toil.lib.misc import call_command, CalledProcessErrorStderr
-
-# Python 3 compatibility imports
-from six.moves.queue import Empty, Queue
-from six import iteritems
 
 from toil.batchSystems import MemoryString
 from toil.batchSystems.abstractGridEngineBatchSystem import AbstractGridEngineBatchSystem
+from toil.lib.misc import CalledProcessErrorStderr, call_command
 
 logger = logging.getLogger(__name__)
 
@@ -76,7 +66,7 @@ class SlurmBatchSystem(AbstractGridEngineBatchSystem):
                 raise e
 
         def getJobExitCode(self, slurmJobID):
-            logger.debug("Getting exit code for slurm job %d", int(slurmJobID))
+            logger.debug(f"Getting exit code for slurm job: {slurmJobID}")
 
             try:
                 state, rc = self._getJobDetailsFromSacct(slurmJobID)
@@ -114,9 +104,9 @@ class SlurmBatchSystem(AbstractGridEngineBatchSystem):
                     # A non-zero signal may indicate e.g. an out-of-memory killed job
                     status = 128 + signal
                 logger.debug("sacct exit code is %s, returning status %d", exitcode, status)
-                return (state, status)
+                return state, status
             logger.debug("Did not find exit code for job in sacct output")
-            return (None, None)
+            return None, None
 
         def _getJobDetailsFromScontrol(self, slurmJobID):
             args = ['scontrol',
@@ -124,17 +114,21 @@ class SlurmBatchSystem(AbstractGridEngineBatchSystem):
                     'job',
                     str(slurmJobID)]
 
-            stderr = call_command(args)
-            job = dict()
-            for line in stdout:
-                logger.debug("%s output %s", args[0], line)
-                values = line.decode('utf-8').strip().split()
+            stdout = call_command(args)
+            if isinstance(stdout, str):
+                values = stdout.strip().split()
+            elif isinstance(stdout, bytes):
+                values = stdout.decode('utf-8').strip().split()
 
-                # If job information is not available an error is issued:
-                # slurm_load_jobs error: Invalid job id specified
-                # There is no job information, so exit.
-                if len(values)>0 and values[0] == 'slurm_load_jobs':
-                    return (None, None)
+            # If job information is not available an error is issued:
+            # slurm_load_jobs error: Invalid job id specified
+            # There is no job information, so exit.
+            if len(values) > 0 and values[0] == 'slurm_load_jobs':
+                return (None, None)
+
+            job = dict()
+            for item in values:
+                logger.debug(f"{args[0]} output {item}")
 
                 # Output is in the form of many key=value pairs, multiple pairs on each line
                 # and multiple lines in the output. Each pair is pulled out of each line and
@@ -158,7 +152,7 @@ class SlurmBatchSystem(AbstractGridEngineBatchSystem):
             except KeyError:
                 rc = None
 
-            return (state, rc)
+            return state, rc
 
         """
         Implementation-specific helper methods
@@ -177,11 +171,11 @@ class SlurmBatchSystem(AbstractGridEngineBatchSystem):
 
                 sbatch_line.append('--export=' + ','.join(argList))
 
-            if mem is not None:
+            if mem is not None and self.boss.config.allocate_mem:
                 # memory passed in is in bytes, but slurm expects megabytes
-                sbatch_line.append('--mem={}'.format(old_div(int(mem), 2 ** 20)))
+                sbatch_line.append(f'--mem={math.ceil(mem / 2 ** 20)}')
             if cpu is not None:
-                sbatch_line.append('--cpus-per-task={}'.format(int(math.ceil(cpu))))
+                sbatch_line.append(f'--cpus-per-task={math.ceil(cpu)}')
 
             stdoutfile = self.boss.formatStdOutErrPath(jobID, 'slurm', '%j', 'std_output')
             stderrfile = self.boss.formatStdOutErrPath(jobID, 'slurm', '%j', 'std_error')
