@@ -1,4 +1,4 @@
-# Copyright (C) 2015-2018 Regents of the University of California
+# Copyright (C) 2015-2021 Regents of the University of California
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -11,44 +11,30 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-
-from __future__ import absolute_import, print_function
-
-from future import standard_library
-
-standard_library.install_aliases()
-from builtins import map
-from builtins import str
-from contextlib import contextmanager
 import errno
 import hashlib
 import logging
 import os
+import stat
 import re
 import shutil
 import sqlite3
-import sys
 import tempfile
 import threading
 import time
 import uuid
+from contextlib import contextmanager
 
 from toil.common import cacheDirName, getDirSizeRecursively, getFileSystemSize
-from toil.lib.bioio import makePublicDir
-from toil.lib.humanize import bytes2human
-from toil.lib.misc import robust_rmtree, atomic_copy, atomic_copyobj
-from toil.lib.retry import retry
-from toil.lib.threading import get_process_name, process_name_exists
+from toil.fileStores import FileID, make_public_dir
 from toil.fileStores.abstractFileStore import AbstractFileStore
-from toil.fileStores import FileID
+from toil.lib.humanize import bytes2human
+from toil.lib.misc import atomic_copy, atomic_copyobj, robust_rmtree
+from toil.lib.retry import ErrorCondition, retry
+from toil.lib.threading import get_process_name, process_name_exists
 
 logger = logging.getLogger(__name__)
 
-if sys.version_info[0] < 3:
-    # Define a usable FileNotFoundError as will be raised by os.remove on a
-    # nonexistent file.
-    FileNotFoundError = OSError
-
 
 # Use longer timeout to avoid hitting 'database is locked' errors.
 SQLITE_TIMEOUT_SECS = 60.0
@@ -183,8 +169,8 @@ class CachingFileStore(AbstractFileStore):
 
     """
 
-    def __init__(self, jobStore, jobGraph, localTempDir, waitForPreviousCommit):
-        super(CachingFileStore, self).__init__(jobStore, jobGraph, localTempDir, waitForPreviousCommit)
+    def __init__(self, jobStore, jobDesc, localTempDir, waitForPreviousCommit):
+        super(CachingFileStore, self).__init__(jobStore, jobDesc, localTempDir, waitForPreviousCommit)
 
         # For testing, we have the ability to force caching to be non-free, by never linking from the file store
         self.forceNonFreeCaching = False
@@ -210,8 +196,8 @@ class CachingFileStore(AbstractFileStore):
 
         # Since each worker has it's own unique CachingFileStore instance, and only one Job can run
         # at a time on a worker, we can track some stuff about the running job in ourselves.
-        self.jobName = str(self.jobGraph)
-        self.jobID = self.jobGraph.jobStoreID
+        self.jobName = str(self.jobDesc)
+        self.jobID = self.jobDesc.jobStoreID
         logger.debug('Starting job (%s) with ID (%s).', self.jobName, self.jobID)
 
         # When the job actually starts, we will fill this in with the job's disk requirement.
@@ -260,6 +246,12 @@ class CachingFileStore(AbstractFileStore):
 
     
     @staticmethod
+    @retry(infinite_retries=True,
+           errors=[
+                         ErrorCondition(
+                             error=sqlite3.OperationalError,
+                             error_message_must_include='is locked')
+                     ])
     def _staticWrite(con, cur, operations):
         """
         Write to the caching database, using the given connection.
@@ -279,40 +271,36 @@ class CachingFileStore(AbstractFileStore):
         :return: Number of rows modified by the last operation
         :rtype: int
         """
-
-        for attempt in retry(timeout=float('inf'), predicate=lambda e: isinstance(e, sqlite3.OperationalError) and 'is locked' in str(e)):
-            # Try forever with backoff
-            with attempt:
-                try:
-                    for item in operations:
-                        if not isinstance(item, tuple):
-                            # Must be a single SQL string. Wrap it.
-                            item = (item,)
-                        # Parse out the command and the variables to substitute
-                        command = item[0]
-                        if len(item) < 2:
-                            args = ()
-                        else:
-                            args = item[1]
-                        # Do it
-                        cur.execute(command, args)
-                except Exception as e:
-                    logging.error('Error talking to caching database: %s', str(e))
-
-                    # Try to make sure we don't somehow leave anything part-done if a
-                    # middle operation somehow fails. 
-                    try:
-                        con.rollback()
-                    except:
-                        # But don't stop if we can't roll back.
-                        pass
-
-                    # Raise and maybe retry
-                    raise e
+        try:
+            for item in operations:
+                if not isinstance(item, tuple):
+                    # Must be a single SQL string. Wrap it.
+                    item = (item,)
+                # Parse out the command and the variables to substitute
+                command = item[0]
+                if len(item) < 2:
+                    args = ()
                 else:
-                    # The transaction worked!
-                    # Now commit the transaction.
-                    con.commit()
+                    args = item[1]
+                # Do it
+                cur.execute(command, args)
+        except Exception as e:
+            logging.error('Error talking to caching database: %s', str(e))
+
+            # Try to make sure we don't somehow leave anything part-done if a
+            # middle operation somehow fails.
+            try:
+                con.rollback()
+            except:
+                # But don't stop if we can't roll back.
+                pass
+
+            # Raise and maybe retry
+            raise e
+        else:
+            # The transaction worked!
+            # Now commit the transaction.
+            con.commit()
 
         return cur.rowcount
 
@@ -646,7 +634,10 @@ class CachingFileStore(AbstractFileStore):
         deadOwners = []
         for owner in owners:
             if not process_name_exists(self.workDir, owner):
+                logger.debug('Owner %s is dead', owner)
                 deadOwners.append(owner)
+            else:
+                logger.debug('Owner %s is alive', owner)
 
         for owner in deadOwners:
             # Try and adopt all the files that any dead owner had
@@ -672,7 +663,7 @@ class CachingFileStore(AbstractFileStore):
                 ('UPDATE files SET owner = NULL, state = ? WHERE owner = ? AND (state = ? OR state = ?)',
                 ('cached', owner, 'uploadable', 'uploading'))])
 
-            logger.debug('Tried to adopt file operations from dead worker %d', owner)
+            logger.debug('Tried to adopt file operations from dead worker %s to ourselves as %s', owner, me)
 
     @classmethod
     def _executePendingDeletions(cls, workDir, con, cur):
@@ -701,9 +692,11 @@ class CachingFileStore(AbstractFileStore):
             filePath = row[1]
             try:
                 os.unlink(filePath)
+                logger.debug('Successfully deleted: %s', filePath)
             except OSError:
                 # Probably already deleted
-                continue
+                logger.debug('File already gone: %s', filePath)
+                # Still need to mark it as deleted
 
             # Whether we deleted the file or just found out that it is gone, we
             # need to take credit for deleting it so that we remove it from the
@@ -982,7 +975,7 @@ class CachingFileStore(AbstractFileStore):
         # Create a working directory for the job
         startingDir = os.getcwd()
         # Move self.localTempDir from the worker directory set up in __init__ to a per-job directory.
-        self.localTempDir = makePublicDir(os.path.join(self.localTempDir, str(uuid.uuid4())))
+        self.localTempDir = make_public_dir(os.path.join(self.localTempDir, str(uuid.uuid4())))
         # Check the status of all jobs on this node. If there are jobs that started and died before
         # cleaning up their presence from the database, clean them up ourselves.
         self._removeDeadJobs(self.workDir, self.con)
@@ -997,7 +990,8 @@ class CachingFileStore(AbstractFileStore):
         self._allocateSpaceForJob(self.jobDiskBytes)
         try:
             os.chdir(self.localTempDir)
-            yield
+            with super().open(job):
+                yield
         finally:
             # See how much disk space is used at the end of the job.
             # Not a real peak disk usage, but close enough to be useful for warning the user.
@@ -1036,12 +1030,12 @@ class CachingFileStore(AbstractFileStore):
         fileSize = os.stat(absLocalFileName).st_size
 
         # Work out who is making the file
-        creatorID = self.jobGraph.jobStoreID
+        creatorID = self.jobDesc.jobStoreID
 
         # Create an empty file to get an ID.
+        # Make sure to pass along the file basename.
         # TODO: this empty file could leak if we die now...
-        fileID = self.jobStore.getEmptyFileStoreID(creatorID, cleanup)
-
+        fileID = self.jobStore.getEmptyFileStoreID(creatorID, cleanup, os.path.basename(localFileName))
         # Work out who we are
         me = get_process_name(self.workDir)
 
@@ -1053,8 +1047,8 @@ class CachingFileStore(AbstractFileStore):
         self._write([('INSERT INTO files VALUES (?, ?, ?, ?, ?)', (fileID, cachePath, fileSize, 'uploadable', me)),
             ('INSERT INTO refs VALUES (?, ?, ?, ?)', (absLocalFileName, fileID, creatorID, 'immutable'))])
 
-        if absLocalFileName.startswith(self.localTempDir):
-            # We should link into the cache, because the upload is coming from our local temp dir
+        if absLocalFileName.startswith(self.localTempDir) and not os.path.islink(absLocalFileName):
+            # We should link into the cache, because the upload is coming from our local temp dir (and not via a symlink in there)
             try:
                 # Try and hardlink the file into the cache.
                 # This can only fail if the system doesn't have hardlinks, or the
@@ -1064,15 +1058,19 @@ class CachingFileStore(AbstractFileStore):
 
                 linkedToCache = True
 
-                logger.debug('Linked file %s into cache at %s; deferring write to job store', localFileName, cachePath)
+                logger.debug('Hardlinked file %s into cache at %s; deferring write to job store', localFileName, cachePath)
+                assert not os.path.islink(cachePath), "Symlink %s has invaded cache!" % cachePath
 
                 # Don't do the upload now. Let it be deferred until later (when the job is committing).
             except OSError:
                 # We couldn't make the link for some reason
                 linkedToCache = False
         else:
-            # The tests insist that if you are uploading a file from outside
-            # the local temp dir, it should not be linked into the cache.
+            # If you are uploading a file that physically exists outside the
+            # local temp dir, it should not be linked into the cache. On
+            # systems that support it, we could end up with a
+            # hardlink-to-symlink in the cache if we break this rule, allowing
+            # files to vanish from our cache.
             linkedToCache = False
 
 
@@ -1088,14 +1086,14 @@ class CachingFileStore(AbstractFileStore):
                 ('DELETE FROM files WHERE id = ?', (fileID,))])
 
             # Save the file to the job store right now
-            logger.debug('Actually executing upload for file %s', fileID)
+            logger.debug('Actually executing upload immediately for file %s', fileID)
             self.jobStore.updateFile(fileID, absLocalFileName)
 
         # Ship out the completed FileID object with its real size.
         return FileID.forPath(fileID, absLocalFileName)
 
     def readGlobalFile(self, fileStoreID, userPath=None, cache=True, mutable=False, symlink=False):
-        
+
         if str(fileStoreID) in self.filesToDelete:
             # File has already been deleted
             raise FileNotFoundError('Attempted to read deleted file: {}'.format(fileStoreID))
@@ -1110,18 +1108,25 @@ class CachingFileStore(AbstractFileStore):
             localFilePath = self.getLocalTempFileName()
 
         # Work out what job we are operating on behalf of
-        readerID = self.jobGraph.jobStoreID
+        readerID = self.jobDesc.jobStoreID
 
         if cache:
             # We want to use the cache
 
             if mutable:
-                return self._readGlobalFileMutablyWithCache(fileStoreID, localFilePath, readerID)
+                finalPath = self._readGlobalFileMutablyWithCache(fileStoreID, localFilePath, readerID)
             else:
-                return self._readGlobalFileWithCache(fileStoreID, localFilePath, symlink, readerID)
+                finalPath = self._readGlobalFileWithCache(fileStoreID, localFilePath, symlink, readerID)
         else:
             # We do not want to use the cache
-            return self._readGlobalFileWithoutCache(fileStoreID, localFilePath, mutable, symlink, readerID)
+            finalPath = self._readGlobalFileWithoutCache(fileStoreID, localFilePath, mutable, symlink, readerID)
+
+        if getattr(fileStoreID, 'executable', False):
+            os.chmod(finalPath, os.stat(finalPath).st_mode | stat.S_IXUSR)
+
+        # Record access in case the job crashes and we have to log it
+        self.logAccess(fileStoreID, finalPath)
+        return finalPath
 
 
     def _readGlobalFileWithoutCache(self, fileStoreID, localFilePath, mutable, symlink, readerID):
@@ -1445,7 +1450,7 @@ class CachingFileStore(AbstractFileStore):
     def _createLinkFromCache(self, cachedPath, localFilePath, symlink=True):
         """
         Create a hardlink or symlink from the given path in the cache to the
-        given user-provided path. Destination must not exist.
+        given user-provided path. Destination must not exist. Source must exist.
 
         Only creates a symlink if a hardlink cannot be created and symlink is
         true.
@@ -1459,6 +1464,8 @@ class CachingFileStore(AbstractFileStore):
         :rtype: bool
         """
 
+        assert os.path.exists(cachedPath), "Cannot create link to missing cache file %s" % cachedPath
+
         try:
             # Try and make the hard link.
             os.link(cachedPath, localFilePath)
@@ -1499,7 +1506,7 @@ class CachingFileStore(AbstractFileStore):
             # Try and create a downloading entry if no entry exists.
             # Make sure to create a reference at the same time if it succeeds, to bill it against our job's space.
             # Don't create the mutable reference yet because we might not necessarily be able to clear that space.
-            logger.debug('Trying to make file record and reference for id %s', fileStoreID)
+            logger.debug('Trying to make file downloading file record and reference for id %s', fileStoreID)
             self._write([('INSERT OR IGNORE INTO files VALUES (?, ?, ?, ?, ?)',
                 (fileStoreID, cachedPath, self.getGlobalFileSize(fileStoreID), 'downloading', me)),
                 ('INSERT INTO refs SELECT ?, id, ?, ? FROM files WHERE id = ? AND state = ? AND owner = ?',
@@ -1541,7 +1548,7 @@ class CachingFileStore(AbstractFileStore):
                     return localFilePath
 
             else:
-                logger.debug('Someone else is already responsible for file %s', fileStoreID)
+                logger.debug('We already have an entry in the cache database for file %s', fileStoreID)
 
                 # A record already existed for this file.
                 # Try and create an immutable reference to an entry that
@@ -1584,6 +1591,9 @@ class CachingFileStore(AbstractFileStore):
                     # finish. If they die, we will notice.
                     self._removeDeadJobs(self.workDir, self.con)
                     self._stealWorkFromTheDead()
+                    # We may have acquired ownership of partially-downloaded
+                    # files, now in deleting state, that we need to delete
+                    # before we can download them.
                     self._executePendingDeletions(self.workDir, self.con, self.cur)
 
                     # Wait for other people's downloads to progress.
@@ -1593,7 +1603,9 @@ class CachingFileStore(AbstractFileStore):
         if str(fileStoreID) in self.filesToDelete:
             # File has already been deleted
             raise FileNotFoundError('Attempted to read deleted file: {}'.format(fileStoreID))
-
+        
+        self.logAccess(fileStoreID)
+        
         # TODO: can we fulfil this from the cache if the file is in the cache?
         # I think we can because if a job is keeping the file data on disk due to having it open, it must be paying for it itself.
         return self.jobStore.readFileStream(fileStoreID)
@@ -1601,7 +1613,7 @@ class CachingFileStore(AbstractFileStore):
     def deleteLocalFile(self, fileStoreID):
         # What job are we operating as?
         jobID = self.jobID
-
+        
         # What paths did we delete
         deleted = []
         # What's the first path, if any, that was missing? If we encounter a
@@ -1626,9 +1638,15 @@ class CachingFileStore(AbstractFileStore):
                     break
                 deleted.append(path)
 
+        if len(deleted) == 0 and not missingFile:
+            # We have to tell the user if they tried to delete 0 local copies.
+            # But if we found a missing local copy, go on to report that instead.
+            raise OSError(errno.ENOENT, "Attempting to delete local copies of a file with none: {}".format(fileStoreID))
+
         for path in deleted:
             # Drop the references
             self._write([('DELETE FROM refs WHERE file_id = ? AND job_id = ? AND path = ?', (fileStoreID, jobID, path))])
+            logger.debug('Deleted local file %s for global file %s', path, fileStoreID)
 
         # Now space has been revoked from the cache because that job needs its space back.
         # That might result in stuff having to be evicted.
@@ -1642,8 +1660,15 @@ class CachingFileStore(AbstractFileStore):
             raise IllegalDeletionCacheError(missingFile)
 
     def deleteGlobalFile(self, fileStoreID):
-        # Delete local copies for this job
-        self.deleteLocalFile(fileStoreID)
+        try:
+            # Delete local copies of the file
+            self.deleteLocalFile(fileStoreID)
+        except OSError as e:
+            if e.errno == errno.ENOENT:
+                # Turns out there weren't any
+                pass
+            else:
+                raise
 
         # Work out who we are
         me = get_process_name(self.workDir)
@@ -1683,12 +1708,17 @@ class CachingFileStore(AbstractFileStore):
 
     def waitForCommit(self):
         # We need to block on the upload thread.
+
         # We may be called even if startCommit is not called. In that
         # case, a new instance of this class should have been created by the
         # worker and ought to pick up all our work by PID via the database, and
         # this instance doesn't actually have to commit.
 
-        if self.commitThread is not None:
+        # If running in the destructor, we may already *be* in the commit
+        # thread. It can do some destructor work after it finishes its real
+        # work.
+
+        if self.commitThread is not None and self.commitThread is not threading.current_thread():
             self.commitThread.join()
 
         return True
@@ -1732,18 +1762,18 @@ class CachingFileStore(AbstractFileStore):
 
                 # Indicate any files that should be deleted once the update of
                 # the job wrapper is completed.
-                self.jobGraph.filesToDelete = list(self.filesToDelete)
+                self.jobDesc.filesToDelete = list(self.filesToDelete)
                 # Complete the job
-                self.jobStore.update(self.jobGraph)
+                self.jobStore.update(self.jobDesc)
                 # Delete any remnant jobs
                 list(map(self.jobStore.delete, self.jobsToDelete))
                 # Delete any remnant files
                 list(map(self.jobStore.deleteFile, self.filesToDelete))
                 # Remove the files to delete list, having successfully removed the files
                 if len(self.filesToDelete) > 0:
-                    self.jobGraph.filesToDelete = []
+                    self.jobDesc.filesToDelete = []
                     # Update, removing emptying files to delete
-                    self.jobStore.update(self.jobGraph)
+                    self.jobStore.update(self.jobDesc)
         except:
             self._terminateEvent.set()
             raise
