@@ -1,4 +1,4 @@
-# Copyright (C) 2015-2018 Regents of the University of California
+# Copyright (C) 2015-2021 Regents of the University of California
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -11,42 +11,31 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-
-from __future__ import absolute_import, print_function
-
-from future import standard_library
-
-standard_library.install_aliases()
-from builtins import map
-from builtins import str
-from collections import defaultdict
-from contextlib import contextmanager
-import dill
 import errno
 import fcntl
 import logging
 import os
+import stat
 import sys
 import uuid
+from collections import defaultdict
+from contextlib import contextmanager
 
+import dill
+
+from toil.common import getDirSizeRecursively, getFileSystemSize
+from toil.fileStores import FileID, make_public_dir
+from toil.fileStores.abstractFileStore import AbstractFileStore
+from toil.lib.humanize import bytes2human
 from toil.lib.misc import robust_rmtree
 from toil.lib.threading import get_process_name, process_name_exists
-from toil.lib.humanize import bytes2human
-from toil.common import getDirSizeRecursively, getFileSystemSize
-from toil.lib.bioio import makePublicDir
-from toil.fileStores.abstractFileStore import AbstractFileStore
-from toil.fileStores import FileID
 
 logger = logging.getLogger(__name__)
 
-if sys.version_info[0] < 3:
-    # Define a usable FileNotFoundError as will be raised by os.oprn on a
-    # nonexistent parent directory.
-    FileNotFoundError = OSError
 
 class NonCachingFileStore(AbstractFileStore):
-    def __init__(self, jobStore, jobGraph, localTempDir, waitForPreviousCommit):
-        super(NonCachingFileStore, self).__init__(jobStore, jobGraph, localTempDir, waitForPreviousCommit)
+    def __init__(self, jobStore, jobDesc, localTempDir, waitForPreviousCommit):
+        super(NonCachingFileStore, self).__init__(jobStore, jobDesc, localTempDir, waitForPreviousCommit)
         # This will be defined in the `open` method.
         self.jobStateFile = None
         self.localFileMap = defaultdict(list)
@@ -55,7 +44,7 @@ class NonCachingFileStore(AbstractFileStore):
     def open(self, job):
         jobReqs = job.disk
         startingDir = os.getcwd()
-        self.localTempDir = makePublicDir(os.path.join(self.localTempDir, str(uuid.uuid4())))
+        self.localTempDir = make_public_dir(os.path.join(self.localTempDir, str(uuid.uuid4())))
         self._removeDeadJobs(self.workDir)
         self.jobStateFile = self._createJobStateFile()
         freeSpace, diskSize = getFileSystemSize(self.localTempDir)
@@ -64,7 +53,8 @@ class NonCachingFileStore(AbstractFileStore):
                            self.jobName)
         try:
             os.chdir(self.localTempDir)
-            yield
+            with super().open(job):
+                yield
         finally:
             diskUsed = getDirSizeRecursively(self.localTempDir)
             logString = ("Job {jobName} used {percent:.2f}% ({humanDisk}B [{disk}B] used, "
@@ -87,9 +77,12 @@ class NonCachingFileStore(AbstractFileStore):
 
     def writeGlobalFile(self, localFileName, cleanup=False):
         absLocalFileName = self._resolveAbsoluteLocalPath(localFileName)
-        creatorID = self.jobGraph.jobStoreID
+        creatorID = self.jobDesc.jobStoreID
         fileStoreID = self.jobStore.writeFile(absLocalFileName, creatorID, cleanup)
-        self.localFileMap[fileStoreID].append(absLocalFileName)
+        if absLocalFileName.startswith(self.localTempDir):
+            # Only files in the appropriate directory should become local files
+            # we can delete with deleteLocalFile
+            self.localFileMap[fileStoreID].append(absLocalFileName)
         return FileID.forPath(fileStoreID, absLocalFileName)
 
     def readGlobalFile(self, fileStoreID, userPath=None, cache=True, mutable=False, symlink=False):
@@ -102,11 +95,15 @@ class NonCachingFileStore(AbstractFileStore):
 
         self.jobStore.readFile(fileStoreID, localFilePath, symlink=symlink)
         self.localFileMap[fileStoreID].append(localFilePath)
+        if getattr(fileStoreID, 'executable', False):
+            os.chmod(localFilePath, os.stat(localFilePath).st_mode | stat.S_IXUSR)
+        self.logAccess(fileStoreID, localFilePath)
         return localFilePath
 
     @contextmanager
     def readGlobalFileStream(self, fileStoreID):
         with self.jobStore.readFileStream(fileStoreID) as f:
+            self.logAccess(fileStoreID)
             yield f
 
     def exportFile(self, jobStoreFileID, dstUrl):
@@ -116,7 +113,7 @@ class NonCachingFileStore(AbstractFileStore):
         try:
             localFilePaths = self.localFileMap.pop(fileStoreID)
         except KeyError:
-            raise OSError(errno.ENOENT, "Attempting to delete a non-local file")
+            raise OSError(errno.ENOENT, "Attempting to delete local copies of a file with none")
         else:
             for localFilePath in localFilePaths:
                 os.remove(localFilePath)
@@ -148,18 +145,18 @@ class NonCachingFileStore(AbstractFileStore):
         try:
             # Indicate any files that should be deleted once the update of
             # the job wrapper is completed.
-            self.jobGraph.filesToDelete = list(self.filesToDelete)
+            self.jobDesc.filesToDelete = list(self.filesToDelete)
             # Complete the job
-            self.jobStore.update(self.jobGraph)
+            self.jobStore.update(self.jobDesc)
             # Delete any remnant jobs
             list(map(self.jobStore.delete, self.jobsToDelete))
             # Delete any remnant files
             list(map(self.jobStore.deleteFile, self.filesToDelete))
             # Remove the files to delete list, having successfully removed the files
             if len(self.filesToDelete) > 0:
-                self.jobGraph.filesToDelete = []
+                self.jobDesc.filesToDelete = []
                 # Update, removing emptying files to delete
-                self.jobStore.update(self.jobGraph)
+                self.jobStore.update(self.jobDesc)
         except:
             self._terminateEvent.set()
             raise
@@ -169,7 +166,6 @@ class NonCachingFileStore(AbstractFileStore):
         Cleanup function that is run when destroying the class instance.  Nothing to do since there
         are no async write events.
         """
-        pass
 
     @classmethod
     def _removeDeadJobs(cls, nodeInfo, batchSystemShutdown=False):
@@ -272,4 +268,3 @@ class NonCachingFileStore(AbstractFileStore):
         :param dir_: The workflow directory that will contain all the individual worker directories.
         """
         cls._removeDeadJobs(dir_, batchSystemShutdown=True)
-
