@@ -1,4 +1,4 @@
-# Copyright (C) 2015-2016 Regents of the University of California
+# Copyright (C) 2015-2021 Regents of the University of California
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -11,43 +11,29 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-from __future__ import absolute_import
-
-from future import standard_library
-standard_library.install_aliases()
-from builtins import str
-from builtins import map
-from builtins import object
-from builtins import super
-import shutil
-
+import logging
+import pickle
 import re
-from abc import ABCMeta, abstractmethod
-from contextlib import contextmanager, closing
+import shutil
+import urllib.parse as urlparse
+from abc import ABC, ABCMeta, abstractmethod
+from contextlib import closing, contextmanager
 from datetime import timedelta
+from http.client import BadStatusLine
+from urllib.request import urlopen
 from uuid import uuid4
-
-# Python 3 compatibility imports
-from six import itervalues
-from six.moves.urllib.request import urlopen
-import six.moves.urllib.parse as urlparse
-
-from toil.lib.retry import retry_http
+from typing import Set, Union
+from requests.exceptions import HTTPError
 
 from toil.common import safeUnpickleFromStream
 from toil.fileStores import FileID
-from toil.job import JobException
+from toil.job import (CheckpointJobDescription,
+                      TemporaryID,
+                      JobException,
+                      ServiceJobDescription)
 from toil.lib.memoize import memoize
 from toil.lib.misc import WriteWatchingStream
-from toil.lib.objects import abstractclassmethod
-from future.utils import with_metaclass
-
-try:
-    import cPickle as pickle
-except ImportError:
-    import pickle
-
-import logging
+from toil.lib.retry import ErrorCondition, retry
 
 logger = logging.getLogger(__name__)
 
@@ -116,9 +102,19 @@ class JobStoreExistsException(Exception):
             "the job store with 'toil clean' to start the workflow from scratch." % locator)
 
 
-class AbstractJobStore(with_metaclass(ABCMeta, object)):
+class AbstractJobStore(ABC):
     """
     Represents the physical storage for the jobs and files in a Toil workflow.
+    
+    JobStores are responsible for storing :class:`toil.job.JobDescription`
+    (which relate jobs to each other) and files.
+    
+    Actual :class:`toil.job.Job` objects are stored in files, referenced by
+    JobDescriptions. All the non-file CRUD methods the JobStore provides deal
+    in JobDescriptions and not full, executable Jobs.
+    
+    To actually get ahold of a :class:`toil.job.Job`, use
+    :meth:`toil.job.Job.loadJob` with a JobStore and the relevant JobDescription.
     """
 
     def __init__(self):
@@ -188,12 +184,12 @@ class AbstractJobStore(with_metaclass(ABCMeta, object)):
 
     def loadRootJob(self):
         """
-        Loads the root job in the current job store.
+        Loads the JobDescription for the root job in the current job store.
 
         :raises toil.job.JobException: If no root job is set or if the root job doesn't exist in
                 this job store
         :return: The root job.
-        :rtype: toil.jobGraph.JobGraph
+        :rtype: toil.job.JobDescription
         """
         try:
             with self.readSharedFileStream(self.rootJobStoreIDFileName) as f:
@@ -207,15 +203,16 @@ class AbstractJobStore(with_metaclass(ABCMeta, object)):
 
     # FIXME: This is only used in tests, why do we have it?
 
-    def createRootJob(self, *args, **kwargs):
+    def createRootJob(self, desc):
         """
-        Create a new job and set it as the root job in this job store
+        Create the given JobDescription and set it as the root job in this job store
 
-        :rtype: toil.jobGraph.JobGraph
+        :param toil.job.JobDescription desc: JobDescription to save and make the root job.
+        :rtype: toil.job.JobDescription
         """
-        rootJob = self.create(*args, **kwargs)
-        self.setRootJob(rootJob.jobStoreID)
-        return rootJob
+        self.create(desc)
+        self.setRootJob(desc.jobStoreID)
+        return desc
 
     def getRootJobReturnValue(self):
         """
@@ -272,7 +269,8 @@ class AbstractJobStore(with_metaclass(ABCMeta, object)):
         """
         Imports the file at the given URL into job store. The ID of the newly imported file is
         returned. If the name of a shared file name is provided, the file will be imported as
-        such and None is returned.
+        such and None is returned. If an executable file on the local filesystem is uploaded, its
+        executability will be preserved when it is downloaded.
 
         Currently supported schemes are:
 
@@ -323,8 +321,8 @@ class AbstractJobStore(with_metaclass(ABCMeta, object)):
         """
         if sharedFileName is None:
             with self.writeFileStream() as (writable, jobStoreFileID):
-                size = otherCls._readFromUrl(url, writable)
-                return FileID(jobStoreFileID, size)
+                size, executable = otherCls._readFromUrl(url, writable)
+                return FileID(jobStoreFileID, size, executable)
         else:
             self._requireValidSharedFileName(sharedFileName)
             with self.writeSharedFileStream(sharedFileName) as writable:
@@ -333,7 +331,9 @@ class AbstractJobStore(with_metaclass(ABCMeta, object)):
 
     def exportFile(self, jobStoreFileID, dstUrl):
         """
-        Exports file to destination pointed at by the destination URL.
+        Exports file to destination pointed at by the destination URL. The exported file will be
+        executable if and only if it was originally uploaded from an executable file on the
+        local filesystem.
 
         Refer to :meth:`.AbstractJobStore.importFile` documentation for currently supported URL schemes.
 
@@ -377,10 +377,14 @@ class AbstractJobStore(with_metaclass(ABCMeta, object)):
 
         :param urlparse.ParseResult url: The parsed URL of the file to export to.
         """
+        executable = False
         with self.readFileStream(jobStoreFileID) as readable:
-            otherCls._writeToUrl(readable, url)
+            if getattr(jobStoreFileID, 'executable', False):
+                executable = jobStoreFileID.executable
+            otherCls._writeToUrl(readable, url, executable)
 
-    @abstractclassmethod
+    @classmethod
+    @abstractmethod
     def getSize(cls, url):
         """
         Get the size in bytes of the file at the given URL, or None if it cannot be obtained.
@@ -390,7 +394,8 @@ class AbstractJobStore(with_metaclass(ABCMeta, object)):
         """
         raise NotImplementedError
 
-    @abstractclassmethod
+    @classmethod
+    @abstractmethod
     def _readFromUrl(cls, url, writable):
         """
         Reads the contents of the object at the specified location and writes it to the given
@@ -403,11 +408,13 @@ class AbstractJobStore(with_metaclass(ABCMeta, object)):
 
         :param writable: a writable stream
 
-        :return int: returns the size of the file in bytes
+        :return: The size of the file in bytes and whether the executable permission bit is set
+        :rtype: Tuple[int, bool]
         """
         raise NotImplementedError()
 
-    @abstractclassmethod
+    @classmethod
+    @abstractmethod
     def _writeToUrl(cls, readable, url):
         """
         Reads the contents of the given readable stream and writes it to the object at the
@@ -422,7 +429,8 @@ class AbstractJobStore(with_metaclass(ABCMeta, object)):
         """
         raise NotImplementedError()
 
-    @abstractclassmethod
+    @classmethod
+    @abstractmethod
     def _supportsUrl(cls, url, export=False):
         """
         Returns True if the job store supports the URL's scheme.
@@ -467,17 +475,16 @@ class AbstractJobStore(with_metaclass(ABCMeta, object)):
         Fixes jobs that might have been partially updated. Resets the try counts and removes jobs
         that are not successors of the current root job.
 
-        :param dict[str,toil.jobGraph.JobGraph] jobCache: if a value it must be a dict
-               from job ID keys to JobGraph object values. Jobs will be loaded from the cache
+        :param dict[str,toil.job.JobDescription] jobCache: if a value it must be a dict
+               from job ID keys to JobDescription object values. Jobs will be loaded from the cache
                (which can be downloaded from the job store in a batch) instead of piecemeal when
                recursed into.
         """
         if jobCache is None:
             logger.warning("Cleaning jobStore recursively. This may be slow.")
 
-        # Functions to get and check the existence of jobs, using the jobCache
-        # if present
-        def getJob(jobId):
+        # Functions to get and check the existence of jobs, using the jobCache if present
+        def getJobDescription(jobId):
             if jobCache is not None:
                 try:
                     return jobCache[jobId]
@@ -487,6 +494,7 @@ class AbstractJobStore(with_metaclass(ABCMeta, object)):
                 return self.load(jobId)
 
         def haveJob(jobId):
+            assert len(jobId) > 1, "Job ID {} too short; is a string being used as a list?".format(jobId)
             if jobCache is not None:
                 if jobId in jobCache:
                     return True
@@ -494,95 +502,123 @@ class AbstractJobStore(with_metaclass(ABCMeta, object)):
                     return self.exists(jobId)
             else:
                 return self.exists(jobId)
-
-        def getJobs():
+                
+        def deleteJob(jobId):
             if jobCache is not None:
-                return itervalues(jobCache)
+                if jobId in jobCache:
+                    del jobCache[jobId]
+            self.delete(jobId)
+            
+        def updateJobDescription(jobDescription):
+            jobCache[jobDescription.jobStoreID] = jobDescription
+            self.update(jobDescription)
+
+        def getJobDescriptions():
+            if jobCache is not None:
+                return jobCache.values()
             else:
                 return self.jobs()
 
-        # Iterate from the root jobGraph and collate all jobs that are reachable from it
-        # All other jobs returned by self.jobs() are orphaned and can be removed
-        reachableFromRoot = set()
+        def get_jobs_reachable_from_root() -> Set[Union[TemporaryID, str]]:
+            """
+            Traverse the job graph from the root job and return a flattened set of all active jobstore IDs.
 
-        def getConnectedJobs(jobGraph):
-            if jobGraph.jobStoreID in reachableFromRoot:
-                return
-            reachableFromRoot.add(jobGraph.jobStoreID)
-            # Traverse jobs in stack
-            for jobs in jobGraph.stack:
-                for successorJobStoreID in [x.jobStoreID for x in jobs]:
-                    if (successorJobStoreID not in reachableFromRoot
-                        and haveJob(successorJobStoreID)):
-                        getConnectedJobs(getJob(successorJobStoreID))
-            # Traverse service jobs
-            for jobs in jobGraph.services:
-                for serviceJobStoreID in [x.jobStoreID for x in jobs]:
-                    if haveJob(serviceJobStoreID):
-                        assert serviceJobStoreID not in reachableFromRoot
-                        reachableFromRoot.add(serviceJobStoreID)
+            Note: Jobs returned by self.jobs(), but not this function, are orphaned, and can be removed as dead jobs.
+            """
+            # Iterate from the root JobDescription and collate all jobs that are reachable from it.
+            root_job_description = self.loadRootJob()
+            reachable_from_root = set()
 
-        logger.debug("Checking job graph connectivity...")
-        getConnectedJobs(self.loadRootJob())
-        logger.debug("%d jobs reachable from root." % len(reachableFromRoot))
+            # Add first root job outside of the loop below.
+            reachable_from_root.add(root_job_description.jobStoreID)
+            # add all of root's linked service jobs as well
+            for service_jobstore_id in root_job_description.services:
+                if haveJob(service_jobstore_id):
+                    reachable_from_root.add(service_jobstore_id)
+
+            # Unprocessed means it might have successor jobs we need to add.
+            unprocessed_job_descriptions = [root_job_description]
+
+            while unprocessed_job_descriptions:
+                new_job_descriptions_to_process = []  # Reset.
+                for job_description in unprocessed_job_descriptions:
+                    for jobs in job_description.stack:
+                        for successor_jobstore_id in jobs:
+                            if successor_jobstore_id not in reachable_from_root and haveJob(successor_jobstore_id):
+                                successor_job_description = getJobDescription(successor_jobstore_id)
+
+                                # Add each successor job.
+                                reachable_from_root.add(successor_job_description.jobStoreID)
+                                # Add all of the successor's linked service jobs as well.
+                                for service_jobstore_id in successor_job_description.services:
+                                    if haveJob(service_jobstore_id):
+                                        reachable_from_root.add(service_jobstore_id)
+
+                                new_job_descriptions_to_process.append(successor_job_description)
+                unprocessed_job_descriptions = new_job_descriptions_to_process
+
+            logger.debug(f"{len(reachable_from_root)} jobs reachable from root.")
+            return reachable_from_root
+
+        reachable_from_root = get_jobs_reachable_from_root()
 
         # Cleanup jobs that are not reachable from the root, and therefore orphaned
-        jobsToDelete = [x for x in getJobs() if x.jobStoreID not in reachableFromRoot]
-        for jobGraph in jobsToDelete:
+        # TODO: Avoid reiterating reachable_from_root (which may be very large)
+        jobsToDelete = [x for x in getJobDescriptions() if x.jobStoreID not in reachable_from_root]
+        for jobDescription in jobsToDelete:
             # clean up any associated files before deletion
-            for fileID in jobGraph.filesToDelete:
+            for fileID in jobDescription.filesToDelete:
                 # Delete any files that should already be deleted
-                logger.warning("Deleting file '%s'. It is marked for deletion but has not yet been "
-                            "removed.", fileID)
+                logger.warning(f"Deleting file '{fileID}'. It is marked for deletion but has not yet been removed.")
                 self.deleteFile(fileID)
-            # Delete the job
-            self.delete(jobGraph.jobStoreID)
+            # Delete the job from us and the cache
+            deleteJob(jobDescription.jobStoreID)
 
-        jobGraphsReachableFromRoot = {id: getJob(id) for id in reachableFromRoot}
+        jobDescriptionsReachableFromRoot = {id: getJobDescription(id) for id in reachable_from_root}
 
         # Clean up any checkpoint jobs -- delete any successors it
-        # may have launched, and restore the job to a pristine
-        # state
+        # may have launched, and restore the job to a pristine state
         jobsDeletedByCheckpoints = set()
-        for jobGraph in [jG for jG in jobGraphsReachableFromRoot.values() if jG.checkpoint is not None]:
-            if jobGraph.jobStoreID in jobsDeletedByCheckpoints:
+        for jobDescription in [desc for desc in jobDescriptionsReachableFromRoot.values() if isinstance(desc, CheckpointJobDescription)]:
+            if jobDescription.jobStoreID in jobsDeletedByCheckpoints:
                 # This is a checkpoint that was nested within an
                 # earlier checkpoint, so it and all its successors are
                 # already gone.
                 continue
-            logger.debug("Restarting checkpointed job %s" % jobGraph)
-            deletedThisRound = jobGraph.restartCheckpoint(self)
-            jobsDeletedByCheckpoints |= set(deletedThisRound)
+            if jobDescription.checkpoint is not None:
+                # The checkpoint actually started and needs to be restarted
+                logger.debug("Restarting checkpointed job %s" % jobDescription)
+                deletedThisRound = jobDescription.restartCheckpoint(self)
+                jobsDeletedByCheckpoints |= set(deletedThisRound)
+                updateJobDescription(jobDescription)
         for jobID in jobsDeletedByCheckpoints:
-            del jobGraphsReachableFromRoot[jobID]
+            del jobDescriptionsReachableFromRoot[jobID]
 
         # Clean up jobs that are in reachable from the root
-        for jobGraph in jobGraphsReachableFromRoot.values():
-            # jobGraphs here are necessarily in reachable from root.
+        for jobDescription in jobDescriptionsReachableFromRoot.values():
+            # jobDescription here are necessarily in reachable from root.
 
-            changed = [False]  # This is a flag to indicate the jobGraph state has
+            changed = [False]  # This is a flag to indicate the jobDescription state has
             # changed
 
             # If the job has files to delete delete them.
-            if len(jobGraph.filesToDelete) != 0:
+            if len(jobDescription.filesToDelete) != 0:
                 # Delete any files that should already be deleted
-                for fileID in jobGraph.filesToDelete:
+                for fileID in jobDescription.filesToDelete:
                     logger.critical("Removing file in job store: %s that was "
                                     "marked for deletion but not previously removed" % fileID)
                     self.deleteFile(fileID)
-                jobGraph.filesToDelete = []
+                jobDescription.filesToDelete = []
                 changed[0] = True
 
             # For a job whose command is already executed, remove jobs from the stack that are
-            # already deleted. This cleans up the case that the jobGraph had successors to run,
+            # already deleted. This cleans up the case that the jobDescription had successors to run,
             # but had not been updated to reflect this.
-            if jobGraph.command is None:
-                stackSizeFn = lambda: sum(map(len, jobGraph.stack))
+            if jobDescription.command is None:
+                stackSizeFn = lambda: sum(map(len, jobDescription.stack))
                 startStackSize = stackSizeFn()
                 # Remove deleted jobs
-                jobGraph.stack = [[y for y in x if self.exists(y.jobStoreID)] for x in jobGraph.stack]
-                # Remove empty stuff from the stack
-                jobGraph.stack = [x for x in jobGraph.stack if len(x) > 0]
+                jobDescription.filterSuccessors(haveJob)
                 # Check if anything got removed
                 if stackSizeFn() != startStackSize:
                     changed[0] = True
@@ -598,67 +634,65 @@ class AbstractJobStore(with_metaclass(ABCMeta, object)):
                 # Make a new flag
                 newFlag = self.getEmptyFileStoreID(jobStoreID, cleanup=False)
 
-                # Load the jobGraph for the service and initialise the link
-                serviceJobGraph = getJob(jobStoreID)
+                # Load the jobDescription for the service and initialise the link
+                serviceJobDescription = getJobDescription(jobStoreID)
+                
+                # Make sure it really is a service
+                assert isinstance(serviceJobDescription, ServiceJobDescription)
 
                 if flag == 1:
                     logger.debug("Recreating a start service flag for job: %s, flag: %s",
                                  jobStoreID, newFlag)
-                    serviceJobGraph.startJobStoreID = newFlag
+                    serviceJobDescription.startJobStoreID = newFlag
                 elif flag == 2:
                     logger.debug("Recreating a terminate service flag for job: %s, flag: %s",
                                  jobStoreID, newFlag)
-                    serviceJobGraph.terminateJobStoreID = newFlag
+                    serviceJobDescription.terminateJobStoreID = newFlag
                 else:
                     logger.debug("Recreating a error service flag for job: %s, flag: %s",
                                  jobStoreID, newFlag)
                     assert flag == 3
-                    serviceJobGraph.errorJobStoreID = newFlag
+                    serviceJobDescription.errorJobStoreID = newFlag
 
                 # Update the service job on disk
-                self.update(serviceJobGraph)
+                updateJobDescription(serviceJobDescription)
 
                 changed[0] = True
 
                 return newFlag
 
-            servicesSizeFn = lambda: sum(map(len, jobGraph.services))
+            servicesSizeFn = lambda: len(jobDescription.services)
             startServicesSize = servicesSizeFn()
 
-            def replaceFlagsIfNeeded(serviceJobNode):
-                serviceJobNode.startJobStoreID = subFlagFile(serviceJobNode.jobStoreID, serviceJobNode.startJobStoreID, 1)
-                serviceJobNode.terminateJobStoreID = subFlagFile(serviceJobNode.jobStoreID, serviceJobNode.terminateJobStoreID, 2)
-                serviceJobNode.errorJobStoreID = subFlagFile(serviceJobNode.jobStoreID, serviceJobNode.errorJobStoreID, 3)
+            def replaceFlagsIfNeeded(serviceJobDescription):
+                # Make sure it really is a service
+                assert isinstance(serviceJobDescription, ServiceJobDescription)
+                serviceJobDescription.startJobStoreID = subFlagFile(serviceJobDescription.jobStoreID, serviceJobDescription.startJobStoreID, 1)
+                serviceJobDescription.terminateJobStoreID = subFlagFile(serviceJobDescription.jobStoreID, serviceJobDescription.terminateJobStoreID, 2)
+                serviceJobDescription.errorJobStoreID = subFlagFile(serviceJobDescription.jobStoreID, serviceJobDescription.errorJobStoreID, 3)
 
-            # jobGraph.services is a list of lists containing serviceNodes
             # remove all services that no longer exist
-            services = jobGraph.services
-            jobGraph.services = []
-            for serviceList in services:
-                existingServices = [service for service in serviceList if self.exists(service.jobStoreID)]
-                if existingServices:
-                    jobGraph.services.append(existingServices)
+            jobDescription.filterServiceHosts(haveJob)
 
-            list(map(lambda serviceList: list(map(replaceFlagsIfNeeded, serviceList)), jobGraph.services))
+            for serviceID in jobDescription.services:
+                replaceFlagsIfNeeded(getJobDescription(serviceID)) 
 
             if servicesSizeFn() != startServicesSize:
                 changed[0] = True
 
-            # Reset the retry count of the jobGraph
-            if jobGraph.remainingRetryCount != self._defaultTryCount():
-                jobGraph.remainingRetryCount = self._defaultTryCount()
-                changed[0] = True
+            # Reset the try count of the JobDescription so it will use the default.
+            changed[0] |= jobDescription.clearRemainingTryCount()
 
             # This cleans the old log file which may
-            # have been left if the jobGraph is being retried after a jobGraph failure.
-            if jobGraph.logJobStoreFileID != None:
-                self.deleteFile(jobGraph.logJobStoreFileID)
-                jobGraph.logJobStoreFileID = None
+            # have been left if the job is being retried after a failure.
+            if jobDescription.logJobStoreFileID != None:
+                self.deleteFile(jobDescription.logJobStoreFileID)
+                jobDescription.logJobStoreFileID = None
                 changed[0] = True
 
             if changed[0]:  # Update, but only if a change has occurred
-                logger.critical("Repairing job: %s" % jobGraph.jobStoreID)
-                self.update(jobGraph)
+                logger.critical("Repairing job: %s" % jobDescription.jobStoreID)
+                updateJobDescription(jobDescription)
 
         # Remove any crufty stats/logging files from the previous run
         logger.debug("Discarding old statistics and logs...")
@@ -678,30 +712,42 @@ class AbstractJobStore(with_metaclass(ABCMeta, object)):
     # The following methods deal with creating/loading/updating/writing/checking for the
     # existence of jobs
     ##########################################
+    
+    @abstractmethod
+    def assignID(self, jobDescription):
+        """
+        Get a new jobStoreID to be used by the described job, and assigns it to the JobDescription.
+        
+        Files associated with the assigned ID will be accepted even if the JobDescription has never been created or updated.
+        
+        :param toil.job.JobDescription jobDescription: The JobDescription to give an ID to
+        """
+        raise NotImplementedError()
 
     @contextmanager
     def batch(self):
         """
-        All calls to create() with this context manager active will be performed in a batch
-        after the context manager is released.
-
+        If supported by the batch system, calls to create() with this context
+        manager active will be performed in a batch after the context manager
+        is released.
         :rtype: None
         """
         yield
 
     @abstractmethod
-    def create(self, jobNode):
+    def create(self, jobDescription):
         """
-        Creates a job graph from the given job node & writes it to the job store.
-
-        :rtype: toil.jobGraph.JobGraph
+        Writes the given JobDescription to the job store. The job must have an ID assigned already.
+       
+        :return: The JobDescription passed.
+        :rtype: toil.job.JobDescription
         """
         raise NotImplementedError()
-
+        
     @abstractmethod
     def exists(self, jobStoreID):
         """
-        Indicates whether the job with the specified jobStoreID exists in the job store
+        Indicates whether a description of the job with the specified jobStoreID exists in the job store
 
         :rtype: bool
         """
@@ -746,30 +792,36 @@ class AbstractJobStore(with_metaclass(ABCMeta, object)):
     @abstractmethod
     def load(self, jobStoreID):
         """
-        Loads the job referenced by the given ID and returns it.
+        Loads the description of the job referenced by the given ID, assigns it
+        the job store's config, and returns it.
+        
+        May declare the job to have failed (see
+        :meth:`toil.job.JobDescription.setupJobAfterFailure`) if there is
+        evidence of a failed update attempt. 
 
         :param str jobStoreID: the ID of the job to load
 
         :raise NoSuchJobException: if there is no job with the given ID
 
-        :rtype: toil.jobGraph.JobGraph
+        :rtype: toil.job.JobDescription
         """
         raise NotImplementedError()
 
     @abstractmethod
-    def update(self, job):
+    def update(self, jobDescription):
         """
-        Persists the job in this store atomically.
+        Persists changes to the state of the given JobDescription in this store atomically.
 
-        :param toil.jobGraph.JobGraph job: the job to write to this job store
+        :param toil.job.JobDescription job: the job to write to this job store
         """
         raise NotImplementedError()
 
     @abstractmethod
     def delete(self, jobStoreID):
         """
-        Removes from store atomically, can not then subsequently call load(), write(), update(),
-        etc. with the job.
+        Removes the JobDescription from the store atomically. You may not then
+        subsequently call load(), write(), update(), etc. with the same
+        jobStoreID or any JobDescription bearing it.
 
         This operation is idempotent, i.e. deleting a job twice or deleting a non-existent job
         will succeed silently.
@@ -780,14 +832,15 @@ class AbstractJobStore(with_metaclass(ABCMeta, object)):
 
     def jobs(self):
         """
-        Best effort attempt to return iterator on all jobs in the store. The iterator may not
-        return all jobs and may also contain orphaned jobs that have already finished successfully
-        and should not be rerun. To guarantee you get any and all jobs that can be run instead
+        Best effort attempt to return iterator on JobDescriptions for all jobs
+        in the store. The iterator may not return all jobs and may also contain
+        orphaned jobs that have already finished successfully and should not be
+        rerun. To guarantee you get any and all jobs that can be run instead
         construct a more expensive ToilState object
 
         :return: Returns iterator on jobs in the store. The iterator may or may not contain all jobs and may contain
                  invalid jobs
-        :rtype: Iterator[toil.jobGraph.JobGraph]
+        :rtype: Iterator[toil.job.jobDescription]
         """
         raise NotImplementedError()
 
@@ -804,6 +857,9 @@ class AbstractJobStore(with_metaclass(ABCMeta, object)):
         not appear in the jobStore until the write has successfully completed.
 
         :param str localFilePath: the path to the local file that will be uploaded to the job store.
+               The last path component (basename of the file) will remain
+               associated with the file in the file store, if supported, so
+               that the file can be searched for by name or name glob. 
 
         :param str jobStoreID: the id of a job, or None. If specified, the may be associated
                with that job in a job-store-specific way. This may influence the returned ID.
@@ -827,7 +883,7 @@ class AbstractJobStore(with_metaclass(ABCMeta, object)):
 
     @abstractmethod
     @contextmanager
-    def writeFileStream(self, jobStoreID=None, cleanup=False):
+    def writeFileStream(self, jobStoreID=None, cleanup=False, basename=None):
         """
         Similar to writeFile, but returns a context manager yielding a tuple of
         1) a file handle which can be written to and 2) the ID of the resulting
@@ -842,6 +898,10 @@ class AbstractJobStore(with_metaclass(ABCMeta, object)):
         :param bool cleanup: Whether to attempt to delete the file when the job
                whose jobStoreID was given as jobStoreID is deleted with
                jobStore.delete(job). If jobStoreID was not given, does nothing.
+               
+        :param str basename: If supported by the implementation, use the given
+               file basename so that when searching the job store with a query
+               matching that basename, the file will be detected.
 
         :raise ConcurrentFileModificationException: if the file was modified concurrently during
                an invocation of this method
@@ -857,7 +917,7 @@ class AbstractJobStore(with_metaclass(ABCMeta, object)):
         raise NotImplementedError()
 
     @abstractmethod
-    def getEmptyFileStoreID(self, jobStoreID=None, cleanup=False):
+    def getEmptyFileStoreID(self, jobStoreID=None, cleanup=False, basename=None):
         """
         Creates an empty file in the job store and returns its ID.
         Call to fileExists(getEmptyFileStoreID(jobStoreID)) will return True.
@@ -868,6 +928,10 @@ class AbstractJobStore(with_metaclass(ABCMeta, object)):
         :param bool cleanup: Whether to attempt to delete the file when the job
                whose jobStoreID was given as jobStoreID is deleted with
                jobStore.delete(job). If jobStoreID was not given, does nothing.
+               
+        :param str basename: If supported by the implementation, use the given
+               file basename so that when searching the job store with a query
+               matching that basename, the file will be detected.
 
         :return: a jobStoreFileID that references the newly created file and can be used to reference the
                  file in the future.
@@ -1069,40 +1133,48 @@ class AbstractJobStore(with_metaclass(ABCMeta, object)):
             raise ValueError("Not a valid shared file name: '%s'." % sharedFileName)
 
 
-class JobStoreSupport(with_metaclass(ABCMeta, AbstractJobStore)):
+class JobStoreSupport(AbstractJobStore, metaclass=ABCMeta):
     @classmethod
     def _supportsUrl(cls, url, export=False):
         return url.scheme.lower() in ('http', 'https', 'ftp') and not export
 
     @classmethod
+    @retry(errors=[BadStatusLine] + [
+                         ErrorCondition(
+                             error=HTTPError,
+                             error_codes=[408, 500, 503]
+                         )
+                     ])
     def getSize(cls, url):
         if url.scheme.lower() == 'ftp':
             return None
-        for attempt in retry_http():
-            with attempt:
-                with closing(urlopen(url.geturl())) as readable:
-                    # just read the header for content length
-                    size = readable.info().get('content-length')
-                    return int(size) if size is not None else None
+        with closing(urlopen(url.geturl())) as readable:
+            # just read the header for content length
+            size = readable.info().get('content-length')
+            return int(size) if size is not None else None
 
     @classmethod
+    @retry(errors=[BadStatusLine] + [
+                         ErrorCondition(
+                             error=HTTPError,
+                             error_codes=[408, 500, 503]
+                         )
+                     ])
     def _readFromUrl(cls, url, writable):
-        for attempt in retry_http():
-            # We can only retry on errors that happen as responses to the request.
-            # If we start getting file data, and the connection drops, we fail.
-            # So we don't have to worry about writing the start of the file twice.
-            with attempt:
-                with closing(urlopen(url.geturl())) as readable:
-                    # Make something to count the bytes we get
-                    # We need to put the actual count in a container so our
-                    # nested function can modify it without creating its own
-                    # local with the same name.
-                    size = [0]
-                    def count(l):
-                        size[0] += l
-                    counter = WriteWatchingStream(writable)
-                    counter.onWrite(count)
-                    
-                    # Do the download
-                    shutil.copyfileobj(readable, counter)
-                    return size[0]
+        # We can only retry on errors that happen as responses to the request.
+        # If we start getting file data, and the connection drops, we fail.
+        # So we don't have to worry about writing the start of the file twice.
+        with closing(urlopen(url.geturl())) as readable:
+            # Make something to count the bytes we get
+            # We need to put the actual count in a container so our
+            # nested function can modify it without creating its own
+            # local with the same name.
+            size = [0]
+            def count(l):
+                size[0] += l
+            counter = WriteWatchingStream(writable)
+            counter.onWrite(count)
+
+            # Do the download
+            shutil.copyfileobj(readable, counter)
+            return size[0], False
