@@ -1,4 +1,4 @@
-# Copyright (C) 2015-2016 Regents of the University of California
+# Copyright (C) 2015-2021 Regents of the University of California
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -11,39 +11,33 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-
-# python 2/3 compatibility
-from __future__ import absolute_import
-from builtins import range
-
-# standard library
-from contextlib import contextmanager
-import logging
-import random
-import shutil
-import os
-import re
-import tempfile
-import stat
 import errno
+import logging
+import os
+import stat
+import pickle
+import random
+import re
+import shutil
+import stat
+import tempfile
 import time
-try:
-    import cPickle as pickle
-except ImportError:
-    import pickle
+import uuid
+from contextlib import contextmanager
 
-# toil dependencies
 from toil.fileStores import FileID
-from toil.lib.bioio import absSymPath
-from toil.lib.misc import robust_rmtree, AtomicFileCreate, atomic_copy, atomic_copyobj
+from toil.job import TemporaryID
 from toil.jobStores.abstractJobStore import (AbstractJobStore,
-                                             NoSuchJobException,
-                                             NoSuchFileException,
                                              JobStoreExistsException,
+                                             NoSuchFileException,
+                                             NoSuchJobException,
                                              NoSuchJobStoreException)
-from toil.jobGraph import JobGraph
+from toil.lib.misc import (AtomicFileCreate,
+                           atomic_copy,
+                           atomic_copyobj,
+                           robust_rmtree)
 
-logger = logging.getLogger( __name__ )
+logger = logging.getLogger(__name__)
 
 
 class FileJobStore(AbstractJobStore):
@@ -75,7 +69,7 @@ class FileJobStore(AbstractJobStore):
                            subdirectories
         """
         super(FileJobStore, self).__init__()
-        self.jobStoreDir = absSymPath(path)
+        self.jobStoreDir = os.path.abspath(path)
         logger.debug("Path to job store directory is '%s'.", self.jobStoreDir)
 
         # Directory where actual job files go, and their job-associated temp files
@@ -130,37 +124,47 @@ class FileJobStore(AbstractJobStore):
     # existence of jobs
     ##########################################
 
-    def create(self, jobNode):
+    def assignID(self, jobDescription):
         # Get the job's name. We want to group jobs with the same name together.
         # This will be e.g. the function name for wrapped-function jobs.
         # Make sure to render it filename-safe
-        usefulFilename = self._makeStringFilenameSafe(jobNode.jobName)
-
+        usefulFilename = self._makeStringFilenameSafe(jobDescription.jobName)
+        
         # Make a unique temp directory under a directory for this job name,
         # possibly sprayed across multiple levels of subdirectories.
         absJobDir = tempfile.mkdtemp(prefix=self.JOB_DIR_PREFIX,
                                      dir=self._getArbitraryJobsDirForName(usefulFilename))
-        # Make the job to save
-        job = JobGraph.fromJobNode(jobNode, jobStoreID=self._getJobIdFromDir(absJobDir),
-                                   tryCount=self._defaultTryCount())
-        if hasattr(self, "_batchedJobGraphs") and self._batchedJobGraphs is not None:
+                                     
+        jobDescription.jobStoreID = self._getJobIdFromDir(absJobDir)
+        
+    def create(self, jobDescription):
+        if hasattr(self, "_batchedUpdates") and self._batchedUpdates is not None:
             # Save it later
-            self._batchedJobGraphs.append(job)
+            self._batchedUpdates.append(jobDescription)
         else:
             # Save it now
-            self.update(job)
-        return job
+            self.update(jobDescription)
+        return jobDescription
 
     @contextmanager
     def batch(self):
-        self._batchedJobGraphs = []
+        self._batchedUpdates = []
         yield
-        for jobGraph in self._batchedJobGraphs:
-            self.update(jobGraph)
-        self._batchedJobGraphs = None
+        for jobDescription in self._batchedUpdates:
+            self.update(jobDescription)
+        self._batchedUpdates = None
 
-    def waitForExists(self, jobStoreID, maxTries=35, sleepTime=1):
-        """Spin-wait and block for a file to appear before returning False if it does not.
+    def _waitForExists(self, jobStoreID, maxTries=35, sleepTime=1):
+        """
+        Spin-wait and block for a job to appear before returning
+        False if it does not.
+        """
+        return self._waitForFile(self._getJobFileName(jobStoreID), maxTries=maxTries, sleepTime=sleepTime)
+
+    def _waitForFile(self, fileName, maxTries=35, sleepTime=1):
+        """
+        Spin-wait and block for a file or directory to appear before returning
+        False if it does not.
 
         The total max wait time is maxTries * sleepTime. The current default is
         tuned to match Linux NFS defaults where the client's cache of the directory
@@ -171,16 +175,16 @@ class FileJobStore(AbstractJobStore):
         The warning will be sent to the log only on the first retry.
 
         In practice, the need for retries happens rarely, but it does happen
-        over the course of large workflows with a jobStore on a busy NFS."""
+        over the course of large workflows with a jobStore on a busy NFS.
+        """
         for iTry in range(1,maxTries+1):
-            jobFile = self._getJobFileName(jobStoreID)
-            if os.path.exists(jobFile):
+            if os.path.exists(fileName):
                 return True
             if iTry >= maxTries:
                 return False
             elif iTry == 1:
-                logger.warning(("Job file `{}` for job `{}` does not exist (yet). We will try #{} more times with {}s "
-                        "intervals.").format(jobFile, jobStoreID, maxTries - iTry, sleepTime))
+                logger.warning(("Path `{}` does not exist (yet). We will try #{} more times with {}s "
+                        "intervals.").format(fileName, maxTries - iTry, sleepTime))
             time.sleep(sleepTime)
         return False
 
@@ -203,20 +207,27 @@ class FileJobStore(AbstractJobStore):
             raise NoSuchFileException(sharedFileName)
 
     def load(self, jobStoreID):
-        self._checkJobStoreId(jobStoreID)
+        self._checkJobStoreIdExists(jobStoreID)
         # Load a valid version of the job
         jobFile = self._getJobFileName(jobStoreID)
         with open(jobFile, 'rb') as fileHandle:
             job = pickle.load(fileHandle)
+        
+        # Pass along the current config, which is the JobStore's responsibility.
+        job.assignConfig(self.config)
+            
         # The following cleans up any issues resulting from the failure of the
         # job during writing by the batch system.
         if os.path.isfile(jobFile + ".new"):
             logger.warning("There was a .new file for the job: %s", jobStoreID)
             os.remove(jobFile + ".new")
-            job.setupJobAfterFailure(self.config)
+            job.setupJobAfterFailure()
         return job
 
     def update(self, job):
+        assert job.jobStoreID is not None, f"Tried to update job {job} without an ID"
+        assert not isinstance(job.jobStoreID, TemporaryID), f"Tried to update job {job} without an assigned ID"
+    
         # The job is serialised to a file suffixed by ".new"
         # The file is then moved to its correct path.
         # Atomicity guarantees use the fact the underlying file systems "move"
@@ -277,12 +288,13 @@ class FileJobStore(AbstractJobStore):
     def _importFile(self, otherCls, url, sharedFileName=None, hardlink=False):
         if issubclass(otherCls, FileJobStore):
             if sharedFileName is None:
+                executable = os.stat(url.path).st_mode & stat.S_IXUSR != 0
                 absPath = self._getUniqueFilePath(url.path)  # use this to get a valid path to write to in job store
                 with self.optionalHardCopy(hardlink):
                     self._copyOrLink(url, absPath)
                 # TODO: os.stat(absPath).st_size consistently gives values lower than
                 # getDirSizeRecursively()
-                return FileID(self._getFileIdFromPath(absPath), os.stat(absPath).st_size)
+                return FileID(self._getFileIdFromPath(absPath), os.stat(absPath).st_size, executable)
             else:
                 self._requireValidSharedFileName(sharedFileName)
                 path = self._getSharedFilePath(sharedFileName)
@@ -300,7 +312,10 @@ class FileJobStore(AbstractJobStore):
             if self.moveExports:
                 self._move_and_linkback(srcPath, destPath)
             else:
-                atomic_copy(srcPath, destPath)
+                executable = False
+                if getattr(jobStoreFileID, 'executable', False):
+                    executable = jobStoreFileID.executable
+                atomic_copy(srcPath, destPath, executable=executable)
         else:
             super(FileJobStore, self)._defaultExportFile(otherCls, jobStoreFileID, url)
 
@@ -327,11 +342,12 @@ class FileJobStore(AbstractJobStore):
         with open(cls._extractPathFromUrl(url), 'rb') as readable:
             shutil.copyfileobj(readable, writable, length=cls.BUFFER_SIZE)
             # Return the number of bytes we read when we reached EOF.
-            return readable.tell()
+            executable = os.stat(readable.name).st_mode & stat.S_IXUSR
+            return readable.tell(), executable
         
 
     @classmethod
-    def _writeToUrl(cls, readable, url):
+    def _writeToUrl(cls, readable, url, executable=False):
         """
         Writes the contents of a file to a source (writes readable to url)
         using a ~10Mb buffer.
@@ -340,7 +356,9 @@ class FileJobStore(AbstractJobStore):
         :param object readable: An open file object to read from.
         """
         # we use a ~10Mb buffer to improve speed
-        atomic_copyobj(readable, cls._extractPathFromUrl(url), length=cls.BUFFER_SIZE)
+        atomic_copyobj(readable, cls._extractPathFromUrl(url), length=cls.BUFFER_SIZE,
+                        executable=executable)
+
 
     @staticmethod
     def _extractPathFromUrl(url):
@@ -355,7 +373,7 @@ class FileJobStore(AbstractJobStore):
     def _supportsUrl(cls, url, export=False):
         return url.scheme.lower() == 'file'
 
-    def _makeStringFilenameSafe(self, arbitraryString):
+    def _makeStringFilenameSafe(self, arbitraryString, maxLength=240):
         """
         Given an arbitrary string, produce a filename-safe though not
         necessarily unique string based on it.
@@ -364,6 +382,9 @@ class FileJobStore(AbstractJobStore):
         other nonempty filename-safe string.
 
         :param str arbitraryString: An arbitrary string
+        :param int maxLength: Maximum length of the result, to keep it plus
+                              any prefix or suffix under the filesystem's
+                              path component length limit
 
         :return: A filename-safe string
         """
@@ -378,8 +399,8 @@ class FileJobStore(AbstractJobStore):
         if len(parts) == 0:
             parts.append("UNPRINTABLE")
 
-        # Glue it all together
-        return '_'.join(parts)
+        # Glue it all together, and truncate to length
+        return '_'.join(parts)[:maxLength]
 
     def writeFile(self, localFilePath, jobStoreID=None, cleanup=False):
         absPath = self._getUniqueFilePath(localFilePath, jobStoreID, cleanup)
@@ -388,8 +409,10 @@ class FileJobStore(AbstractJobStore):
         return relPath
 
     @contextmanager
-    def writeFileStream(self, jobStoreID=None, cleanup=False):
-        absPath = self._getUniqueFilePath('stream', jobStoreID, cleanup)
+    def writeFileStream(self, jobStoreID=None, cleanup=False, basename=None):
+        if not basename:
+            basename = 'stream'
+        absPath = self._getUniqueFilePath(basename, jobStoreID, cleanup)
         relPath = self._getFileIdFromPath(absPath)
         with open(absPath, 'wb') as f:
             # Don't yield while holding an open file descriptor to the temp
@@ -397,8 +420,8 @@ class FileJobStore(AbstractJobStore):
             # to clean ourselves up, somehow, for certain workloads.
             yield f, relPath
 
-    def getEmptyFileStoreID(self, jobStoreID=None, cleanup=False):
-        with self.writeFileStream(jobStoreID, cleanup) as (fileHandle, jobStoreFileID):
+    def getEmptyFileStoreID(self, jobStoreID=None, cleanup=False, basename=None):
+        with self.writeFileStream(jobStoreID, cleanup, basename) as (fileHandle, jobStoreFileID):
             return jobStoreFileID
 
     def updateFile(self, jobStoreFileID, localFilePath):
@@ -563,11 +586,11 @@ class FileJobStore(AbstractJobStore):
 
     def writeStatsAndLogging(self, statsAndLoggingString):
         # Temporary files are placed in the stats directory tree
-        fd, tempStatsFile = tempfile.mkstemp(prefix="stats", suffix=".new", dir=self._getArbitraryStatsDir())
+        tempStatsFileName = "stats" + str(uuid.uuid4().hex) + ".new"
+        tempStatsFile = os.path.join(self._getArbitraryStatsDir(), tempStatsFileName)
         writeFormat = 'w' if isinstance(statsAndLoggingString, str) else 'wb'
         with open(tempStatsFile, writeFormat) as f:
             f.write(statsAndLoggingString)
-        os.close(fd)
         os.rename(tempStatsFile, tempStatsFile[:-4])  # This operation is atomic
 
     def readStatsAndLogging(self, callback, readAll=False):
@@ -610,7 +633,7 @@ class FileJobStore(AbstractJobStore):
 
     def _getJobFileName(self, jobStoreID):
         """
-        Return the path to the file containing the serialised JobGraph instance for the given
+        Return the path to the file containing the serialised JobDescription instance for the given
         job.
 
         :rtype: str
@@ -645,11 +668,25 @@ class FileJobStore(AbstractJobStore):
 
         return os.path.join(self.jobFilesDir, jobStoreID, "cleanup")
 
-    def _checkJobStoreId(self, jobStoreID):
+    def _checkJobStoreIdAssigned(self, jobStoreID):
+        """
+        Do nothing if the given job store ID has been assigned by
+        :meth:`assignID`, and the corresponding job has not yet been
+        deleted, even if the JobDescription hasn't yet been saved for the first
+        time.
+        
+        If the ID has not been assigned, raises a NoSuchJobException.
+        """
+        
+        if not self._waitForFile(self._getJobDirFromId(jobStoreID)):
+            raise NoSuchJobException(jobStoreID)
+
+
+    def _checkJobStoreIdExists(self, jobStoreID):
         """
         Raises a NoSuchJobException if the job with ID jobStoreID does not exist.
         """
-        if not self.waitForExists(jobStoreID,30):
+        if not self._waitForExists(jobStoreID, 30):
             raise NoSuchJobException(jobStoreID)
 
     def _getFilePathFromId(self, jobStoreFileID):
@@ -891,7 +928,7 @@ class FileJobStore(AbstractJobStore):
             # Make a temporary file within the job's files directory
 
             # Make sure the job is legit
-            self._checkJobStoreId(jobStoreID)
+            self._checkJobStoreIdAssigned(jobStoreID)
             # Find where all its created files should live, depending on if
             # they need to go away when the job is deleted or not.
             jobFilesDir = self._getJobFilesDir(jobStoreID) if not cleanup \
@@ -902,7 +939,11 @@ class FileJobStore(AbstractJobStore):
             os.makedirs(jobFilesDir, exist_ok=True)
 
             # Then make a temp directory inside it
-            return tempfile.mkdtemp(prefix='file-', dir=jobFilesDir)
+            filesDir = os.path.join(jobFilesDir, 'file-' + uuid.uuid4().hex)
+            os.mkdir(filesDir)
+            return filesDir
         else:
             # Make a temporary file within the non-job-associated files hierarchy
-            return tempfile.mkdtemp(prefix='file-', dir=self._getArbitraryFilesDir())
+            filesDir = os.path.join(self._getArbitraryFilesDir(), 'file-' + uuid.uuid4().hex)
+            os.mkdir(filesDir)
+            return filesDir
