@@ -1,30 +1,31 @@
-# coding=utf-8
-from six import iteritems
+# Copyright (C) 2015-2021 Regents of the University of California
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
 import json
-import os
-import urllib
-import re
 import logging
-import inspect
-
-try:
-    from urllib.parse import unquote
-except ImportError:
-    from urllib import unquote
+import os
+import re
+from urllib.parse import unquote
 
 from boto import iam, sns, sqs, vpc
 from boto.exception import BotoServerError
 from boto.s3.connection import S3Connection
-from boto.sqs.connection import SQSConnection
-from boto.sns.connection import SNSConnection
-from boto.vpc import VPCConnection
-from boto.iam.connection import IAMConnection
 from boto.utils import get_instance_metadata
 
-from toil.lib.memoize import memoize
 from toil.lib.ec2 import UserError
+from toil.lib.memoize import memoize
 
-log = logging.getLogger(__name__)
+logger = logging.getLogger(__name__)
 
 
 class Context(object):
@@ -480,7 +481,7 @@ class Context(object):
         try:
             return self.iam.get_user().user_name
         except BaseException:
-            log.warning("IAMConnection.get_user() failed.", exc_info=True)
+            logger.warning("IAMConnection.get_user() failed.", exc_info=True)
             return None
 
     current_user_placeholder = '__me__'
@@ -556,7 +557,7 @@ class Context(object):
             delete_policy(entity_name, policy_name)
 
         # Create expected policies
-        for policy_name, policy in iteritems(policies):
+        for policy_name, policy in policies.items():
             current_policy = None
             try:
                 current_policy = json.loads(unquote(
