@@ -1,12 +1,12 @@
-import random
-from math import sqrt
 import logging
 import os
+import stat
+import random
 import shutil
+import subprocess
 import sys
 import uuid
 from contextlib import contextmanager
-import subprocess
 
 logger = logging.getLogger(__name__)
 
@@ -67,106 +67,6 @@ def robust_rmtree(path):
             # File went away
             return
 
-def mean(xs):
-    """
-    Return the mean value of a sequence of values.
-
-    >>> mean([2,4,4,4,5,5,7,9])
-    5.0
-    >>> mean([9,10,11,7,13])
-    10.0
-    >>> mean([1,1,10,19,19])
-    10.0
-    >>> mean([10,10,10,10,10])
-    10.0
-    >>> mean([1,"b"])
-    Traceback (most recent call last):
-      ...
-    ValueError: Input can't have non-numeric elements
-    >>> mean([])
-    Traceback (most recent call last):
-      ...
-    ValueError: Input can't be empty
-    """
-    try:
-        return sum(xs) / float(len(xs))
-    except TypeError:
-        raise ValueError("Input can't have non-numeric elements")
-    except ZeroDivisionError:
-        raise ValueError("Input can't be empty")
-
-
-def std_dev(xs):
-    """
-    Returns the standard deviation of the given iterable of numbers.
-
-    From http://rosettacode.org/wiki/Standard_deviation#Python
-
-    An empty list, or a list with non-numeric elements will raise a TypeError.
-
-    >>> std_dev([2,4,4,4,5,5,7,9])
-    2.0
-
-    >>> std_dev([9,10,11,7,13])
-    2.0
-
-    >>> std_dev([1,1,10,19,19])
-    8.049844718999243
-
-    >>> std_dev({1,1,10,19,19}) == std_dev({19,10,1})
-    True
-
-    >>> std_dev([10,10,10,10,10])
-    0.0
-
-    >>> std_dev([1,"b"])
-    Traceback (most recent call last):
-    ...
-    ValueError: Input can't have non-numeric elements
-
-    >>> std_dev([])
-    Traceback (most recent call last):
-    ...
-    ValueError: Input can't be empty
-    """
-    m = mean(xs)  # this checks our pre-conditions, too
-    return sqrt(sum((x - m) ** 2 for x in xs) / float(len(xs)))
-
-
-def partition_seq(seq, size):
-    """
-    Splits a sequence into an iterable of subsequences. All subsequences are of the given size,
-    except the last one, which may be smaller. If the input list is modified while the returned
-    list is processed, the behavior of the program is undefined.
-
-    :param seq: the list to split
-    :param size: the desired size of the sublists, must be > 0
-    :type size: int
-    :return: an iterable of sublists
-
-    >>> list(partition_seq("",1))
-    []
-    >>> list(partition_seq("abcde",2))
-    ['ab', 'cd', 'e']
-    >>> list(partition_seq("abcd",2))
-    ['ab', 'cd']
-    >>> list(partition_seq("abcde",1))
-    ['a', 'b', 'c', 'd', 'e']
-    >>> list(partition_seq("abcde",0))
-    Traceback (most recent call last):
-    ...
-    ValueError: Size must be greater than 0
-    >>> l=[1,2,3,4]
-    >>> i = iter( partition_seq(l,2) )
-    >>> l.pop(0)
-    1
-    >>> next(i)
-    [2, 3]
-    """
-    if size < 1:
-        raise ValueError('Size must be greater than 0')
-    return (seq[pos:pos + size] for pos in range(0, len(seq), size))
-
 
 def truncExpBackoff():
     # as recommended here https://forums.aws.amazon.com/thread.jspa?messageID=406788#406788
@@ -219,16 +119,21 @@ def AtomicFileCreate(final_path, keep=False):
                 pass
         raise
 
-def atomic_copy(src_path, dest_path):
+def atomic_copy(src_path, dest_path, executable=False):
     """Copy a file using posix atomic creations semantics."""
     with AtomicFileCreate(dest_path) as dest_path_tmp:
         shutil.copyfile(src_path, dest_path_tmp)
+        if executable:
+            os.chmod(dest_path_tmp, os.stat(dest_path_tmp).st_mode | stat.S_IXUSR)
 
-def atomic_copyobj(src_fh, dest_path, length=16384):
+def atomic_copyobj(src_fh, dest_path, length=16384, executable=False):
     """Copy an open file using posix atomic creations semantics."""
     with AtomicFileCreate(dest_path) as dest_path_tmp:
         with open(dest_path_tmp, 'wb') as dest_path_fh:
             shutil.copyfileobj(src_fh, dest_path_fh, length=length)
+        if executable:
+            os.chmod(dest_path_tmp, os.stat(dest_path_tmp).st_mode | stat.S_IXUSR)
+
 
 class WriteWatchingStream(object):
     """
@@ -294,7 +199,7 @@ class CalledProcessErrorStderr(subprocess.CalledProcessError):
     """Version of CalledProcessError that include stderr in the error message if it is set"""
 
     def __str__(self):
-        if (self.returncode and (self.returncode < 0)) or (self.stderr is None):
+        if (self.returncode < 0) or (self.stderr is None):
             return str(super())
         else:
             err = self.stderr if isinstance(self.stderr, str) else self.stderr.decode("ascii", errors="replace")
