@@ -1,4 +1,4 @@
-# Copyright (C) 2015 UCSC Computational Genomics Lab
+# Copyright (C) 2015-2021 Regents of the University of California
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -11,22 +11,21 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-from builtins import range
-import os
-import time
-import threading
 import json
-import requests
-import uuid
 import logging
+import os
+import threading
+import time
+import uuid
 
-from libcloud.compute.types import Provider
-from libcloud.compute.providers import get_driver
+import requests
 from libcloud.compute.drivers.gce import GCEFailedNode
+from libcloud.compute.providers import get_driver
+from libcloud.compute.types import Provider
 
-from toil.provisioners.abstractProvisioner import AbstractProvisioner, Shape
-from toil.provisioners import NoSuchClusterException
 from toil.jobStores.googleJobStore import GoogleJobStore
+from toil.provisioners import NoSuchClusterException
+from toil.provisioners.abstractProvisioner import AbstractProvisioner, Shape
 from toil.provisioners.node import Node
 
 logger = logging.getLogger(__name__)
@@ -135,8 +134,7 @@ class GCEProvisioner(AbstractProvisioner):
             tags.update(kwargs['userTags'])
         self._tags = json.dumps(tags)
 
-        userData =  self._getCloudConfigUserData('leader')
-        metadata = {'items': [{'key': 'user-data', 'value': userData}]}
+        metadata = {'items': [{'key': 'user-data', 'value': self._getCloudConfigUserData('leader')}]}
         imageType = 'flatcar-stable'
         sa_scopes = [{'scopes': ['compute', 'storage-full']}]
         disk = {}
@@ -146,14 +144,19 @@ class GCEProvisioner(AbstractProvisioner):
         disk.update({'boot': True,
              'autoDelete': True })
         name= 'l' + str(uuid.uuid4())
-        leader = self._gceDriver.create_node(name, leaderNodeType, imageType,
-                                            location=self._zone,
-                                            ex_service_accounts=sa_scopes,
-                                            ex_metadata=metadata,
-                                            ex_subnetwork=self._vpcSubnet,
-                                            ex_disks_gce_struct = [disk],
-                                            description=self._tags,
-                                            ex_preemptible=False)
+
+        leader = self._gceDriver.create_node(
+            name,
+            leaderNodeType,
+            imageType,
+            location=self._zone,
+            ex_service_accounts=sa_scopes,
+            ex_metadata=metadata,
+            ex_subnetwork=self._vpcSubnet,
+            ex_disks_gce_struct = [disk],
+            description=self._tags,
+            ex_preemptible=False
+        )
 
         self._instanceGroup.add_instances([leader])
         self._leaderPrivateIP = leader.private_ips[0] # needed if adding workers
