@@ -1,4 +1,4 @@
-# Copyright (C) 2015-2018 Regents of the University of California
+# Copyright (C) 2015-2021 Regents of the University of California
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -11,29 +11,28 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-
-from __future__ import absolute_import
-
-from builtins import str
-from builtins import object
 import gzip
 import json
 import logging
 import os
 import time
-from threading import Thread, Event
+from argparse import ArgumentParser
+from threading import Event, Thread
+from typing import Optional, Union, TextIO, BinaryIO
 
 from toil.lib.expando import Expando
-from toil.lib.bioio import getTotalCpuTime
+from toil.lib.resources import get_total_cpu_time
 
-logger = logging.getLogger( __name__ )
+logger = logging.getLogger(__name__)
+root_logger = logging.getLogger()
+toil_logger = logging.getLogger('toil')
+
+DEFAULT_LOGLEVEL = logging.INFO
+__loggingFiles = []
 
 
-class StatsAndLogging( object ):
-    """
-    Class manages a thread that aggregates statistics and logging information on a toil run.
-    """
-
+class StatsAndLogging:
+    """A thread to aggregate statistics and logging."""
     def __init__(self, jobStore, config):
         self._stop = Event()
         self._worker = Thread(target=self.statsAndLoggingAggregator,
@@ -41,43 +40,26 @@ class StatsAndLogging( object ):
                               daemon=True)
 
     def start(self):
-        """
-        Start the stats and logging thread.
-        """
+        """Start the stats and logging thread."""
         self._worker.start()
         
     @classmethod
-    def formatLogStream(cls, stream, identifier=None):
+    def formatLogStream(cls, stream: Union[TextIO, BinaryIO], job_name: Optional[str] = None):
         """
         Given a stream of text or bytes, and the job name, job itself, or some
         other optional stringifyable identity info for the job, return a big
         text string with the formatted job log, suitable for printing for the
         user.
-        
+
         We don't want to prefix every line of the job's log with our own
         logging info, or we get prefixes wider than any reasonable terminal
         and longer than the messages.
         """
-        
-        lines = []
-        
-        if identifier is not None:
-            if isinstance(identifier, bytes):
-                # Decode the identifier if it is bytes
-                identifier = identifier.decode('utf-8', error='replace')
-            elif not isinstance(identifier, str):
-                # Otherwise just stringify it
-                identifier = str(identifier)
-                
-            lines.append('Log from job %s follows:' % identifier)
-        else:
-            lines.append('Log from job follows:')
-            
-        lines.append('=========>')
-        
+        lines = [f'Log from job "{job_name}" follows:', '=========>']
+
         for line in stream:
             if isinstance(line, bytes):
-                line = line.decode('utf-8')
+                line = line.decode('utf-8', errors='replace')
             lines.append('\t' + line.rstrip('\n'))
             
         lines.append('<=========')
@@ -89,7 +71,7 @@ class StatsAndLogging( object ):
     def logWithFormatting(cls, jobStoreID, jobLogs, method=logger.debug, message=None):
         if message is not None:
             method(message)
-            
+
         # Format and log the logs, identifying the job with its job store ID.
         method(cls.formatLogStream(jobLogs, jobStoreID))
         
@@ -131,10 +113,8 @@ class StatsAndLogging( object ):
         fullName = createName(path, mainFileName, extension, failed)
         with writeFn(fullName, 'wb') as f:
             for l in jobLogList:
-                try:
+                if isinstance(l, bytes):
                     l = l.decode('utf-8')
-                except AttributeError:
-                    pass
                 if not l.endswith('\n'):
                     l += '\n'
                 f.write(l.encode('utf-8'))
@@ -153,7 +133,7 @@ class StatsAndLogging( object ):
         """
         #  Overall timing
         startTime = time.time()
-        startClock = getTotalCpuTime()
+        startClock = get_total_cpu_time()
 
         def callback(fileHandle):
             statsStr = fileHandle.read()
@@ -192,7 +172,7 @@ class StatsAndLogging( object ):
 
         # Finish the stats file
         text = json.dumps(dict(total_time=str(time.time() - startTime),
-                               total_clock=str(getTotalCpuTime() - startClock)), ensure_ascii=True)
+                               total_clock=str(get_total_cpu_time() - startClock)), ensure_ascii=True)
         jobStore.writeStatsAndLogging(text)
 
     def check(self):
@@ -204,12 +184,102 @@ class StatsAndLogging( object ):
             raise RuntimeError("Stats and logging thread has quit")
 
     def shutdown(self):
-        """
-        Finish up the stats/logging aggregation thread
-        """
+        """Finish up the stats/logging aggregation thread."""
         logger.debug('Waiting for stats and logging collator thread to finish ...')
         startTime = time.time()
         self._stop.set()
         self._worker.join()
         logger.debug('... finished collating stats and logs. Took %s seconds', time.time() - startTime)
         # in addition to cleaning on exceptions, onError should clean if there are any failed jobs
+
+
+def set_log_level(level, set_logger=None):
+    """Sets the root logger level to a given string level (like "INFO")."""
+    level = "CRITICAL" if level.upper() == "OFF" else level.upper()
+    set_logger = set_logger if set_logger else root_logger
+    set_logger.setLevel(level)
+
+    # Suppress any random loggers introduced by libraries we use.
+    # Especially boto/boto3.  They print too much.  -__-
+    suppress_exotic_logging(__name__)
+
+
+def add_logging_options(parser: ArgumentParser):
+    """Add logging options to set the global log level."""
+    group = parser.add_argument_group("Logging Options")
+    default_loglevel = logging.getLevelName(DEFAULT_LOGLEVEL)
+
+    levels = ['Critical', 'Error', 'Warning', 'Debug', 'Info']
+    for level in levels:
+        group.add_argument(f"--log{level}", dest="logLevel", default=default_loglevel, action="store_const",
+                           const=level, help=f"Turn on loglevel {level}.  Default: {default_loglevel}.")
+
+    levels += [l.lower() for l in levels] + [l.upper() for l in levels]
+    group.add_argument("--logOff", dest="logLevel", default=default_loglevel,
+                       action="store_const", const="CRITICAL", help="Same as --logCRITICAL.")
+    group.add_argument("--logLevel", dest="logLevel", default=default_loglevel, choices=levels,
+                       help=f"Set the log level. Default: {default_loglevel}.  Options: {levels}.")
+    group.add_argument("--logFile", dest="logFile", help="File to log in.")
+    group.add_argument("--rotatingLogging", dest="logRotating", action="store_true", default=False,
+                       help="Turn on rotating logging, which prevents log files from getting too big.")
+
+
+def configure_root_logger():
+    """
+    Set up the root logger with handlers and formatting.
+
+    Should be called before any entry point tries to log anything,
+    to ensure consistent formatting.
+    """
+    logging.basicConfig(format='[%(asctime)s] [%(threadName)-10s] [%(levelname).1s] [%(name)s] %(message)s',
+                        datefmt='%Y-%m-%dT%H:%M:%S%z')
+    root_logger.setLevel(DEFAULT_LOGLEVEL)
+
+
+def log_to_file(log_file, log_rotation):
+    if log_file and log_file not in __loggingFiles:
+        logger.debug(f"Logging to file '{log_file}'.")
+        __loggingFiles.append(log_file)
+        if log_rotation:
+            handler = logging.handlers.RotatingFileHandler(log_file, maxBytes=1000000, backupCount=1)
+        else:
+            handler = logging.FileHandler(log_file)
+        root_logger.addHandler(handler)
+
+
+def set_logging_from_options(options):
+    configure_root_logger()
+    options.logLevel = options.logLevel or logging.getLevelName(root_logger.getEffectiveLevel())
+    set_log_level(options.logLevel)
+    logger.debug(f"Root logger is at level '{logging.getLevelName(root_logger.getEffectiveLevel())}', "
+                 f"'toil' logger at level '{logging.getLevelName(toil_logger.getEffectiveLevel())}'.")
+
+    # start logging to log file if specified
+    log_to_file(options.logFile, options.logRotating)
+
+
+def suppress_exotic_logging(local_logger):
+    """
+    Attempts to suppress the loggers of all non-Toil packages by setting them to CRITICAL.
+
+    For example: 'requests_oauthlib', 'google', 'boto', 'websocket', 'oauthlib', etc.
+
+    This will only suppress loggers that have already been instantiated and can be seen in the environment,
+    except for the list declared in "always_suppress".
+
+    This is important because some packages, particularly boto3, are not always instantiated yet in the
+    environment when this is run, and so we create the logger and set the level preemptively.
+    """
+    never_suppress = ['toil', '__init__', '__main__', 'toil-rt', 'cwltool']
+    always_suppress = ['boto3', 'boto', 'botocore']  # ensure we suppress even before instantiated
+
+    top_level_loggers = list()
+    for pkg_logger in list(logging.Logger.manager.loggerDict.keys()) + always_suppress:
+        if pkg_logger != local_logger:
+            # many sub-loggers may exist, like "boto.a", "boto.b", "boto.c"; we only want the top_level: "boto"
+            top_level_logger = pkg_logger.split('.')[0] if '.' in pkg_logger else pkg_logger
+
+            if top_level_logger not in top_level_loggers + never_suppress:
+                top_level_loggers.append(top_level_logger)
+                logging.getLogger(top_level_logger).setLevel(logging.CRITICAL)
+    logger.debug(f'Suppressing the following loggers: {set(top_level_loggers)}')
