@@ -1,4 +1,4 @@
-# Copyright (C) 2015-2016 Regents of the University of California
+# Copyright (C) 2015-2021 Regents of the University of California
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -11,57 +11,47 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-
-from __future__ import absolute_import
-from __future__ import division
-from builtins import map
-from builtins import str
-from builtins import range
-from builtins import object
-from past.utils import old_div
+import fcntl
+import itertools
+import logging
+import os
+import subprocess
+import sys
+import tempfile
+import time
 from abc import ABCMeta, abstractmethod
 from fractions import Fraction
 from inspect import getsource
-import logging
-import os
-import fcntl
-import itertools
-import tempfile
 from textwrap import dedent
-import time
-import sys
-import subprocess
 from unittest import skipIf
 
-from toil.common import Config
+from toil.batchSystems.abstractBatchSystem import (BatchSystemSupport,
+                                                   InsufficientSystemResources)
 # Don't import any batch systems here that depend on extras
 # in order to import properly. Import them later, in tests 
 # protected by annotations.
 from toil.batchSystems.mesos.test import MesosTestSupport
-from toil.batchSystems.parasolTestSupport import ParasolTestSupport
 from toil.batchSystems.parasol import ParasolBatchSystem
+from toil.test.batchSystems.parasolTestSupport import ParasolTestSupport
 from toil.batchSystems.singleMachine import SingleMachineBatchSystem
-from toil.batchSystems.abstractBatchSystem import (InsufficientSystemResources,
-                                                   BatchSystemSupport)
-from toil.job import Job, JobNode
+from toil.common import Config
+from toil.job import Job, JobDescription
 from toil.lib.threading import cpu_count
 from toil.test import (ToilTest,
                        needs_aws_s3,
-                       needs_lsf,
-                       needs_kubernetes,
                        needs_fetchable_appliance,
+                       needs_gridengine,
+                       needs_htcondor,
+                       needs_kubernetes,
+                       needs_lsf,
                        needs_mesos,
                        needs_parasol,
-                       needs_gridengine,
                        needs_slurm,
                        needs_torque,
-                       needs_htcondor,
                        slow,
-                       tempFileContaining,
                        travis_test)
-from future.utils import with_metaclass
 
-log = logging.getLogger(__name__)
+logger = logging.getLogger(__name__)
 
 # How many cores should be utilized by this test. The test will fail if the running system
 # doesn't have at least that many cores.
@@ -80,7 +70,7 @@ class hidden(object):
     http://stackoverflow.com/questions/1323455/python-unit-test-with-base-and-sub-class#answer-25695512
     """
 
-    class AbstractBatchSystemTest(with_metaclass(ABCMeta, ToilTest)):
+    class AbstractBatchSystemTest(ToilTest, metaclass=ABCMeta):
         """
         A base test case with generic tests that every batch system should pass.
 
@@ -120,6 +110,24 @@ class hidden(object):
             :rtype: toil.common.Config
             """
             return self.createConfig()
+            
+        def _mockJobDescription(self, jobStoreID=None, command=None, **kwargs):
+            """
+            Create a mock-up JobDescription with the given ID, command, and other parameters.
+            """
+           
+            # TODO: Use a real unittest.Mock? For now we make a real instance and just hack it up.
+            
+            desc = JobDescription(**kwargs)
+            # Normally we can't pass in a command or ID, and the job
+            # serialization logic takes care of filling them in. We set them
+            # here.
+            if command is not None:
+                desc.command = command
+            if jobStoreID is not None:
+                desc.jobStoreID = jobStoreID
+                
+            return desc
 
         @classmethod
         def setUpClass(cls):
@@ -140,12 +148,12 @@ class hidden(object):
             self.assertTrue(cpu_count() >= numCores)
         
         def testRunJobs(self):
-            jobNode1 = JobNode(command='sleep 1000', jobName='test1', unitName=None,
-                               jobStoreID='1', requirements=defaultRequirements)
-            jobNode2 = JobNode(command='sleep 1000', jobName='test2', unitName=None,
-                               jobStoreID='2', requirements=defaultRequirements)
-            job1 = self.batchSystem.issueBatchJob(jobNode1)
-            job2 = self.batchSystem.issueBatchJob(jobNode2)
+            jobDesc1 = self._mockJobDescription(command='sleep 1000', jobName='test1', unitName=None,
+                                                jobStoreID='1', requirements=defaultRequirements)
+            jobDesc2 = self._mockJobDescription(command='sleep 1000', jobName='test2', unitName=None,
+                                                jobStoreID='2', requirements=defaultRequirements)
+            job1 = self.batchSystem.issueBatchJob(jobDesc1)
+            job2 = self.batchSystem.issueBatchJob(jobDesc2)
 
             issuedIDs = self._waitForJobsToIssue(2)
             self.assertEqual(set(issuedIDs), {job1, job2})
@@ -174,13 +182,13 @@ class hidden(object):
             # then check for it having happened, but we can't guarantee that
             # the batch system will run against the same filesystem we are
             # looking at.
-            jobNode3 = JobNode(command="mktemp -d", jobName='test3', unitName=None,
-                               jobStoreID='3', requirements=defaultRequirements)
-            job3 = self.batchSystem.issueBatchJob(jobNode3)
+            jobDesc3 = self._mockJobDescription(command="mktemp -d", jobName='test3', unitName=None,
+                                                jobStoreID='3', requirements=defaultRequirements)
+            job3 = self.batchSystem.issueBatchJob(jobDesc3)
 
             jobUpdateInfo = self.batchSystem.getUpdatedBatchJob(maxWait=1000)
             jobID, exitStatus, wallTime = jobUpdateInfo.jobID, jobUpdateInfo.exitStatus, jobUpdateInfo.wallTime
-            log.info('Third job completed: {} {} {}'.format(jobID, exitStatus, wallTime))
+            logger.info('Third job completed: {} {} {}'.format(jobID, exitStatus, wallTime))
 
             # Since the first two jobs were killed, the only job in the updated jobs queue should
             # be job 3. If the first two jobs were (incorrectly) added to the queue, this will
@@ -206,22 +214,22 @@ class hidden(object):
             script_shell = 'if [ "x${FOO}" == "xbar" ] ; then exit 23 ; else exit 42 ; fi'
 
             # Escape the semicolons
-            script_protected = script_shell.replace(';', '\;') 
+            script_protected = script_shell.replace(';', r'\;') 
              
             # Turn into a string which convinces bash to take all args and paste them back together and run them
             command = "bash -c \"\\${@}\" bash eval " + script_protected
-            jobNode4 = JobNode(command=command, jobName='test4', unitName=None,
-                               jobStoreID='4', requirements=defaultRequirements)
-            job4 = self.batchSystem.issueBatchJob(jobNode4)
+            jobDesc4 = self._mockJobDescription(command=command, jobName='test4', unitName=None,
+                                                jobStoreID='4', requirements=defaultRequirements)
+            job4 = self.batchSystem.issueBatchJob(jobDesc4)
             jobUpdateInfo = self.batchSystem.getUpdatedBatchJob(maxWait=1000)
             jobID, exitStatus, wallTime = jobUpdateInfo.jobID, jobUpdateInfo.exitStatus, jobUpdateInfo.wallTime
             self.assertEqual(exitStatus, 42)
             self.assertEqual(jobID, job4)
             # Now set the variable and ensure that it is present
             self.batchSystem.setEnv('FOO', 'bar')
-            jobNode5 = JobNode(command=command, jobName='test5', unitName=None,
-                               jobStoreID='5', requirements=defaultRequirements)
-            job5 = self.batchSystem.issueBatchJob(jobNode5)
+            jobDesc5 = self._mockJobDescription(command=command, jobName='test5', unitName=None,
+                                                jobStoreID='5', requirements=defaultRequirements)
+            job5 = self.batchSystem.issueBatchJob(jobDesc5)
             jobUpdateInfo = self.batchSystem.getUpdatedBatchJob(maxWait=1000)
             self.assertEqual(jobUpdateInfo.exitStatus, 23)
             self.assertEqual(jobUpdateInfo.jobID, job5)
@@ -268,14 +276,14 @@ class hidden(object):
             # prevent an endless loop, give it a few tries
             for it in range(tries):
                 running = self.batchSystem.getRunningBatchJobIDs()
-                log.info('Running jobs now: {}'.format(running))
+                logger.info('Running jobs now: {}'.format(running))
                 runningIDs = list(running.keys())
                 if len(runningIDs) == numJobs:
                     break
                 time.sleep(1)
             return runningIDs
 
-    class AbstractBatchSystemJobTest(with_metaclass(ABCMeta, ToilTest)):
+    class AbstractBatchSystemJobTest(ToilTest, metaclass=ABCMeta):
         """
         An abstract base class for batch system tests that use a full Toil workflow rather
         than using the batch system directly.
@@ -332,7 +340,7 @@ class hidden(object):
                                                 cores=coresPerJob, memory='1M', disk='1Mi'))
                 Job.Runner.startToil(root, options)
                 _, maxValue = getCounters(counterPath)
-                self.assertEqual(maxValue, old_div(self.cpuCount, coresPerJob))
+                self.assertEqual(maxValue, self.cpuCount // coresPerJob)
 
     class AbstractGridEngineBatchSystemTest(AbstractBatchSystemTest):
         """
@@ -395,9 +403,9 @@ class MesosBatchSystemTest(hidden.AbstractBatchSystemTest, MesosTestSupport):
 
     def testIgnoreNode(self):
         self.batchSystem.ignoreNode('localhost')
-        jobNode = JobNode(command='sleep 1000', jobName='test2', unitName=None,
-                           jobStoreID='1', requirements=defaultRequirements)
-        job = self.batchSystem.issueBatchJob(jobNode)
+        jobDesc = self._mockJobDescription(command='sleep 1000', jobName='test2', unitName=None,
+                                           jobStoreID='1', requirements=defaultRequirements)
+        job = self.batchSystem.issueBatchJob(jobDesc)
 
         issuedID = self._waitForJobsToIssue(1)
         self.assertEqual(set(issuedID), {job})
@@ -454,7 +462,10 @@ class MaxCoresSingleMachineBatchSystemTest(ToilTest):
         self.counterPath = writeTempFile('0,0')
 
         def script():
-            import os, sys, fcntl, time
+            import fcntl
+            import os
+            import sys
+            import time
             def count(delta):
                 """
                 Adjust the first integer value in a file by the given amount. If the result
@@ -519,12 +530,12 @@ class MaxCoresSingleMachineBatchSystemTest(ToilTest):
                         try:
                             jobIds = set()
                             for i in range(0, int(jobs)):
-                                jobIds.add(bs.issueBatchJob(JobNode(command=self.scriptCommand(),
-                                                                    requirements=dict(
-                                                                        cores=float( coresPerJob),
-                                                                        memory=1, disk=1,
-                                                                        preemptable=preemptable),
-                                                                    jobName=str(i), unitName='', jobStoreID=str(i))))
+                                jobIds.add(bs.issueBatchJob(JobDescription(command=self.scriptCommand(),
+                                                                           requirements=dict(
+                                                                               cores=float(coresPerJob),
+                                                                               memory=1, disk=1,
+                                                                               preemptable=preemptable),
+                                                                           jobName=str(i), unitName='')))
                             self.assertEqual(len(jobIds), jobs)
                             while jobIds:
                                 job = bs.getUpdatedBatchJob(maxWait=10)
@@ -537,11 +548,11 @@ class MaxCoresSingleMachineBatchSystemTest(ToilTest):
                             bs.shutdown()
                         concurrentTasks, maxConcurrentTasks = getCounters(self.counterPath)
                         self.assertEqual(concurrentTasks, 0)
-                        log.info('maxCores: {maxCores}, '
+                        logger.info('maxCores: {maxCores}, '
                                  'coresPerJob: {coresPerJob}, '
                                  'load: {load}'.format(**locals()))
                         # This is the key assertion:
-                        expectedMaxConcurrentTasks = min(old_div(maxCores, coresPerJob), jobs)
+                        expectedMaxConcurrentTasks = min(maxCores // coresPerJob, jobs)
                         self.assertEqual(maxConcurrentTasks, expectedMaxConcurrentTasks)
                         resetCounters(self.counterPath)
 
@@ -554,7 +565,7 @@ class MaxCoresSingleMachineBatchSystemTest(ToilTest):
         Job.Runner.startToil(Job.wrapJobFn(parentJob, self.scriptCommand()), options)
         with open(self.counterPath, 'r+') as f:
             s = f.read()
-        log.info('Counter is %s', s)
+        logger.info('Counter is %s', s)
         self.assertEqual(getCounters(self.counterPath), (0, 3))
 
 
@@ -625,19 +636,17 @@ class ParasolBatchSystemTest(hidden.AbstractBatchSystemTest, ParasolTestSupport)
         self._stopParasol()
 
     def testBatchResourceLimits(self):
-        jobNode1 = JobNode(command="sleep 1000",
-                           requirements=dict(memory=1 << 30, cores=1,
-                                             disk=1000, preemptable=preemptable),
-                           jobName='testResourceLimits', unitName=None,
-                           jobStoreID='1')
-        job1 = self.batchSystem.issueBatchJob(jobNode1)
+        jobDesc1 = JobDescription(command="sleep 1000",
+                                  requirements=dict(memory=1 << 30, cores=1,
+                                                    disk=1000, preemptable=preemptable),
+                                  jobName='testResourceLimits')
+        job1 = self.batchSystem.issueBatchJob(jobDesc1)
         self.assertIsNotNone(job1)
-        jobNode2 = JobNode(command="sleep 1000",
-                           requirements=dict(memory=2 << 30, cores=1,
-                                             disk=1000, preemptable=preemptable),
-                           jobName='testResourceLimits', unitName=None,
-                           jobStoreID='2')
-        job2 = self.batchSystem.issueBatchJob(jobNode2)
+        jobDesc2 = JobDescription(command="sleep 1000",
+                                  requirements=dict(memory=2 << 30, cores=1,
+                                                    disk=1000, preemptable=preemptable),
+                                  jobName='testResourceLimits')
+        job2 = self.batchSystem.issueBatchJob(jobDesc2)
         self.assertIsNotNone(job2)
         batches = self._getBatchList()
         self.assertEqual(len(batches), 2)
@@ -646,7 +655,7 @@ class ParasolBatchSystemTest(hidden.AbstractBatchSystemTest, ParasolTestSupport)
         self.assertNotEqual(batches[0]['ram'], batches[1]['ram'])
         # Need to kill one of the jobs because there are only two cores available
         self.batchSystem.killBatchJobs([job2])
-        job3 = self.batchSystem.issueBatchJob(jobNode1)
+        job3 = self.batchSystem.issueBatchJob(jobDesc1)
         self.assertIsNotNone(job3)
         batches = self._getBatchList()
         self.assertEqual(len(batches), 1)
@@ -654,7 +663,7 @@ class ParasolBatchSystemTest(hidden.AbstractBatchSystemTest, ParasolTestSupport)
     def _parseBatchString(self, batchString):
         import re
         batchInfo = dict()
-        memPattern = re.compile("(\d+\.\d+)([kgmbt])")
+        memPattern = re.compile(r"(\d+\.\d+)([kgmbt])")
         items = batchString.split()
         batchInfo["cores"] = int(items[7])
         memMatch = memPattern.match(items[8])
@@ -770,7 +779,7 @@ class SingleMachineBatchSystemJobTest(hidden.AbstractBatchSystemJobTest):
     """
 
     def getBatchSystemName(self):
-        return "singleMachine"
+        return "single_machine"
 
     @slow
     def testConcurrencyWithDisk(self):
@@ -782,7 +791,7 @@ class SingleMachineBatchSystemJobTest(hidden.AbstractBatchSystemJobTest):
         options = Job.Runner.getDefaultOptions(self._getTestJobStorePath())
         options.workDir = tempDir
         from toil import physicalDisk
-        availableDisk = physicalDisk('', toilWorkflowDir=options.workDir)
+        availableDisk = physicalDisk(options.workDir)
         options.batchSystem = self.batchSystemName
 
         counterPath = os.path.join(tempDir, 'counter')
@@ -794,9 +803,9 @@ class SingleMachineBatchSystemJobTest(hidden.AbstractBatchSystemJobTest):
         # Physically, we're asking for 50% of disk and 50% of disk + 500bytes in the two jobs. The
         # batchsystem should not allow the 2 child jobs to run concurrently.
         root.addChild(Job.wrapFn(measureConcurrency, counterPath, self.sleepTime, cores=1,
-                                    memory='1M', disk=old_div(availableDisk,2)))
+                                    memory='1M', disk=availableDisk // 2))
         root.addChild(Job.wrapFn(measureConcurrency, counterPath, self.sleepTime, cores=1,
-                                 memory='1M', disk=(old_div(availableDisk, 2)) + 500))
+                                 memory='1M', disk=(availableDisk // 2) + 500))
         Job.Runner.startToil(root, options)
         _, maxValue = getCounters(counterPath)
         self.assertEqual(maxValue, 1)
@@ -832,15 +841,15 @@ class SingleMachineBatchSystemJobTest(hidden.AbstractBatchSystemJobTest):
 
         # Should block off 50% of memory while waiting for it's 3 cores
         firstJobChild = Job.wrapFn(_resourceBlockTestAuxFn, outFile=outFile, sleepTime=0,
-                                   writeVal='fJC', cores=3, memory=int(old_div(availableMemory,2)), disk='1M')
+                                   writeVal='fJC', cores=3, memory=int(availableMemory // 2), disk='1M')
 
         # These two shouldn't be able to run before B because there should be only
         # (50% of memory - 1M) available (firstJobChild should be blocking 50%)
         secondJobChild = Job.wrapFn(_resourceBlockTestAuxFn, outFile=outFile, sleepTime=5,
-                                    writeVal='sJC', cores=2, memory=int(old_div(availableMemory,1.5)),
+                                    writeVal='sJC', cores=2, memory=int(availableMemory // 1.5),
                                     disk='1M')
         secondJobGrandChild = Job.wrapFn(_resourceBlockTestAuxFn, outFile=outFile, sleepTime=5,
-                                         writeVal='sJGC', cores=2, memory=int(old_div(availableMemory,1.5)),
+                                         writeVal='sJGC', cores=2, memory=int(availableMemory // 1.5),
                                          disk='1M')
 
         root.addChild(blocker)
@@ -895,7 +904,6 @@ class MesosBatchSystemJobTest(hidden.AbstractBatchSystemJobTest, MesosTestSuppor
     """
     Tests Toil workflow against the Mesos batch system
     """
-
     def getOptions(self, tempDir):
         options = super(MesosBatchSystemJobTest, self).getOptions(tempDir)
         options.mesosMasterAddress = 'localhost:5050'
@@ -939,12 +947,12 @@ def count(delta, file_path):
         fcntl.flock(fd, fcntl.LOCK_EX)
         try:
             s = os.read(fd, 10)
-            value, maxValue = list(map(int, s.decode('utf-8').split(',')))
+            value, maxValue = [int(i) for i in s.decode('utf-8').split(',')]
             value += delta
             if value > maxValue: maxValue = value
             os.lseek(fd, 0, 0)
             os.ftruncate(fd, 0)
-            os.write(fd, ','.join(map(str, (value, maxValue))).encode('utf-8'))
+            os.write(fd, f'{value},{maxValue}'.encode('utf-8'))
         finally:
             fcntl.flock(fd, fcntl.LOCK_UN)
     finally:
@@ -954,8 +962,7 @@ def count(delta, file_path):
 
 def getCounters(path):
     with open(path, 'r+') as f:
-        s = f.read()
-        concurrentTasks, maxConcurrentTasks = list(map(int, s.split(',')))
+        concurrentTasks, maxConcurrentTasks = [int(i) for i in f.read().split(',')]
     return concurrentTasks, maxConcurrentTasks
 
 
