@@ -1,4 +1,4 @@
-# Copyright (C) 2015-2020 Regents of the University of California
+# Copyright (C) 2015-2021 Regents of the University of California
 # Copyright (C) 2015 Curoverse, Inc
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -13,31 +13,75 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 import json
+import logging
 import os
+import re
+import shutil
+import subprocess
 import sys
 import unittest
-import re
-import logging
-import shutil
-import zipfile
-import pytest
 import uuid
-from urllib.request import urlretrieve
+import zipfile
 from io import StringIO
+from urllib.request import urlretrieve
+
+import psutil
+import pytest
 
 pkg_root = os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))  # noqa
 sys.path.insert(0, pkg_root)  # noqa
 
-logger = logging.getLogger(__name__)
-import subprocess
-from toil.test import (ToilTest, needs_cwl, slow, needs_docker, needs_lsf,
-                       needs_mesos, needs_parasol, needs_gridengine, needs_slurm,
-                       needs_torque)
-
+from toil.test import (ToilTest,
+                       needs_aws_s3,
+                       needs_cwl,
+                       needs_docker,
+                       needs_gridengine,
+                       needs_lsf,
+                       needs_mesos,
+                       needs_parasol,
+                       needs_slurm,
+                       needs_torque,
+                       slow)
 
+log = logging.getLogger(__name__)
 CONFORMANCE_TEST_TIMEOUT = 3600
 
 
+def run_conformance_tests(workDir, yml, caching=False, batchSystem=None, selected_tests=None):
+    try:
+        cmd = ['cwltest',
+               '--tool=toil-cwl-runner',
+               f'-j{int(psutil.cpu_count()/2)}',
+               f'--test={yml}',
+               '--timeout=2400',
+               f'--basedir={workDir}']
+        if selected_tests:
+            cmd.append(f'-n={selected_tests}')
+
+        args_passed_directly_to_toil = [f'--disableCaching={not caching}', '--clean=always']
+        if batchSystem:
+            args_passed_directly_to_toil.append(f"--batchSystem={batchSystem}")
+        cmd.extend(['--'] + args_passed_directly_to_toil)
+
+        log.info("Running: '%s'", "' '".join(cmd))
+        subprocess.check_output(cmd, cwd=workDir, stderr=subprocess.STDOUT)
+    except subprocess.CalledProcessError as e:
+        only_unsupported = False
+        # check output -- if we failed but only have unsupported features, we're okay
+        p = re.compile(r"(?P<failures>\d+) failures, (?P<unsupported>\d+) unsupported features")
+
+        error_log = e.output.decode('utf-8')
+        for line in error_log.split('\n'):
+            m = p.search(line)
+            if m:
+                if int(m.group("failures")) == 0 and int(m.group("unsupported")) > 0:
+                    only_unsupported = True
+                    break
+        if not only_unsupported:
+            print(error_log)
+            raise e
+
+
 @needs_cwl
 class CWLv10Test(ToilTest):
     def setUp(self):
@@ -49,7 +93,7 @@ class CWLv10Test(ToilTest):
         self.workDir = os.path.join(self.cwlSpec, 'v1.0')
         # The latest cwl git commit hash from https://github.com/common-workflow-language/common-workflow-language.
         # Update it to get the latest tests.
-        testhash = '40fcfc01812046f012acf5153cc955ee848e69e3'  # Date:   Tue Jan 21 07:36:37 2020 +0100
+        testhash = '6a955874ade22080b8ef962b4e0d6e408112c1ef'  # Date:   Tue Dec 16 2020 8:43pm PST
         url = 'https://github.com/common-workflow-language/common-workflow-language/archive/%s.zip' % testhash
         if not os.path.exists(self.cwlSpec):
             urlretrieve(url, 'spec.zip')
@@ -94,12 +138,27 @@ class CWLv10Test(ToilTest):
                   'src/toil/test/cwl/revsort-job.json',
                   self._expected_revsort_output(self.outDir))
 
-    def download(self,inputs, tester_fn):
+    def download(self, inputs, tester_fn):
         input_location = os.path.join('src/toil/test/cwl', inputs)
         tester_fn('src/toil/test/cwl/download.cwl',
                   input_location,
                   self._expected_download_output(self.outDir))
 
+    @needs_aws_s3
+    def test_s3_as_secondary_file(self):
+        from toil.cwl import cwltoil
+        st = StringIO()
+        main_args = ['--jobStore', f'aws:us-west-2:deleteme-cwl-s3-secondary-file-test-{str(uuid.uuid4())[-12:]}',
+                     '--outdir', self.outDir,
+                     os.path.join(self.rootDir, 'src/toil/test/cwl/s3_secondary_file.cwl'),
+                     os.path.join(self.rootDir, 'src/toil/test/cwl/s3_secondary_file.json')]
+        cwltoil.main(main_args, stdout=st)
+        out = json.loads(st.getvalue())
+        self.assertEqual(out['output']['checksum'], 'sha1$d14dd02e354918b4776b941d154c18ebc15b9b38')
+        self.assertEqual(out['output']['size'], 24)
+        with open(out['output']['location'][len('file://'):], 'r') as f:
+            self.assertEqual(f.read().strip(), 'When is s4 coming out?')
+
     def test_run_revsort(self):
         self.revsort('revsort.cwl', self._tester)
 
@@ -109,6 +168,7 @@ class CWLv10Test(ToilTest):
     def test_run_revsort_debug_worker(self):
         self.revsort('revsort.cwl', self._debug_worker_tester)
 
+    @needs_aws_s3
     def test_run_s3(self):
         self.download('download_s3.json', self._tester)
 
@@ -137,7 +197,7 @@ class CWLv10Test(ToilTest):
     @slow
     def test_restart(self):
         """Enable restarts with toil-cwl-runner -- run failing test, re-run correct test."""
-        logger.info('Running CWL Test Restart.  Expecting failure, then success.')
+        log.info('Running CWL Test Restart.  Expecting failure, then success.')
         from toil.cwl import cwltoil
         from toil.jobStores.abstractJobStore import NoSuchJobStoreException
         from toil.leader import FailedJobsException
@@ -180,53 +240,34 @@ class CWLv10Test(ToilTest):
     @slow
     @pytest.mark.timeout(CONFORMANCE_TEST_TIMEOUT)
     def test_run_conformance(self, batchSystem=None, caching=False):
-        try:
-            cmd = ['cwltest', '--tool', 'toil-cwl-runner', '--test=conformance_test_v1.0.yaml',
-                   '--timeout=2400', '--basedir=' + self.workDir]
-            if batchSystem:
-                cmd.extend(["--batchSystem", batchSystem])
-            if caching:
-                cmd.extend(['--', '--disableCaching="False"'])
-            subprocess.check_output(cmd, cwd=self.workDir, stderr=subprocess.STDOUT)
-        except subprocess.CalledProcessError as e:
-            only_unsupported = False
-            # check output -- if we failed but only have unsupported features, we're okay
-            p = re.compile(r"(?P<failures>\d+) failures, (?P<unsupported>\d+) unsupported features")
-
-            error_log = e.output.decode('utf-8')
-            for line in error_log.split('\n'):
-                m = p.search(line)
-                if m:
-                    if int(m.group("failures")) == 0 and int(m.group("unsupported")) > 0:
-                        only_unsupported = True
-                        break
-            if not only_unsupported:
-                print(error_log)
-                raise e
+        run_conformance_tests(workDir=self.workDir,
+                              yml='conformance_test_v1.0.yaml',
+                              caching=caching,
+                              batchSystem=batchSystem)
 
     @slow
     @needs_lsf
     @unittest.skip
     def test_lsf_cwl_conformance_with_caching(self):
-        return self.test_run_conformance(batchSystem="LSF", caching=True)
+        return self.test_run_conformance(batchSystem="lsf", caching=True)
 
     @slow
     @needs_slurm
     @unittest.skip
     def test_slurm_cwl_conformance_with_caching(self):
-        return self.test_run_conformance(batchSystem="Slurm", caching=True)
+        return self.test_run_conformance(batchSystem="slurm", caching=True)
 
     @slow
     @needs_torque
     @unittest.skip
     def test_torque_cwl_conformance_with_caching(self):
-        return self.test_run_conformance(batchSystem="Torque", caching=True)
+        return self.test_run_conformance(batchSystem="torque", caching=True)
 
     @slow
     @needs_gridengine
     @unittest.skip
     def test_gridengine_cwl_conformance_with_caching(self):
-        return self.test_run_conformance(batchSystem="gridEngine", caching=True)
+        return self.test_run_conformance(batchSystem="grid_engine", caching=True)
 
     @slow
     @needs_mesos
@@ -244,25 +285,25 @@ class CWLv10Test(ToilTest):
     @needs_lsf
     @unittest.skip
     def test_lsf_cwl_conformance(self):
-        return self.test_run_conformance(batchSystem="LSF")
+        return self.test_run_conformance(batchSystem="lsf")
 
     @slow
     @needs_slurm
     @unittest.skip
     def test_slurm_cwl_conformance(self):
-        return self.test_run_conformance(batchSystem="Slurm")
+        return self.test_run_conformance(batchSystem="slurm")
 
     @slow
     @needs_torque
     @unittest.skip
     def test_torque_cwl_conformance(self):
-        return self.test_run_conformance(batchSystem="Torque")
+        return self.test_run_conformance(batchSystem="torque")
 
     @slow
     @needs_gridengine
     @unittest.skip
     def test_gridengine_cwl_conformance(self):
-        return self.test_run_conformance(batchSystem="gridEngine")
+        return self.test_run_conformance(batchSystem="grid_engine")
 
     @slow
     @needs_mesos
@@ -328,7 +369,7 @@ class CWLv11Test(ToilTest):
         cls.test_yaml = os.path.join(cls.cwlSpec, 'conformance_tests.yaml')
         # TODO: Use a commit zip in case someone decides to rewrite master's history?
         url = 'https://github.com/common-workflow-language/cwl-v1.1.git'
-        commit = 'a22b7580c6b50e77c0a181ca59d3828dd5c69143'
+        commit = '664835e83eb5e57eee18a04ce7b05fb9d70d77b7'
         p = subprocess.Popen(f'git clone {url} {cls.cwlSpec} && cd {cls.cwlSpec} && git checkout {commit}', shell=True)
         p.communicate()
 
@@ -340,43 +381,16 @@ class CWLv11Test(ToilTest):
 
     @slow
     @pytest.mark.timeout(CONFORMANCE_TEST_TIMEOUT)
-    # Cannot work until we fix https://github.com/DataBiosphere/toil/issues/2801
-    @pytest.mark.xfail
     def test_run_conformance_with_caching(self):
         self.test_run_conformance(caching=True)
 
     @slow
     @pytest.mark.timeout(CONFORMANCE_TEST_TIMEOUT)
     def test_run_conformance(self, batchSystem=None, caching=False):
-        try:
-            # TODO: we do not currently pass tests: 213, 236
-            selected_tests = '1-212,214-235,237-253'
-            cmd = [f'cwltest',
-                   f'--tool=toil-cwl-runner',
-                   f'--test={self.test_yaml}',
-                   f'--timeout=2400',
-                   f'--basedir={self.cwlSpec}',
-                   f'-n={selected_tests}']
-            if batchSystem:
-                cmd.extend(["--batchSystem", batchSystem])
-            if caching:
-                cmd.extend(['--', '--disableCaching="False"'])
-            subprocess.check_output(cmd, cwd=self.cwlSpec, stderr=subprocess.STDOUT)
-        except subprocess.CalledProcessError as e:
-            only_unsupported = False
-            # check output -- if we failed but only have unsupported features, we're okay
-            p = re.compile(r"(?P<failures>\d+) failures, (?P<unsupported>\d+) unsupported features")
-
-            error_log = e.output.decode('utf-8')
-            for line in error_log.split('\n'):
-                m = p.search(line)
-                if m:
-                    if int(m.group("failures")) == 0 and int(m.group("unsupported")) > 0:
-                        only_unsupported = True
-                        break
-            if not only_unsupported:
-                print(error_log)
-                raise e
+        run_conformance_tests(workDir=self.cwlSpec,
+                              yml=self.test_yaml,
+                              caching=caching,
+                              batchSystem=batchSystem)
 
 @needs_cwl
 class CWLv12Test(ToilTest):
@@ -390,7 +404,7 @@ class CWLv12Test(ToilTest):
         cls.test_yaml = os.path.join(cls.cwlSpec, 'conformance_tests.yaml')
         # TODO: Use a commit zip in case someone decides to rewrite master's history?
         url = 'https://github.com/common-workflow-language/cwl-v1.2.git'
-        commit = '936961d1c1c5c4ea13df253b1316a264e48df740'
+        commit = '8c3fd9d9f0209a51c5efacb1c7bc02a1164688d6'
         p = subprocess.Popen(f'git clone {url} {cls.cwlSpec} && cd {cls.cwlSpec} && git checkout {commit}', shell=True)
         p.communicate()
 
@@ -402,44 +416,35 @@ class CWLv12Test(ToilTest):
 
     @slow
     @pytest.mark.timeout(CONFORMANCE_TEST_TIMEOUT)
-    # Cannot work until we fix https://github.com/DataBiosphere/toil/issues/2801
-    @pytest.mark.xfail
     def test_run_conformance_with_caching(self):
         self.test_run_conformance(caching=True)
 
     @slow
     @pytest.mark.timeout(CONFORMANCE_TEST_TIMEOUT)
     def test_run_conformance(self, batchSystem=None, caching=False):
-        try:
-            # TODO: we do not currently pass tests: 214, 237 (offset from other versions)
-            selected_tests = '1-213,215-236,238-276'
-            cmd = [f'cwltest',
-                   f'--tool=toil-cwl-runner',
-                   f'--test={self.test_yaml}',
-                   f'--timeout=2400',
-                   f'--basedir={self.cwlSpec}',
-                   f'-n={selected_tests}']
-            if batchSystem:
-                cmd.extend(["--batchSystem", batchSystem])
+        run_conformance_tests(workDir=self.cwlSpec,
+                              yml=self.test_yaml,
+                              caching=caching,
+                              batchSystem=batchSystem)
 
-            args_passed_directly_to_toil = ['--enable-dev']
-            if caching:
-                args_passed_directly_to_toil.extend(['--disableCaching="False"'])
-            cmd.extend(['--'] + args_passed_directly_to_toil)
+    def test_usage_message(self):
+        """
+        This is purely to ensure a (more) helpful error message is printed if a user does
+        not order their positional args correctly [cwl, cwl-job (json/yml/yaml), jobstore].
+        """
+        toil = 'toil-cwl-runner'
+        cwl = 'test/cwl/revsort.cwl'
+        cwl_job_json = 'test/cwl/revsort-job.json'
+        jobstore = 'delete-test-toil'
+        random_option_1 = '--logInfo'
+        random_option_2 = '--disableCaching=false'
+        cmd_wrong_ordering_1 = [toil, cwl, cwl_job_json, jobstore, random_option_1, random_option_2]
+        cmd_wrong_ordering_2 = [toil, cwl, jobstore, random_option_1, random_option_2, cwl_job_json]
+        cmd_wrong_ordering_3 = [toil, jobstore, random_option_1, random_option_2, cwl, cwl_job_json]
 
-            subprocess.check_output(cmd, cwd=self.cwlSpec, stderr=subprocess.STDOUT)
-        except subprocess.CalledProcessError as e:
-            only_unsupported = False
-            # check output -- if we failed but only have unsupported features, we're okay
-            p = re.compile(r"(?P<failures>\d+) failures, (?P<unsupported>\d+) unsupported features")
-
-            error_log = e.output.decode('utf-8')
-            for line in error_log.split('\n'):
-                m = p.search(line)
-                if m:
-                    if int(m.group("failures")) == 0 and int(m.group("unsupported")) > 0:
-                        only_unsupported = True
-                        break
-            if not only_unsupported:
-                print(error_log)
-                raise e
+        for cmd in [cmd_wrong_ordering_1, cmd_wrong_ordering_2, cmd_wrong_ordering_3]:
+            p = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+            stdout, stderr = p.communicate()
+            self.assertIn(b'Usage: toil-cwl-runner [options] example.cwl example-job.yaml', stderr)
+            self.assertIn(b'All positional arguments [cwl, yml_or_json] '
+                          b'must always be specified last for toil-cwl-runner.', stderr)
