@@ -1,17 +1,13 @@
-# coding=utf-8
-from builtins import str
-from builtins import object
 import logging
-import sys
+import subprocess
+import time
 from contextlib import contextmanager
 
 from toil.lib.iterables import concat
-
-import subprocess
+from toil.test import ApplianceTestSupport, needs_appliance, needs_mesos, slow
 from toil.version import exactPython
-from toil.test import needs_mesos, ApplianceTestSupport, needs_appliance, slow
 
-log = logging.getLogger(__name__)
+logger = logging.getLogger(__name__)
 
 
 @needs_mesos
@@ -55,8 +51,8 @@ class AutoDeploymentTest(ApplianceTestSupport):
         """
         with self._venvApplianceCluster() as (leader, worker):
             def userScript():
-                from toil.job import Job
                 from toil.common import Toil
+                from toil.job import Job
 
                 # noinspection PyUnusedLocal
                 def job(job, disk='10M', cores=1, memory='10M'):
@@ -117,11 +113,12 @@ class AutoDeploymentTest(ApplianceTestSupport):
 
             # Deploy the user script
             def userScript():
-                from toil.job import Job
-                from toil.common import Toil
                 # noinspection PyUnresolvedReferences
                 from toil_lib.foo import libraryJob
 
+                from toil.common import Toil
+                from toil.job import Job
+
                 # noinspection PyUnusedLocal
                 def job(job, disk='10M', cores=1, memory='10M'):
                     # Double the requirements to prevent chaining as chaining might hide problems
@@ -165,8 +162,8 @@ class AutoDeploymentTest(ApplianceTestSupport):
         """
         with self._venvApplianceCluster() as (leader, worker):
             def userScript():
-                from toil.job import Job
                 from toil.common import Toil
+                from toil.job import Job
 
                 # A user-defined type, i.e. a type defined in the user script
                 class X(object):
@@ -248,12 +245,8 @@ class AutoDeploymentTest(ApplianceTestSupport):
         """
         with self._venvApplianceCluster() as (leader, worker):
             def userScript():
-                import time
-                import logging
-                from toil.job import Job
                 from toil.common import Toil
-
-                log = logging.getLogger(__name__)
+                from toil.job import Job
 
                 def root(rootJob):
                     def nullFile():
@@ -354,8 +347,9 @@ class AutoDeploymentTest(ApplianceTestSupport):
             def userScript():
                 import os
                 import time
-                from toil.job import Job
+
                 from toil.common import Toil
+                from toil.job import Job
                 from toil.leader import FailedJobsException
 
                 TIMEOUT = 10
