@@ -1,4 +1,4 @@
-# Copyright (C) 2015-2016 Regents of the University of California
+# Copyright (C) 2015-2021 Regents of the University of California
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -11,46 +11,32 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-
-from __future__ import absolute_import
-from __future__ import print_function
-
-from builtins import str
-from builtins import range
-from builtins import object
-import filecmp
-from abc import abstractmethod, ABCMeta
-from struct import pack, unpack
-from uuid import uuid4
-
-from toil.job import Job
-from toil.fileStores import FileID
-from toil.fileStores.cachingFileStore import IllegalDeletionCacheError, CacheUnbalancedError, CachingFileStore
-from toil.test import ToilTest, needs_aws_ec2, needs_google, slow, travis_test
-from toil.leader import FailedJobsException
-from toil.jobStores.abstractJobStore import NoSuchFileException
-from toil.realtimeLogger import RealtimeLogger
-
 import collections
 import datetime
+import errno
+import filecmp
 import inspect
 import logging
 import os
+import stat
 import random
 import signal
-import sys
 import time
+from abc import ABCMeta
+from struct import pack, unpack
+from uuid import uuid4
+
 import pytest
-import subprocess
 
-# Python 3 compatibility imports
-from six.moves import xrange
-from future.utils import with_metaclass
-
-if sys.version_info[0] < 3:
-    # Define a usable FileNotFoundError as will be raised by os.remove on a
-    # nonexistent file.
-    FileNotFoundError = OSError
+from toil.common import Toil
+from toil.fileStores import FileID
+from toil.fileStores.cachingFileStore import (CacheUnbalancedError,
+                                              IllegalDeletionCacheError)
+from toil.job import Job
+from toil.jobStores.abstractJobStore import NoSuchFileException
+from toil.leader import FailedJobsException
+from toil.realtimeLogger import RealtimeLogger
+from toil.test import ToilTest, needs_aws_ec2, needs_google, slow, travis_test
 
 # Some tests take too long on the AWS jobstore and are unquitable for CI.  They can be
 # be run during manual tests by setting this to False.
@@ -59,12 +45,12 @@ testingIsAutomatic = True
 logger = logging.getLogger(__name__)
 
 
-class hidden(object):
+class hidden:
     """
     Hiding the abstract test classes from the Unittest loader so it can be inherited in different
     test suites for the different job stores.
     """
-    class AbstractFileStoreTest(with_metaclass(ABCMeta, ToilTest)):
+    class AbstractFileStoreTest(ToilTest, metaclass=ABCMeta):
         """
         An abstract base class for testing the various general functions described in
         :class:toil.fileStores.abstractFileStore.AbstractFileStore
@@ -116,6 +102,53 @@ class hidden(object):
             C.addChild(D)
             Job.Runner.startToil(A, self.options)
 
+        @slow
+        def testFileStoreLogging(self):
+            """
+            Write a couple of files to the jobstore.  Delete a couple of them.  Read back written
+            and locally deleted files.
+            """
+
+            class WatchingHandler(logging.Handler):
+                """
+                A logging handler that watches for a certain substring and
+                trips a flag if it appears.
+                """
+                def __init__(self, match: str):
+                    super().__init__()
+                    self.match = match
+                    self.seen = False
+                def emit(self, record):
+                    if self.match in record.getMessage():
+                        self.seen = True
+
+            handler = WatchingHandler("cats.txt")
+
+            logging.getLogger().addHandler(handler)
+
+            F = Job.wrapJobFn(self._accessAndFail,
+                              disk='100M')
+            try:
+                Job.Runner.startToil(F, self.options)
+            except FailedJobsException:
+                # We expect this.
+                pass
+
+            logging.getLogger().removeHandler(handler)
+
+            assert handler.seen, "Downloaded file name not found in logs of failing Toil run"
+
+        @staticmethod
+        def _accessAndFail(job):
+            with job.fileStore.writeGlobalFileStream() as (stream, fileID):
+                stream.write('Cats'.encode('utf-8'))
+            localPath = os.path.join(job.fileStore.getLocalTempDir(), 'cats.txt')
+            job.fileStore.readGlobalFile(fileID, localPath)
+            with job.fileStore.readGlobalFileStream(fileID) as stream2:
+                pass
+            raise RuntimeError("I do not like this file")
+
+
         # Test filestore operations.  This is a slightly less intense version of the cache specific
         # test `testReturnFileSizes`
         @slow
@@ -143,8 +176,13 @@ class hidden(object):
             cls = hidden.AbstractNonCachingFileStoreTest
             fsId, _ = cls._writeFileToJobStore(job, isLocalFile=True, nonLocalDir=nonLocalDir,
                                                fileMB=writeFileSize)
+
+            # Fill in the size of the local file we just made
             writtenFiles[fsId] = writeFileSize
-            localFileIDs.add(list(writtenFiles.keys())[0])
+            # Remember it actually should be local
+            localFileIDs.add(fsId)
+            logger.info('Now have local file: %s', fsId)
+
             i = 0
             while i <= numIters:
                 randVal = random.random()
@@ -155,7 +193,9 @@ class hidden(object):
                                                        nonLocalDir=nonLocalDir,
                                                        fileMB=writeFileSize)
                     writtenFiles[fsID] = writeFileSize
-                    localFileIDs.add(fsID)
+                    if isLocalFile:
+                        localFileIDs.add(fsID)
+                    logger.info('Wrote %s file of size %d MB: %s', 'local' if isLocalFile else 'non-local', writeFileSize, fsID)
                 else:
                     if len(writtenFiles) == 0:
                         continue
@@ -168,18 +208,121 @@ class hidden(object):
                         job.fileStore.readGlobalFile(fsID, '/'.join([work_dir, str(uuid4())]),
                                                      cache=cache, mutable=mutable)
                         localFileIDs.add(fsID)
+                        logger.info('Read %s %s local copy of: %s', 'mutable' if mutable else 'immutable', 'cached' if cache else 'uncached', fsID)
                     else:  # Delete
                         if rdelRandVal <= 0.5:  # Local Delete
                             if fsID not in localFileIDs:
-                                continue
-                            job.fileStore.deleteLocalFile(fsID)
+                                # Make sure trying to deleteLocalFile this file fails properly
+                                try:
+                                    job.fileStore.deleteLocalFile(fsID)
+                                except OSError as e:
+                                    if e.errno != errno.ENOENT:
+                                        # This is supposed to produce an
+                                        # ENOENT. If it doesn't something is
+                                        # broken.
+                                        raise
+                                    logger.info('Correctly fail to local-delete non-local file: %s', fsID)
+                                else:
+                                    assert False, "Was able to delete non-local file {}".format(fsID)
+                            else:
+                                logger.info('Delete local file: %s', fsID)
+                                job.fileStore.deleteLocalFile(fsID)
                         else:  # Global Delete
                             job.fileStore.deleteGlobalFile(fsID)
                             writtenFiles.pop(fsID)
                         if fsID in localFileIDs:
                             localFileIDs.remove(fsID)
+                            logger.info('No longer have file: %s', fsID)
                 i += 1
 
+        def testWriteReadGlobalFilePermissions(self):
+            """
+            Ensures that uploaded files preserve their file permissions when they
+            are downloaded again. This function checks that a written executable file
+            maintains its executability after being read.
+            """
+            for executable in True,False:
+                A = Job.wrapJobFn(self._testWriteReadGlobalFilePermissions, executable=executable)
+                Job.Runner.startToil(A, self.options)
+
+        @staticmethod
+        def _testWriteReadGlobalFilePermissions(job, executable):
+            srcFile = job.fileStore.getLocalTempFile()
+            with open(srcFile, 'w') as f:
+                f.write('Hello')
+
+            if executable:
+                os.chmod(srcFile, os.stat(srcFile).st_mode | stat.S_IXUSR)
+
+            # Initial file owner execute permissions
+            initialPermissions = os.stat(srcFile).st_mode & stat.S_IXUSR
+            fileID = job.fileStore.writeGlobalFile(srcFile)
+
+            for mutable in True,False:
+                for symlink in True,False:
+                    dstFile = job.fileStore.getLocalTempFileName()
+                    job.fileStore.readGlobalFile(fileID, userPath=dstFile, mutable=mutable, symlink=symlink)
+                    # Current file owner execute permissions
+                    currentPermissions = os.stat(dstFile).st_mode & stat.S_IXUSR
+
+                    assert initialPermissions == currentPermissions
+
+        def testWriteExportFileCompatibility(self):
+            """
+            Ensures that files created in a job preserve their executable permissions
+            when they are exported from the leader.
+            """
+            for executable in True,False:
+                A = Job.wrapJobFn(self._testWriteExportFileCompatibility, executable=executable)
+                with Toil(self.options) as toil:
+                    initialPermissions, fileID = toil.start(A)
+                    dstFile = os.path.join(self._createTempDir(), str(uuid4()))
+                    toil.exportFile(fileID, 'file://' + dstFile)
+                    currentPermissions = os.stat(dstFile).st_mode & stat.S_IXUSR
+
+                    assert initialPermissions == currentPermissions
+
+        @staticmethod
+        def _testWriteExportFileCompatibility(job, executable):
+            srcFile = job.fileStore.getLocalTempFile()
+            with open(srcFile, 'w') as f:
+                f.write('Hello')
+            if executable:
+                os.chmod(srcFile, os.stat(srcFile).st_mode | stat.S_IXUSR)
+            initialPermissions = os.stat(srcFile).st_mode & stat.S_IXUSR
+            fileID = job.fileStore.writeGlobalFile(srcFile)
+            return initialPermissions, fileID
+
+        def testImportReadFileCompatibility(self):
+            """
+            Ensures that files imported to the leader preserve their executable permissions
+            when they are read by the fileStore.
+            """
+            with Toil(self.options) as toil:
+                workDir = self._createTempDir()
+                for executable in True, False:
+                    srcFile = os.path.join(workDir, str(uuid4()))
+                    with open(srcFile, 'w') as f:
+                        f.write('Hello')
+                    if executable:
+                        os.chmod(srcFile, os.stat(srcFile).st_mode | stat.S_IXUSR)
+                    initialPermissions = os.stat(srcFile).st_mode & stat.S_IXUSR
+                    jobStoreFileID = toil.importFile('file://' + srcFile)
+                    for mutable in True,False:
+                        for symlink in True, False:
+                            with self.subTest(f'Now testing readGlobalFileWith: mutable={mutable} symlink={symlink}'):
+                                dstFile = os.path.join(workDir, str(uuid4()))
+                                A = Job.wrapJobFn(_testImportReadFileCompatibility, fileID=jobStoreFileID, dstFile=dstFile,
+                                                    initialPermissions=initialPermissions, mutable=mutable, symlink=symlink)
+                                toil.start(A)
+
+        @staticmethod
+        def _testImportReadFileCompatibility(job, fileID, dstFile, initialPermissions, mutable, symlink):
+            dstFile = job.fileStore.readGlobalFile(fileID, mutable=mutable, symlink=symlink)
+            currentPermissions = os.stat(dstFile).st_mode & stat.S_IXUSR
+
+            assert initialPermissions == currentPermissions
+
         @staticmethod
         def _writeFileToJobStore(job, isLocalFile, nonLocalDir=None, fileMB=1):
             """
@@ -200,7 +343,7 @@ class hidden(object):
 
             return job.fileStore.writeGlobalFile(testFile.name), testFile
 
-    class AbstractNonCachingFileStoreTest(with_metaclass(ABCMeta, AbstractFileStoreTest)):
+    class AbstractNonCachingFileStoreTest(AbstractFileStoreTest, metaclass=ABCMeta):
         """
         Abstract tests for the the various functions in
         :class:toil.fileStores.nonCachingFileStore.NonCachingFileStore. These
@@ -212,7 +355,7 @@ class hidden(object):
             super(hidden.AbstractNonCachingFileStoreTest, self).setUp()
             self.options.disableCaching = True
 
-    class AbstractCachingFileStoreTest(with_metaclass(ABCMeta, AbstractFileStoreTest)):
+    class AbstractCachingFileStoreTest(AbstractFileStoreTest, metaclass=ABCMeta):
         """
         Abstract tests for the the various cache-related functions in
         :class:toil.fileStores.cachingFileStore.CachingFileStore.
@@ -357,7 +500,7 @@ class hidden(object):
                                     is created.
             :param int fileMB: Size of the created file in MB
             :param bool expectAsyncUpload: Whether we expect the upload to hit
-                                           the job store later(T) or immediately(F) 
+                                           the job store later(T) or immediately(F)
             """
             cls = hidden.AbstractNonCachingFileStoreTest
             fsID, testFile = cls._writeFileToJobStore(job, isLocalFile, nonLocalDir, fileMB)
@@ -385,7 +528,7 @@ class hidden(object):
             if not isLocalFile:
                 # Make sure it isn't cached if we don't want it to be
                 assert not job.fileStore.fileIsCached(fsID), "File uploaded from non-local-temp directory %s should not be cached" % nonLocalDir
-            
+
             return fsID
 
         @staticmethod
@@ -404,7 +547,7 @@ class hidden(object):
             newTotalMB, changing the maximum cache disk space allowed for the
             run.
 
-            :param int newTotalMB: New total cache disk space limit in MB. 
+            :param int newTotalMB: New total cache disk space limit in MB.
             """
 
             # Convert to bytes and pass on to the actual cache
@@ -493,7 +636,7 @@ class hidden(object):
             job.fileStore.logToMaster('Copy 2 ID: {}'.format(fsID))
 
             hidden.AbstractCachingFileStoreTest._readFromJobStoreWithoutAssertions(job, fsID)
-            
+
             job.fileStore.logToMaster('Writing copy 3 and returning ID')
             return job.fileStore.writeGlobalFile(testFile.name)
 
@@ -510,7 +653,7 @@ class hidden(object):
             job.fileStore.readGlobalFile(fsID)
 
         # writeGlobalFile tests
-        
+
         @travis_test
         def testWriteNonLocalFileToJobStore(self):
             """
@@ -521,7 +664,7 @@ class hidden(object):
             A = Job.wrapJobFn(self._writeFileToJobStoreWithAsserts, isLocalFile=False,
                               nonLocalDir=workdir)
             Job.Runner.startToil(A, self.options)
-        
+
         @travis_test
         def testWriteLocalFileToJobStore(self):
             """
@@ -532,7 +675,7 @@ class hidden(object):
             Job.Runner.startToil(A, self.options)
 
         # readGlobalFile tests
-        
+
         @travis_test
         def testReadCacheMissFileFromJobStoreWithoutCachingReadFile(self):
             """
@@ -540,7 +683,7 @@ class hidden(object):
             cache the read file.  Ensure the number of links on the file are appropriate.
             """
             self._testCacheMissFunction(cacheReadFile=False)
-        
+
         @travis_test
         def testReadCacheMissFileFromJobStoreWithCachingReadFile(self):
             """
@@ -607,7 +750,7 @@ class hidden(object):
                 return None
             else:
                 return outfile
-        
+
         @travis_test
         def testReadCachHitFileFromJobStore(self):
             """
@@ -759,7 +902,7 @@ class hidden(object):
             """
             Write a couple of files to the jobstore. Delete a couple of them.
             Read back written and locally deleted files. Ensure that after
-            every step that the cache is in a valid state. 
+            every step that the cache is in a valid state.
             """
             self.options.retryCount = 20
             self.options.badWorker = 0.5
@@ -776,10 +919,10 @@ class hidden(object):
         def _returnFileTestFn(job, jobDisk, initialCachedSize, nonLocalDir, numIters=100):
             """
             Aux function for jobCacheTest.testReturnFileSizes Conduct numIters operations and ensure
-            the cache has the right amount of data in it at all times. 
+            the cache has the right amount of data in it at all times.
 
             Track the cache calculations even thought they won't be used in filejobstore
-            
+
             Assumes nothing is evicted from the cache.
 
             :param float jobDisk: The value of disk passed to this job.
@@ -788,7 +931,8 @@ class hidden(object):
             RealtimeLogger.info('Expecting %d bytes cached initially', cached)
             work_dir = job.fileStore.getLocalTempDir()
             writtenFiles = {}  # fsID: (size, isLocal)
-            localFileIDs = collections.defaultdict(list)  # fsid: local/non-local/mutable/immutable
+            # fsid: local/mutable/immutable for all operations that should make local files as tracked by the FileStore
+            localFileIDs = collections.defaultdict(list)
             # Add one file for the sake of having something in the job store
             writeFileSize = random.randint(0, 30)
             jobDisk -= writeFileSize * 1024 * 1024
@@ -815,6 +959,7 @@ class hidden(object):
                         RealtimeLogger.info('Writing a local file of %d MB', writeFileSize)
                         fsID = cls._writeFileToJobStoreWithAsserts(job, isLocalFile=True,
                                                                    fileMB=writeFileSize)
+                        RealtimeLogger.info('Wrote local file: %s', fsID)
                         writtenFiles[fsID] = writeFileSize
                         localFileIDs[fsID].append('local')
                         jobDisk -= writeFileSize * 1024 * 1024
@@ -828,8 +973,9 @@ class hidden(object):
                         fsID = cls._writeFileToJobStoreWithAsserts(job, isLocalFile=False,
                                                                    nonLocalDir=nonLocalDir,
                                                                    fileMB=writeFileSize)
+                        RealtimeLogger.info('Wrote non-local file: %s', fsID)
                         writtenFiles[fsID] = writeFileSize
-                        localFileIDs[fsID].append('non-local')
+                        # Don't record in localFileIDs because we're not local
                         # No change to the job since there was no caching
                     RealtimeLogger.info('Checking for %d bytes cached', cached)
                     cls._requirementsConcur(job, jobDisk, cached)
@@ -841,7 +987,7 @@ class hidden(object):
                         rdelRandVal = random.random()
                         fileWasCached = job.fileStore.fileIsCached(fsID)
                     if randVal < 0.66:  # Read
-                        RealtimeLogger.info('Reading a file with size %d and previous cache status %s', rdelFileSize, str(fileWasCached))
+                        RealtimeLogger.info('Reading a file with size %d and previous cache status %s: %s', rdelFileSize, str(fileWasCached), fsID)
                         if rdelRandVal <= 0.5:  # Read as mutable, uncached
                             RealtimeLogger.info('Reading as mutable and uncached; should still have %d bytes cached', cached)
                             job.fileStore.readGlobalFile(fsID, '/'.join([work_dir, str(uuid4())]),
@@ -868,10 +1014,10 @@ class hidden(object):
                         if rdelRandVal <= 0.5:  # Local Delete
                             if fsID not in list(localFileIDs.keys()):
                                 continue
-                            RealtimeLogger.info('Deleting a file locally')
+                            RealtimeLogger.info('Deleting a file locally with history %s: %s', localFileIDs[fsID], fsID)
                             job.fileStore.deleteLocalFile(fsID)
                         else:  # Global Delete
-                            RealtimeLogger.info('Deleting a file globally')
+                            RealtimeLogger.info('Deleting a file globally: %s', fsID)
                             job.fileStore.deleteGlobalFile(fsID)
                             try:
                                 job.fileStore.readGlobalFile(fsID)
@@ -884,7 +1030,7 @@ class hidden(object):
                             writtenFiles.pop(fsID)
                         if fsID in list(localFileIDs.keys()):
                             for lFID in localFileIDs[fsID]:
-                                if lFID not in ('non-local', 'mutable'):
+                                if lFID != 'mutable':
                                     jobDisk += rdelFileSize * 1024 * 1024
                             localFileIDs.pop(fsID)
                         if fileWasCached:
@@ -915,9 +1061,9 @@ class hidden(object):
             else:
                 RealtimeLogger.info('Caching is free; %d bytes are used and %d bytes would be expected if caching were not free', used, cached)
                 assert used == 0, 'Cache should have nothing in it, but actually has %d bytes used' % used
-            
+
             jobUnused = job.fileStore.getCacheUnusedJobRequirement()
-            
+
             assert jobUnused == jobDisk, 'Job should have %d bytes of disk for non-FileStore use but the FileStore reports %d' % (jobDisk, jobUnused)
 
         # Testing the resumability of a failed worker
@@ -942,7 +1088,7 @@ class hidden(object):
             """
             This is the aux function for the controlled failed worker test.  It does a couple of
             cache operations, fails, then checks whether the new worker starts with the expected
-            value, and whether it computes cache statistics correctly. 
+            value, and whether it computes cache statistics correctly.
 
             :param float jobDisk: Disk space supplied for this job
             :param str testDir: Testing directory
@@ -1030,7 +1176,7 @@ class hidden(object):
                     outfile = testFile.name
                 else:
                     break
-        
+
         @travis_test
         def testDeleteLocalFile(self):
             """
@@ -1059,8 +1205,17 @@ class hidden(object):
             # Delete fsid of local file. The file should be deleted
             job.fileStore.deleteLocalFile(localFsID)
             assert not os.path.exists(localFile.name)
-            # Delete fsid of non-local file. The file should persist
-            job.fileStore.deleteLocalFile(nonLocalFsID)
+            # Delete fsid of non-local file. It should fail and the file should persist
+            try:
+                job.fileStore.deleteLocalFile(nonLocalFsID)
+            except OSError as e:
+                if e.errno != errno.ENOENT:
+                    # This is supposed to produce an
+                    # ENOENT. If it doesn't something is
+                    # broken.
+                    raise
+            else:
+                assert False, "Error should have been raised"
             assert os.path.exists(nonLocalFile.name)
             # Read back one file and then delete it
             readBackFile1 = job.fileStore.readGlobalFile(localFsID)
@@ -1097,7 +1252,7 @@ class hidden(object):
             """
             self.options.retryCount = 0
             self.options.disableChaining = True
-            
+
             # Make a file
             parent = Job.wrapJobFn(self._createUncachedFileStream)
             # Now make a bunch of children fight over it
@@ -1139,7 +1294,7 @@ class hidden(object):
 
             readStart = datetime.datetime.now()
             logger.debug('Begin read at %s', str(readStart))
-            
+
             localPath = job.fileStore.readGlobalFile(fileID, cache=True, mutable=True)
 
             readEnd = datetime.datetime.now()
@@ -1148,7 +1303,7 @@ class hidden(object):
             with open(localPath, 'rb') as fh:
                 text = fh.read().decode('utf-8').strip()
             logger.debug('Got file contents: %s', text)
-                
+
 
 
 class NonCachingFileStoreTestWithFileJobStore(hidden.AbstractNonCachingFileStoreTest):
