@@ -1,4 +1,4 @@
-# Copyright (C) 2015-2016 Regents of the University of California
+# Copyright (C) 2015-2021 Regents of the University of California
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -11,25 +11,18 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-from __future__ import absolute_import, print_function
-from __future__ import division
-from builtins import chr
-from builtins import map
-from builtins import str
-from builtins import range
-from past.utils import old_div
-import unittest
+import collections
 import logging
 import os
 import random
+import unittest
 
-# Python 3 compatibility imports
-from six.moves import xrange
+import pytest
 
 from toil.common import Toil
+from toil.job import Job, JobFunctionWrappingJob, JobGraphDeadlockException
 from toil.leader import FailedJobsException
-from toil.lib.bioio import getTempFile
-from toil.job import Job, JobGraphDeadlockException, JobFunctionWrappingJob
+from toil.test import get_temp_file
 from toil.test import ToilTest, slow, travis_test
 
 logger = logging.getLogger(__name__)
@@ -46,7 +39,7 @@ class JobTest(ToilTest):
 
     @slow
     def testStatic(self):
-        """
+        r"""
         Create a DAG of jobs non-dynamically and run it. DAG is:
         
         A -> F
@@ -57,7 +50,7 @@ class JobTest(ToilTest):
           
         Follow on is marked by ->
         """
-        outFile = getTempFile(rootDir=self._createTempDir())
+        outFile = get_temp_file(rootDir=self._createTempDir())
         try:
 
             # Create the jobs
@@ -91,7 +84,7 @@ class JobTest(ToilTest):
     
     @travis_test
     def testStatic2(self):
-        """
+        r"""
         Create a DAG of jobs non-dynamically and run it. DAG is:
         
         A -> F
@@ -102,7 +95,7 @@ class JobTest(ToilTest):
           
         Follow on is marked by ->
         """
-        outFile = getTempFile(rootDir=self._createTempDir())
+        outFile = get_temp_file(rootDir=self._createTempDir())
         try:
 
             # Create the jobs
@@ -146,9 +139,11 @@ class JobTest(ToilTest):
                 self.fail()
     
     @travis_test
+    @pytest.mark.timeout(30)
     def testDAGConsistency(self):
         options = Job.Runner.getDefaultOptions(self._createTempDir() + '/jobStore')
         options.clean = 'always'
+        options.logLevel = 'debug'
         i = Job.wrapJobFn(parent)
         with Toil(options) as toil:
             try:
@@ -465,7 +460,7 @@ class JobTest(ToilTest):
         referring to nodes and the edge is from a to b.
         """
         # Pick number of total edges to create
-        edgeNumber = random.choice(range(nodeNumber - 1, 1 + old_div((nodeNumber * (nodeNumber - 1)), 2)))
+        edgeNumber = random.choice(range(nodeNumber - 1, 1 + (nodeNumber * (nodeNumber - 1) // 2)))
         # Make a spanning tree of edges so that nodes are connected
         edges = set([(random.choice(range(i)), i) for i in range(1, nodeNumber)])
         # Add extra random edges until there are edgeNumber edges
@@ -576,26 +571,31 @@ class JobTest(ToilTest):
         # Make the jobs
         jobs = [makeJob(str(i)) for i in range(nodeNumber)]
         
+        # Record predecessors for sampling
+        predecessors = collections.defaultdict(list)
+        
         # Make the edges
         for fNode, tNode in childEdges:
             jobs[fNode].addChild(jobs[tNode])
+            predecessors[jobs[tNode]].append(jobs[fNode])
         for fNode, tNode in followOnEdges:
             jobs[fNode].addFollowOn(jobs[tNode])
+            predecessors[jobs[tNode]].append(jobs[fNode])
             
         # Map of jobs to return values
         jobsToRvs = dict([(job, job.addService(TrivialService(job.rv())) if addServices else job.rv()) for job in jobs])
 
         def getRandomPredecessor(job):
-            predecessor = random.choice(list(job._directPredecessors))
-            while random.random() > 0.5 and len(predecessor._directPredecessors) > 0:
-                predecessor = random.choice(list(predecessor._directPredecessors))
+            predecessor = random.choice(list(predecessors[job]))
+            while random.random() > 0.5 and len(predecessors[predecessor]) > 0:
+                predecessor = random.choice(list(predecessors[predecessor]))
             return predecessor
 
         # Connect up set of random promises compatible with graph                                          
         while random.random() > 0.01:
             job = random.choice(list(jobsToPromisesMap.keys()))
             promises = jobsToPromisesMap[job]
-            if len(job._directPredecessors) > 0:
+            if len(predecessors[job]) > 0:
                 predecessor = getRandomPredecessor(job)
                 promises.append(jobsToRvs[predecessor])
 
