@@ -1,4 +1,4 @@
-# Copyright (C) 2015-2018 Regents of the University of California
+# Copyright (C) 2015-2021 Regents of the University of California
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -11,28 +11,21 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-
 """Kills rogue toil processes."""
-from __future__ import absolute_import
 import logging
 import os
 import signal
 
-from toil.lib.bioio import getBasicOptionParser
-from toil.lib.bioio import parseBasicOptions
-from toil.common import Toil, jobStoreLocatorHelp, Config
-from toil.version import version
+from toil.common import Config, Toil, parser_with_common_options
+from toil.statsAndLogging import set_logging_from_options
+
+logger = logging.getLogger(__name__)
 
-logger = logging.getLogger( __name__ )
 
 def main():
-    parser = getBasicOptionParser()
-
-    parser.add_argument("jobStore", type=str,
-                        help="The location of the job store used by the workflow whose jobs should "
-                             "be killed." + jobStoreLocatorHelp)
-    parser.add_argument("--version", action='version', version=version)
-    options = parseBasicOptions(parser)
+    parser = parser_with_common_options()
+    options = parser.parse_args()
+    set_logging_from_options(options)
     config = Config()
     config.setOptions(options)
     config.jobStore = config.jobStore[5:] if config.jobStore.startswith('file:') else config.jobStore
@@ -41,7 +34,7 @@ def main():
     if ':' in config.jobStore:
         jobStore = Toil.resumeJobStore(config.jobStore)
         logger.info("Starting routine to kill running jobs in the toil workflow: %s", config.jobStore)
-        # TODO: This behaviour is now broken src: https://github.com/DataBiosphere/toil/commit/a3d65fc8925712221e4cda116d1825d4a1e963a1
+        # TODO: This behaviour is now broken: https://github.com/DataBiosphere/toil/commit/a3d65fc8925712221e4cda116d1825d4a1e963a1
         batchSystem = Toil.createBatchSystem(jobStore.config)  # Should automatically kill existing jobs, so we're good.
         for jobID in batchSystem.getIssuedBatchJobIDs():  # Just in case we do it again.
             batchSystem.killBatchJobs(jobID)
