@@ -1,4 +1,4 @@
-# Copyright (C) 2015 UCSC Computational Genomics Lab
+# Copyright (C) 2015-2021 Regents of the University of California
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -11,31 +11,30 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-"""
-SSHs into the toil appliance container running on the leader of the cluster
-"""
+"""SSH into the toil appliance container running on the leader of the cluster."""
 import argparse
 import logging
 import sys
-from toil.provisioners import clusterFactory
-from toil.lib.bioio import parseBasicOptions, getBasicOptionParser
-from toil.utils import addBasicProvisionerOptions
+
+from toil.common import parser_with_common_options
+from toil.provisioners import cluster_factory
+from toil.statsAndLogging import set_logging_from_options
 
 logger = logging.getLogger(__name__)
 
 
 def main():
-    parser = getBasicOptionParser()
-    parser = addBasicProvisionerOptions(parser)
+    parser = parser_with_common_options(provisioner_options=True, jobstore_option=False)
     parser.add_argument("--insecure", action='store_true',
                         help="Temporarily disable strict host key checking.")
     parser.add_argument("--sshOption", dest='sshOptions', default=[], action='append',
                         help="Pass an additional option to the SSH command.")
     parser.add_argument('args', nargs=argparse.REMAINDER)
-    config = parseBasicOptions(parser)
-    cluster = clusterFactory(provisioner=config.provisioner,
-                             clusterName=config.clusterName,
-                             zone=config.zone)
-    command = config.args if config.args else ['bash']
-    cluster.getLeader().sshAppliance(*command, strict=not config.insecure, tty=sys.stdin.isatty(),
-                                     sshOptions=config.sshOptions)
+    options = parser.parse_args()
+    set_logging_from_options(options)
+    cluster = cluster_factory(provisioner=options.provisioner,
+                              clusterName=options.clusterName,
+                              zone=options.zone)
+    command = options.args if options.args else ['bash']
+    cluster.getLeader().sshAppliance(*command, strict=not options.insecure, tty=sys.stdin.isatty(),
+                                     sshOptions=options.sshOptions)
