@@ -1,4 +1,4 @@
-# Copyright (C) 2018 UCSC Computational Genomics Lab
+# Copyright (C) 2015-2021 Regents of the University of California
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -11,18 +11,18 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-from __future__ import absolute_import
-from __future__ import print_function
-from __future__ import division
-from six import iteritems
-
-import os
 import logging
+import os
 
-import toil.wdl.wdl_parser as wdl_parser
 from toil.wdl.wdl_functions import heredoc_wdl
+from toil.wdl.wdl_types import (WDLArrayType,
+                                WDLCompoundType,
+                                WDLFileType,
+                                WDLMapType,
+                                WDLPairType,
+                                WDLType)
 
-wdllogger = logging.getLogger(__name__)
+logger = logging.getLogger(__name__)
 
 
 class SynthesizeWDL:
@@ -52,6 +52,13 @@ class SynthesizeWDL:
                  jobstore=None,
                  destBucket=None):
         self.output_directory = output_directory
+        if not os.path.exists(self.output_directory):
+            try:
+                os.makedirs(self.output_directory)
+            except:
+                raise OSError(
+                    'Could not create directory.  Insufficient permissions or disk space most likely.')
+
         self.output_file = os.path.join(self.output_directory, 'toilwdl_compiled.py')
 
         self.jobstore = jobstore if jobstore else './toilWorkflowRun'
@@ -81,7 +88,19 @@ class SynthesizeWDL:
                     from toil.job import Job
                     from toil.common import Toil
                     from toil.lib.docker import apiDockerCall
+                    from toil.wdl.wdl_types import WDLType
+                    from toil.wdl.wdl_types import WDLStringType
+                    from toil.wdl.wdl_types import WDLIntType
+                    from toil.wdl.wdl_types import WDLFloatType
+                    from toil.wdl.wdl_types import WDLBooleanType
+                    from toil.wdl.wdl_types import WDLFileType
+                    from toil.wdl.wdl_types import WDLArrayType
+                    from toil.wdl.wdl_types import WDLPairType
+                    from toil.wdl.wdl_types import WDLMapType
+                    from toil.wdl.wdl_types import WDLFile
+                    from toil.wdl.wdl_types import WDLPair
                     from toil.wdl.wdl_functions import generate_docker_bashscript_file
+                    from toil.wdl.wdl_functions import generate_stdout_file
                     from toil.wdl.wdl_functions import select_first
                     from toil.wdl.wdl_functions import sub
                     from toil.wdl.wdl_functions import size
@@ -94,13 +113,28 @@ class SynthesizeWDL:
                     from toil.wdl.wdl_functions import parse_memory
                     from toil.wdl.wdl_functions import parse_cores
                     from toil.wdl.wdl_functions import parse_disk
-                    from toil.wdl.wdl_functions import read_string
-                    from toil.wdl.wdl_functions import read_int
-                    from toil.wdl.wdl_functions import read_float
+                    from toil.wdl.wdl_functions import read_lines
                     from toil.wdl.wdl_functions import read_tsv
                     from toil.wdl.wdl_functions import read_csv
+                    from toil.wdl.wdl_functions import read_json
+                    from toil.wdl.wdl_functions import read_map
+                    from toil.wdl.wdl_functions import read_int
+                    from toil.wdl.wdl_functions import read_string
+                    from toil.wdl.wdl_functions import read_float
+                    from toil.wdl.wdl_functions import read_boolean
+                    from toil.wdl.wdl_functions import write_lines
+                    from toil.wdl.wdl_functions import write_tsv
+                    from toil.wdl.wdl_functions import write_json
+                    from toil.wdl.wdl_functions import write_map
                     from toil.wdl.wdl_functions import defined
                     from toil.wdl.wdl_functions import basename
+                    from toil.wdl.wdl_functions import floor
+                    from toil.wdl.wdl_functions import ceil
+                    from toil.wdl.wdl_functions import wdl_range
+                    from toil.wdl.wdl_functions import transpose
+                    from toil.wdl.wdl_functions import length
+                    from toil.wdl.wdl_functions import wdl_zip
+                    from toil.wdl.wdl_functions import cross
                     import fnmatch
                     import textwrap
                     import subprocess
@@ -112,7 +146,7 @@ class SynthesizeWDL:
                     import uuid
                     import logging
                     
-                    asldijoiu23r8u34q89fho934t8u34fcurrentworkingdir = os.getcwd()
+                    _toil_wdl_internal__current_working_dir = os.getcwd()
 
                     logger = logging.getLogger(__name__)
 
@@ -174,28 +208,24 @@ class SynthesizeWDL:
 
     def write_main_wfdeclarations(self):
         main_section = ''
-        for wfname, wf in iteritems(self.workflows_dictionary):
+        for wfname, wf in self.workflows_dictionary.items():
             if 'wf_declarations' in wf:
-                for var, var_expressn in iteritems(wf['wf_declarations']):
+                for var, var_expressn in wf['wf_declarations'].items():
+                    var_type = var_expressn['type']
 
                     # check the json file for the expression's value
                     # this is a higher priority and overrides anything written in the .wdl
                     json_expressn = self.json_var(wf=wfname, var=var)
-                    if json_expressn:
+                    if json_expressn is not None:
                         var_expressn['value'] = json_expressn
 
-                    # empty string
-                    if not var_expressn['value'] and (var_expressn['type'] in ['String', 'File']):
-                        main_section += '        {} = ""\n'.format(var)
-                    # None
-                    elif not var_expressn['value'] and not (var_expressn['type'] in ['String', 'File']):
-                        main_section += '        {} = None\n'.format(var)
+                    main_section += '        {} = {}.create(\n                {})\n'\
+                        .format(var, self.write_declaration_type(var_type), var_expressn['value'])
+
                     # import filepath into jobstore
-                    elif var_expressn['value'] and (var_expressn['type'] == 'File'):
-                        main_section += '        {} = process_infile({}, fileStore)\n'.format(var, var_expressn['value'])
-                    # normal declaration
-                    else:
-                        main_section += '        {} = {}\n'.format(var, var_expressn['value'])
+                    if self.needs_file_import(var_type) and var_expressn['value']:
+                        main_section += '        {} = process_infile({}, fileStore)\n'.format(var, var)
+
         return main_section
 
     def write_main_jobwrappers(self):
@@ -320,7 +350,7 @@ class SynthesizeWDL:
 
         scatternamespace = []
 
-        for wfname, wf in iteritems(self.workflows_dictionary):
+        for wfname, wf in self.workflows_dictionary.items():
             if 'wf_declarations' in wf:
                 for var in wf['wf_declarations']:
                     scatternamespace.append(var)
@@ -363,8 +393,6 @@ class SynthesizeWDL:
                     return inputs_list, False
         return inputs_list, True
 
-
-
     def write_main_jobwrappers_call(self, task):
         main_section = '        {} = job0.addChild({}Cls('.format(task['alias'], task['task'])
         for var in task['io']:
@@ -420,7 +448,6 @@ class SynthesizeWDL:
                 fn_section += self.write_scatterfunctions_within_if(ifstatement[assignment]['body'])
         return fn_section
 
-
     def write_scatterfunction(self, job, scattername):
         '''
         Writes out a python function for each WDL "scatter" object.
@@ -442,7 +469,6 @@ class SynthesizeWDL:
 
         return fn_section
 
-
     def write_scatterfunction_header(self, scattername):
         """
 
@@ -502,7 +528,6 @@ class SynthesizeWDL:
 
         return fn_section
 
-
     def write_scatterfunction_loop(self, job, scatter_outputs):
         """
 
@@ -645,17 +670,22 @@ class SynthesizeWDL:
         if 'inputs' in self.tasks_dictionary[job]:
             for i in self.tasks_dictionary[job]['inputs']:
                 var = i[0]
+                var_type = i[1]
                 var_expressn = i[2]
                 json_expressn = self.json_var(task=job, var=var)
 
                 # json declarations have priority and can overwrite
                 # whatever is in the wdl file
-                if json_expressn:
+                if json_expressn is not None:
                     var_expressn = json_expressn
-                if not var_expressn:
-                    var_expressn = var
 
-                fn_section += '        self.id_{} = {}\n'.format(var, var_expressn)
+                if var_expressn is None:
+                    # declarations from workflow
+                    fn_section += '        self.id_{} = {}\n'.format(var, var)
+                else:
+                    # declarations from a WDL or JSON file
+                    fn_section += '        self.id_{} = {}.create(\n                {})\n'\
+                        .format(var, self.write_declaration_type(var_type), var_expressn)
 
         fn_section += heredoc_wdl('''
 
@@ -663,6 +693,9 @@ class SynthesizeWDL:
                                  fileStore.logToMaster("{jobname}")
                                  tempDir = fileStore.getLocalTempDir()
                                  
+                                 _toil_wdl_internal__stdout_file = os.path.join(tempDir, 'stdout')
+                                 _toil_wdl_internal__stderr_file = os.path.join(tempDir, 'stderr')
+                                 
                                  try:
                                      os.makedirs(os.path.join(tempDir, 'execution'))
                                  except OSError as e:
@@ -673,9 +706,18 @@ class SynthesizeWDL:
             for i in self.tasks_dictionary[job]['inputs']:
                 var = i[0]
                 var_type = i[1]
+
                 docker_bool = str(self.needsdocker(job))
-                if var_type == 'File':
-                    fn_section += '        {} = process_and_read_file(abspath_file(self.id_{}, asldijoiu23r8u34q89fho934t8u34fcurrentworkingdir), tempDir, fileStore, docker={})\n'.format(var, var, docker_bool)
+
+                if self.needs_file_import(var_type):
+                    args = ', '.join(
+                        [
+                            f'abspath_file(self.id_{var}, _toil_wdl_internal__current_working_dir)',
+                            'tempDir',
+                            'fileStore',
+                            f'docker={docker_bool}'
+                        ])
+                    fn_section += '        {} = process_and_read_file({})\n'.format(var, args)
                 else:
                     fn_section += '        {} = self.id_{}\n'.format(var, var)
 
@@ -706,6 +748,50 @@ class SynthesizeWDL:
 
         return None
 
+    def needs_file_import(self, var_type: WDLType) -> bool:
+        """
+        Check if the given type contains a File type. A return value of True
+        means that the value with this type has files to import.
+        """
+        if isinstance(var_type, WDLFileType):
+            return True
+
+        if isinstance(var_type, WDLCompoundType):
+            if isinstance(var_type, WDLArrayType):
+                return self.needs_file_import(var_type.element)
+            elif isinstance(var_type, WDLPairType):
+                return self.needs_file_import(var_type.left) or self.needs_file_import(var_type.right)
+            elif isinstance(var_type, WDLMapType):
+                return self.needs_file_import(var_type.key) or self.needs_file_import(var_type.value)
+            else:
+                raise NotImplementedError
+        return False
+
+    def write_declaration_type(self, var_type: WDLType):
+        """
+        Return a string that preserves the construction of the given WDL type
+        so it can be passed into the compiled script.
+        """
+        section = var_type.__class__.__name__ + '('  # e.g.: 'WDLIntType('
+
+        if isinstance(var_type, WDLCompoundType):
+            if isinstance(var_type, WDLArrayType):
+                section += self.write_declaration_type(var_type.element)
+            elif isinstance(var_type, WDLPairType):
+                section += self.write_declaration_type(var_type.left) + ', '
+                section += self.write_declaration_type(var_type.right)
+            elif isinstance(var_type, WDLMapType):
+                section += self.write_declaration_type(var_type.key) + ', '
+                section += self.write_declaration_type(var_type.value)
+            else:
+                raise ValueError(var_type)
+
+        if var_type.optional:
+            if isinstance(var_type, WDLCompoundType):
+                section += ', '
+            section += 'optional=True'
+        return section + ')'
+
     def write_function_bashscriptline(self, job):
         '''
         Writes a function to create a bashscript for injection into the docker
@@ -739,18 +825,23 @@ class SynthesizeWDL:
                        "job_task_reference": job,
                        "docker_user": str(self.docker_user)}
         docker_template = heredoc_wdl('''
-        stdout = apiDockerCall(self, 
-                               image={docker_image}, 
-                               working_dir=tempDir, 
-                               parameters=[os.path.join(tempDir, "{job_task_reference}_script.sh")], 
-                               entrypoint="/bin/bash", 
-                               user={docker_user}, 
-                               stderr=True, 
-                               volumes={{tempDir: {{"bind": tempDir}}}})
-        writetype = 'wb' if isinstance(stdout, bytes) else 'w'
-        with open(os.path.join(asldijoiu23r8u34q89fho934t8u34fcurrentworkingdir, '{job_task_reference}.log'), writetype) as f:
-            f.write(stdout)
-            ''', docker_dict, indent='        ')[1:]
+        # apiDockerCall() with demux=True returns a tuple of bytes objects (stdout, stderr).
+        _toil_wdl_internal__stdout, _toil_wdl_internal__stderr = \\
+            apiDockerCall(self, 
+                          image={docker_image}, 
+                          working_dir=tempDir, 
+                          parameters=[os.path.join(tempDir, "{job_task_reference}_script.sh")], 
+                          entrypoint="/bin/bash", 
+                          user={docker_user}, 
+                          stderr=True, 
+                          demux=True, 
+                          volumes={{tempDir: {{"bind": tempDir}}}})
+        with open(os.path.join(_toil_wdl_internal__current_working_dir, '{job_task_reference}.log'), 'wb') as f:
+            if _toil_wdl_internal__stdout:
+                f.write(_toil_wdl_internal__stdout)
+            if _toil_wdl_internal__stderr:
+                f.write(_toil_wdl_internal__stderr)
+        ''', docker_dict, indent='        ')[1:]
 
         return docker_template
 
@@ -770,7 +861,7 @@ class SynthesizeWDL:
         if 'raw_commandline' in self.tasks_dictionary[job]:
             for cmd in self.tasks_dictionary[job]['raw_commandline']:
                 if not cmd.startswith("r'''"):
-                    cmd = 'str({i} if not isinstance({i}, tuple) else process_and_read_file({i}, tempDir, fileStore)).strip("{nl}")'.format(i=cmd, nl=r"\n")
+                    cmd = 'str({i} if not isinstance({i}, WDLFile) else process_and_read_file({i}, tempDir, fileStore)).strip("{nl}")'.format(i=cmd, nl=r"\n")
                 fn_section = fn_section + heredoc_wdl('''
                         try:
                             # Intended to deal with "optional" inputs that may not exist
@@ -802,7 +893,7 @@ class SynthesizeWDL:
         '''
         fn_section = heredoc_wdl('''
                 this_process = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
-                stdout, stderr = this_process.communicate()\n''', indent='        ')
+                _toil_wdl_internal__stdout, _toil_wdl_internal__stderr = this_process.communicate()\n''', indent='        ')
 
         return fn_section
 
@@ -818,6 +909,17 @@ class SynthesizeWDL:
         '''
 
         fn_section = ''
+
+        fn_section += heredoc_wdl('''
+            _toil_wdl_internal__stdout_file = generate_stdout_file(_toil_wdl_internal__stdout, 
+                                                                   tempDir, 
+                                                                   fileStore=fileStore)
+            _toil_wdl_internal__stderr_file = generate_stdout_file(_toil_wdl_internal__stderr, 
+                                                                   tempDir, 
+                                                                   fileStore=fileStore, 
+                                                                   stderr=True)
+        ''', indent='        ')[1:]
+
         if 'outputs' in self.tasks_dictionary[job]:
             return_values = []
             for output in self.tasks_dictionary[job]['outputs']:
@@ -825,17 +927,17 @@ class SynthesizeWDL:
                 output_type = output[1]
                 output_value = output[2]
 
-                if output_type == 'File':
+                if self.needs_file_import(output_type):
                     nonglob_dict = {
                         "output_name": output_name,
+                        "output_type": self.write_declaration_type(output_type),
                         "expression": output_value,
-                        "out_dir": self.output_directory,
-                        "output_type": output_type}
+                        "out_dir": self.output_directory}
 
                     nonglob_template = heredoc_wdl('''
-                        # output-type: {output_type}
-                        output_filename = {expression}
-                        {output_name} = process_outfile(output_filename, fileStore, tempDir, '{out_dir}')
+                        {output_name} = {output_type}.create(
+                            {expression}, output=True)
+                        {output_name} = process_outfile({output_name}, fileStore, tempDir, '{out_dir}')
                     ''', nonglob_dict, indent='        ')[1:]
                     fn_section += nonglob_template
                     return_values.append(output_name)
@@ -972,15 +1074,3 @@ class SynthesizeWDL:
             f.write(pretty(i.tasks_dictionary))
             f.write('\n\n\n\n\n\n')
             f.write(pretty(i.workflows_dictionary))
-
-def write_AST(wdl_file, outdir=None):
-    '''
-    Writes a file with the AST for a wdl file in the outdir.
-    '''
-    if outdir is None:
-        outdir = os.getcwd()
-    with open(os.path.join(outdir, 'AST.out'), 'w') as f:
-        with open(wdl_file, 'r') as wdl:
-            wdl_string = wdl.read()
-            ast = wdl_parser.parse(wdl_string).ast()
-            f.write(ast.dumps(indent=2))
